package com.lea.dao;

import java.io.File;
import java.lang.ref.Reference;
import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.ImageIcon;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;

import com.lea.controler.Observable;
import com.lea.controler.Observateur;
import com.lea.domain.Magazine;
import com.lea.exception.DataAccessException;
import com.lea.exception.FinderException;



public abstract class AbstractMagazineDAO implements DataAccessConstants,
		Observable {

	protected static HashMap<String, Magazine> mapMagazine = new HashMap<String, Magazine>();
	protected static HashMap<String, Magazine> mapAmmo = new HashMap<String, Magazine>();
	protected static HashMap<String, Magazine> mapExplosive = new HashMap<String, Magazine>();
	protected static HashMap<String, Magazine> mapEquip = new HashMap<String, Magazine>();
	protected static HashMap<String, Magazine> mapCrewserved = new HashMap<String, Magazine>();
	private static Set<String> availableMods = new TreeSet<String>();
	private static ArrayList<Document> documentMagazines = null;

	private ArrayList<Observateur> listObservateur = new ArrayList<Observateur>();
	private int count;

	public int readMagazinesClasslist(String installationPath) {

		documentMagazines = new ArrayList<Document>();
		SAXReader reader = new SAXReader();

		File directory = new File(installationPath + "/" + MAGAZINES_CLASSLIST_FILES);
		File[] subfiles = directory.listFiles();
		int numberMagazines = 0;
		String classlistXmlPath= "";
		try {
			for (int i = 0; i < subfiles.length; i++) {
				classlistXmlPath = subfiles[i].getAbsolutePath();
				Document documentMagazine = reader.read(classlistXmlPath);
				Element root = documentMagazine.getRootElement();
				numberMagazines = root.elements().size();
				documentMagazines.add(documentMagazine);
			}
		} catch (DocumentException e) {
			e.printStackTrace();
			throw new DataAccessException(
					"file classlist " + classlistXmlPath + " not found: "
							+ e.getMessage(), e);
		}
		return numberMagazines;
	}

	public void createDBMagazines() {

		for (int l = 0; l < documentMagazines.size(); l++) {

			Element root = documentMagazines.get(l).getRootElement();
			for (Iterator i = root.elementIterator(); i.hasNext();) {
				Element element = (Element) i.next();
				try {
					Magazine magazine = new Magazine();
					String classname = "";
					String name = "";
					String caliber = "";
					double aceWeight = 0;
					boolean ace = false;
					List<String> listFaction = new ArrayList<String>();
					ImageIcon imageIcon = null;
					String mod = "";
					String type = "";
					String category = "";
					String slot = "";

					classname = element.selectSingleNode("name").getText();
					name = element.selectSingleNode("displayname").getText();

					ace = Boolean.parseBoolean(element.selectSingleNode("ace")
							.getText());

					caliber = element.selectSingleNode("caliber").getText();

					String number = element.selectSingleNode("ace_weight")
							.getText();
					Pattern pattern = Pattern.compile(",");
					Matcher matcher = pattern.matcher(number);
					if (matcher.find()) {
						StringTokenizer sb = new StringTokenizer(number, ",");
						String integer = sb.nextToken();
						String decimal = sb.nextToken();
						number = integer + "." + decimal;
					}
					Element factionsElement = (Element) element
							.selectSingleNode("factions");
					treeWalk(factionsElement, listFaction);

					aceWeight = Double.parseDouble(number);

					mod = element.selectSingleNode("mod").getText();

					type = element.selectSingleNode("type").getText();

					category = element.selectSingleNode("category").getText();

					slot = element.selectSingleNode("slot").getText();

					magazine.setName(name);
					magazine.setClassname(classname);
					magazine.setAceWeight(aceWeight);
					magazine.setCaliber(caliber);
					magazine.setListFaction(listFaction);
					magazine.setAce(ace);
					magazine.setMod(mod);
					magazine.setType(type);
					magazine.setCategory(category);
					magazine.setSlot(Integer.parseInt(slot));
					magazine.checkData();

					mapMagazine.put(classname, magazine);
					this.updateObservateur();
					count++;
					
					if (!mod.equals("")){
						availableMods.add(mod.toUpperCase());
					}

					if (type.equals("MagAmmo"))
						mapAmmo.put(classname, magazine);
					if (type.equals("MagEquip"))
						mapEquip.put(classname, magazine);
					if (type.equals("MagExplosive"))
						mapExplosive.put(classname, magazine);
					if (type.equals("MagCrewserved"))
						mapCrewserved.put(classname, magazine);

				} catch (Exception e) {
					e.printStackTrace();
					throw new DataAccessException(
							"error in parsing " + l
									+ e.getMessage(), e);
				}
			}
		}
		documentMagazines=null;
	}

	private static void treeWalk(Element element, List<String> list) {

		for (int i = 0, size = element.nodeCount(); i < size; i++) {
			Node node = element.node(i);
			if (node instanceof Element) {
				if (node.getName().equals("name")) {
					list.add(node.getText());
				}
				treeWalk((Element) node, list);
			}
		}
	}
	
	public Set<String> getAvailableMods() {
		return availableMods;
	}
	
	public Magazine findByClassname(String magazineClassname) {
		return getMap().get(magazineClassname);
	}

	public WeakReference<ImageIcon> findImageIcon(String installPath,String classnameMagazine) {

		String stg = installPath + "/"+MAGAZINE_PICTURES + "/" + classnameMagazine + ".png";

		ImageIcon imageIcon = new ImageIcon(stg);
		WeakReference<ImageIcon> ref = new WeakReference<ImageIcon>(imageIcon);
		return ref;

	}
	
	public Reference<ImageIcon> FindMagazinesImageIconInIconesFolder(
			String installPath,String classnameMagazine) throws FinderException {
		
		String stg = installPath + "/" + MAGAZINE_ICONES + "/" + classnameMagazine + ".png";

		ImageIcon imageIcon = new ImageIcon(stg);
		
		if (imageIcon==null)
			throw new FinderException("Can't find icone for magazine " + classnameMagazine);
		
		WeakReference<ImageIcon> ref = new WeakReference<ImageIcon>(imageIcon);
		return ref;
	}

	public Collection findByFaction(String factionName) {
		Collection<Magazine> magazines = new ArrayList<Magazine>();
		Set<Map.Entry<String, Magazine>> entrees = getMap().entrySet();
		Iterator<Map.Entry<String, Magazine>> iter = entrees.iterator();
		while (iter.hasNext()) {
			Map.Entry<String, Magazine> entree = (Map.Entry) iter.next();
			Magazine magazine = ((Magazine) entree.getValue());
			if (magazine.getListFaction().contains(factionName)) {
				magazines.add(magazine);
			}
		}
		return magazines;
	}

	public abstract HashMap<String, Magazine> getMap();

	public Collection findAll() {
		return getMap().values();
	}

	/**
	 * Ajoute un observateur � la liste
	 */
	public void addObservateur(Observateur obs) {
		this.listObservateur.add(obs);
	}

	/**
	 * Retire tous les observateurs de la liste
	 */
	public void delObservateur() {
		this.listObservateur = new ArrayList<Observateur>();
	}

	/**
	 * Avertit les observateurs que l'observable a chang� et invoque la
	 * m�thode update de chaque observateur !
	 */
	public void updateObservateur() {
		for (Observateur obs : this.listObservateur)
			obs.update(this.count);
	}
}
