package com.lea.dao;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.Node;
import org.dom4j.io.SAXReader;

import com.lea.controler.Observable;
import com.lea.controler.Observateur;
import com.lea.domain.Unit;
import com.lea.exception.DataAccessException;



public abstract class AbstractUnitDAO implements DataAccessConstants,
		Observable {

	private static ArrayList<Document> documentUnits = null;

	protected static HashMap<String, Unit> mapUnit = new HashMap<String, Unit>();
	protected ArrayList<Observateur> listObservateur = new ArrayList<Observateur>();
	protected int count = 0;

	public int readUnitsClasslist(String installationPath) {

		documentUnits = new ArrayList<Document>();
		SAXReader reader = new SAXReader();
		File directory = new File(installationPath + "/"
				+ UNITS_CLASSLIST_FILES);

		System.out.println(installationPath);
		File[] subfiles = directory.listFiles();
		int numberUnits = 0;
		try {
			for (int i = 0; i < subfiles.length; i++) {
				String classlistXmlPath = subfiles[i].getAbsolutePath();
				Document documentUnit = reader.read(classlistXmlPath);
				Element root = documentUnit.getRootElement();
				numberUnits = root.elements().size();
				documentUnits.add(documentUnit);
			}
		} catch (DocumentException e) {
			e.printStackTrace();
			System.out.println(installationPath);
			throw new DataAccessException("file classlist unit xml not found: "
					+ e.getMessage(), e);
		}
		return numberUnits;
	}

	public void createDBUnits() {

		for (int l = 0; l < documentUnits.size(); l++) {

			Element root = documentUnits.get(l).getRootElement();

			for (Iterator i = root.elementIterator(); i.hasNext();) {
				Element element = (Element) i.next();

				try {
					Unit unit = new Unit();
					String classname = "";
					String name = "";
					boolean ace = false;
					boolean vehicle = false;
					String faction = "";
					String mod = "";
					int numberPrimarySlot = 12;
					int numberSecondarySlot = 8;
					
					List<String> listMagazine = new ArrayList<String>();
					List<String> listWeapon = new ArrayList<String>();

					classname = element.selectSingleNode("name").getText();
					name = element.selectSingleNode("displayname").getText();
					ace = Boolean.parseBoolean(element.selectSingleNode("ace")
							.getText());
					mod = element.selectSingleNode("mod").getText();
					vehicle = Boolean.parseBoolean(element.selectSingleNode(
							"vehicle").getText());

					faction = element.selectSingleNode("faction").getText();
					numberPrimarySlot = Integer.parseInt(element.selectSingleNode("numberprimaryslot").getText());
					numberSecondarySlot = Integer.parseInt(element.selectSingleNode("numbersecondaryslot").getText());
					
					
					Element magazines = (Element) element
							.selectSingleNode("magazines");
					treeWalk(magazines, listMagazine);
					Element weapons = (Element) element
							.selectSingleNode("weapons");
					treeWalk(weapons, listWeapon);
					
					unit.setClassname(classname);
					unit.setName(name);
					unit.setAce(ace);
					unit.setListMagazineClassname(listMagazine);
					unit.setListWeaponClassname(listWeapon);
					unit.setFaction(faction);
					unit.setMod(mod);
					// System.out.println(classname);
					unit.setVehicle(vehicle);
					// System.out.println(vehicle);
					unit.setNumberPrimarySlot(numberPrimarySlot);
					unit.setNumberSecondarySlot(numberSecondarySlot);
					unit.checkData();

					mapUnit.put(classname, unit);
					this.updateObservateur();
					count++;

				} catch (Exception e) {
					// e.printStackTrace();
					throw new DataAccessException(
							"error in parsing classlist_units.xml : "
									+ e.getMessage(), e);
				}
			}

		}
		documentUnits = null;
	}

	public abstract HashMap<String, Unit> getMap();

	public Unit findByClassname(String unitClassname) {
		return getMap().get(unitClassname);
	}

	private static void treeWalk(Element element, List<String> list) {

		for (int i = 0, size = element.nodeCount(); i < size; i++) {
			Node node = element.node(i);
			if (node instanceof Element) {
				// System.out.println(node.getName());
				if (node.getName().equals("name")) {
					// System.out.println(node.getName());
					// System.out.println(node.getText());
					list.add(node.getText());
				}
				treeWalk((Element) node, list);
			}
		}
	}

	/* Observer/Observable pattern */
	
	public void addObservateur(Observateur obs) {
		this.listObservateur.add(obs);
	}

	public void delObservateur() {
		this.listObservateur = new ArrayList<Observateur>();
	}

	public void updateObservateur() {

		for (Observateur obs : this.listObservateur) {
			obs.update(this.count);
		}
	}
}
