package com.lea.dao;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Vector;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SealedObject;

import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.dom4j.io.SAXReader;

import com.lea.domain.Ftp;


public class FtpDAO extends AbstractFtpDAO {

	private FileInputStream fis;
	private UploadCountingIntputStream uis;
	private FileOutputStream fos;
	private DownloadCountingOutputStream dos;

	@Override
	public void writeAutoConfig(Cipher cipher, Ftp ftp, String path)
			throws IllegalBlockSizeException, IOException {
		SealedObject sealedObject = new SealedObject(ftp, cipher);
		String filePath = path + "/" + "auto-config" + ".zip";
		ObjectOutputStream fWo = new ObjectOutputStream(new GZIPOutputStream(
				new FileOutputStream(filePath)));
		if (sealedObject != null)
			fWo.writeObject(sealedObject);
		fWo.close();
	}

	@Override
	public Ftp readAutoConfig(Cipher cipher, String path)
			throws FileNotFoundException, IOException, ClassNotFoundException,
			NoSuchAlgorithmException, NoSuchPaddingException,
			InvalidKeyException, IllegalBlockSizeException, BadPaddingException {

		ObjectInputStream fRo = new ObjectInputStream(new GZIPInputStream(
				new FileInputStream(path)));
		SealedObject sealedObject = (SealedObject) fRo.readObject();
		Ftp ftp = (Ftp) sealedObject.getObject(cipher);
		fRo.close();
		return ftp;
	}

	public void writeFtpHistory(Cipher cipher, Ftp ftp, String intallationPath)
			throws IllegalBlockSizeException, IOException {

		String identifier = ftp.getLogin() + "@" + ftp.getUrl() + "-"
				+ ftp.getPort();
		SealedObject sealedObject = new SealedObject(ftp, cipher);
		String filePath = intallationPath + "/" + FTP_FOLDER + "/" + identifier
				+ ".zip";
		ObjectOutputStream fWo = new ObjectOutputStream(new GZIPOutputStream(
				new FileOutputStream(filePath)));
		if (sealedObject != null)
			fWo.writeObject(sealedObject);
		fWo.close();
	}

	public Collection<Ftp> readFtpHistory(Cipher cipher, String intallationPath) {

		String path = intallationPath + "/" + FTP_FOLDER;
		File directory = new File(path);
		File[] subfiles = directory.listFiles();
		Collection<Ftp> ftps = new ArrayList<Ftp>();

		for (int i = 0; i < subfiles.length; i++) {
			try {
				String ftpPath = subfiles[i].getAbsolutePath();
				ObjectInputStream fRo = new ObjectInputStream(
						new GZIPInputStream(new FileInputStream(ftpPath)));
				SealedObject sealedObject = (SealedObject) fRo.readObject();
				Ftp ftp = (Ftp) sealedObject.getObject(cipher);
				ftps.add(ftp);
				fRo.close();
			} catch (Exception e) {
				// throw new LoadingException("Can't load ftp! " +
				// e.getMessage());
				e.printStackTrace();
			}
		}
		return ftps;
	}

	@Override
	public boolean removeFtpHistory(Ftp ftp, String intallationPath) {

		boolean response = false;
		String path = intallationPath + "/" + FTP_FOLDER;
		File directory = new File(path);
		File[] subfiles = directory.listFiles();
		String identifier = ftp.getLogin() + "@" + ftp.getUrl() + "-"
				+ ftp.getPort();
		for (int i = 0; i < subfiles.length; i++) {
			String ftpPath = subfiles[i].getAbsolutePath();
			if (ftpPath.contains(identifier)) {
				subfiles[i].delete();
				response = true;
			}
		}
		return response;
	}

	@Override
	public long getFtpFilesSize(Vector selectedFiles, FTPClient ftpClient)
			throws IOException {
		long size = 0;
		for (int i = 0; i < selectedFiles.size(); i++) {
			FTPFile[] ftpFiles = ftpClient.listFiles(((FTPFile) selectedFiles
					.elementAt(i)).getName());
			if (ftpFiles.length != 0) {
				size = size + ftpFiles[0].getSize();
			}
		}
		return size;
	}

	@Override
	public void setDownload(String fileName, String localActualPath,
			FTPClient ftpClient) throws FileNotFoundException {
		File file = new File(localActualPath + "/" + fileName);
		fos = new FileOutputStream(file);
		dos = new DownloadCountingOutputStream(fos);
	}

	@Override
	public void download(String fileName, FTPClient ftpClient)
			throws IOException {
		ftpClient.retrieveFile(fileName, dos);
		ftpClient.noop();
		try {
			if (dos != null) {
				dos.close();
			}
			if (fos != null) {
				fos.close();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public void setUpload(File file, FTPClient ftpClient)
			throws FileNotFoundException {
		fis = new FileInputStream(file);
		uis = new UploadCountingIntputStream(fis);
	}

	@Override
	public void upload(String fileName, FTPClient ftpClient) throws IOException {
		ftpClient.storeFile(fileName, uis);
		ftpClient.noop();
		try {
			if (fis != null) {
				fis.close();
			}
			if (uis != null) {
				uis.close();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@Override
	public FtpDAO getFtpDAO() {
		return this;
	}

	@Override
	public String downloadXMLupdateFile(String installPath, FTPClient ftpClient)
			throws IOException, DocumentException {
		ftpClient.changeWorkingDirectory(UPDATE_REPOSITORY);
		File file = new File(installPath + "/" + "lea.xml");
		FileOutputStream fos = new FileOutputStream(file);
		boolean found = ftpClient.retrieveFile("lea.xml", fos);
		String nom = null;
		fos.close();
		if (found){
			SAXReader reader = new SAXReader();
			Document documentLeaVersion = reader.read(file);
			Element root = documentLeaVersion.getRootElement();
			nom = root.selectSingleNode("nom").getText();
		}
		return nom;
	}

	public DownloadCountingOutputStream getDos() {
		return this.dos;
	}

	public UploadCountingIntputStream getUis() {
		return this.uis;
	}

}
