package com.lea.dao;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;

import com.lea.domain.Magazine;
import com.lea.domain.Mission;
import com.lea.domain.Unit;
import com.lea.domain.Weapon;
import com.lea.exception.WritingException;



public class LoadoutSoloDAO extends AbstractScriptDAO {

	private boolean noACE;

	@Override
	public void writeScript(Object object, String path,boolean noACE)
			throws WritingException, IOException {
		
		this.noACE = noACE;

		File file = new File(path + "/lea");
		if (!file.exists()){
			file.mkdir();
		}
		
		Mission mission = (Mission) object;
		
		String missionFolderPath = mission.getMissionFolderPath();
		String relativePath =  path.subSequence(missionFolderPath.length(), path.length()).toString();

		if (relativePath.length()!=0){
			relativePath =	relativePath.substring(1) + "\\";
		}
		
		PrintWriter fWo = new PrintWriter(new FileWriter(path
				+ "/lea/loadout-solo.sqf"));

		fWo.println("/**");
		fWo.println("* Generated by LEA for Arma 2 Combined Operation & ACE 2");
		fWo.println("* " + new Date());
		fWo.println("* Mission file name: " + mission.getMissionFileName());
		fWo.println("* Mission name: " + mission.getMissionFileName());
		fWo.println("* Required mods:");
		for (String mod : mission.getListMods()) {
			if (mod != null) {
				if (!mod.equals(""))
					fWo.println("* + " + mod.trim());
			}
		}
		fWo.println("* ");
		fWo.println("* Call the script: _dummy = [] execvm "
				+ "\"" + relativePath + "lea\\loadout-solo.sqf" + "\"" + ";");
		fWo.println("* ");
		fWo.println("*/");

		Collection<Unit> units = null;
		try {
			units = mission.getMapUnit().values();
		} catch (Exception e) {
			throw new WritingException(
					"Can't generate loadout.sqf.There is no unit.");
		}
		Collection<Unit> vehicleUnits = new ArrayList<Unit>();
		Collection<Unit> aiUnits = new ArrayList<Unit>();
		Collection<Unit> playableUnits = new ArrayList<Unit>();

		for (Unit unit : units) {
			if (unit.isEditable()&&!unit.getEditorName().equals("not assigned")) {
				if (unit.getType().equals("EMPTY")) {
					vehicleUnits.add(unit);
				}
				if (unit.getType().equals("AI") && !unit.isVehicle()) {
					aiUnits.add(unit);
				}
				if (unit.getType().equals("AI") && unit.isVehicle()) {
					vehicleUnits.add(unit);
				}
				if (unit.getType().equals("PLAYABLE") && !unit.isVehicle()) {
					playableUnits.add(unit);
				}
				if (unit.getType().equals("PLAYABLE") && unit.isVehicle()) {
					vehicleUnits.add(unit);
				}
			}
		}

		if (vehicleUnits.size() == 0) {
			fWo.println("");
			fWo.println("/**");
			fWo.println("* Loadout for vehicles");
			fWo.println("*/");
			fWo.println("");
			fWo.println("//No named vehicles in this mission.");
		} else {
			fWo.println("");
			fWo.println("/**");
			fWo.println("* Loadout for Vehicles");
			fWo.println("*/");
			fWo.println("");
			writeVehicleUnits(vehicleUnits, fWo);
		}

		if (aiUnits.size() == 0) {
			fWo.println("");
			fWo.println("/**");
			fWo.println("* Loadout for ai units");
			fWo.println("*/");
			fWo.println("");
			fWo.println("//No named AI units in this mission.");
		} else {
			fWo.println("");
			fWo.println("/**");
			fWo.println("* Loadout for ai units");
			fWo.println("*/");
			fWo.println("");
			writeAIUnits(aiUnits, fWo);
		}

		if (playableUnits.size() == 0) {
			fWo.println("");
			fWo.println("/**");
			fWo.println("* Loadout for playable units");
			fWo.println("*/");
			fWo.println("");
			fWo.println("//No named playable units in this mission.");
		} else {
			fWo.println("");
			fWo.println("/**");
			fWo.println("* Loadout for playable units");
			fWo.println("*/");
			fWo.println("");
			writePlayableUnits(playableUnits, fWo);
		}
		fWo.close();
	}

	private void writeVehicleUnits(Collection<Unit> emptyVehicleUnits,
			PrintWriter fWo) {

		for (Unit unit : emptyVehicleUnits) {

			fWo.println("");
			fWo.println(" clearweaponcargo " + unit.getEditorName() + ";");
			fWo.println(" clearmagazinecargo " + unit.getEditorName() + ";");

			if (unit.getRuckWeaponLoadout().size() != 0) {
				for (int i = 0; i < unit.getRuckWeaponLoadout().size(); i++) {
					fWo.println(" " + unit.getEditorName()
							+ " addweaponcargo [" + "\""
							+ unit.getRuckWeaponLoadout().get(i).getClassname()
							+ "\"" + ","
							+ unit.getRuckWeaponLoadout().get(i).getQuantity()
							+ "];");
				}
			}
			if (unit.getRuckMagazineLoadout().size() != 0) {
				for (int i = 0; i < unit.getRuckMagazineLoadout().size(); i++) {
					fWo.println(" "
							+ unit.getEditorName()
							+ " addmagazinecargo ["
							+ "\""
							+ unit.getRuckMagazineLoadout().get(i)
									.getClassname()
							+ "\""
							+ ","
							+ unit.getRuckMagazineLoadout().get(i)
									.getQuantity() + "];");
				}
			}
			fWo.println("");
		}
	}
	
	private void writeAIUnits(Collection<Unit> aiUnits, PrintWriter fWo) {
		
		for (Unit unit : aiUnits) {

			unit.getListWeaponClassname().clear();
			if (!unit.getPrimaryWeaponLoadout().getClassname().equals(""))
				unit.getListWeaponClassname().add(
						unit.getPrimaryWeaponLoadout().getClassname());
			if (!unit.getSecondaryWeaponLoadout().getClassname().equals(""))
				unit.getListWeaponClassname().add(
						unit.getSecondaryWeaponLoadout().getClassname());
			if (!unit.getPistolLoadout().getClassname().equals(""))
				unit.getListWeaponClassname().add(
						unit.getPistolLoadout().getClassname());

			for (Weapon weapon : unit.getOpticalLoadout())
				unit.getListWeaponClassname().add(weapon.getClassname());
			for (Weapon weapon : unit.getItemLoadout())
				unit.getListWeaponClassname().add(weapon.getClassname());

			unit.getListMagazineClassname().clear();
			for (Magazine magazine : unit.getPrimaryMagazineLoadout())
				unit.getListMagazineClassname().add(magazine.getClassname());
			for (Magazine magazine : unit.getSecondaryMagazineLoadout())
				unit.getListMagazineClassname().add(magazine.getClassname());

			fWo.println("");
			fWo.println(" removeallweapons " + unit.getEditorName() + ";");
			fWo.println(" removeallitems " + unit.getEditorName() + ";");
			fWo.println(" removeBackpack " + unit.getEditorName() + ";");
			fWo.println("");

			fWo.println(" " + "_" + unit.getEditorName() + "aiweap = [");
			if (unit.getListWeaponClassname().size() != 0) {
				for (int i = 0; i < unit.getListWeaponClassname().size() - 1; i++) {
					String stg = ("\t " + "\""
							+ unit.getListWeaponClassname().get(i) + "\"");
					fWo.println(stg + ",");
				}
				fWo.println("\t "
						+ "\""
						+ unit.getListWeaponClassname().get(
								unit.getListWeaponClassname().size() - 1)
						+ "\"");
			}
			fWo.println(" ];");
			fWo.println("");
			fWo.println(" " + "_" + unit.getEditorName() + "aiammo = [");
			if (unit.getListMagazineClassname().size() != 0) {
				for (int i = 0; i < unit.getListMagazineClassname().size() - 1; i++) {
					String stg = ("\t " + "\""
							+ unit.getListMagazineClassname().get(i) + "\"");
					fWo.println(stg + ",");
				}
				fWo.println("\t "
						+ "\""
						+ unit.getListMagazineClassname().get(
								unit.getListMagazineClassname().size() - 1)
						+ "\"");
			}

			fWo.println(" ];");
			fWo.println("");
			fWo.println(" " + "_" + unit.getEditorName() + "aiweapback = "
					+ "\"" + unit.getBackWeaponLoadout().getClassname() + "\""
					+ ";");
			fWo.println("");

			fWo.print(" " + "_" + unit.getEditorName() + "airuck = [");

			if (unit.getRuckWeaponLoadout().size() != 0) {

				for (int i = 0; i < unit.getRuckWeaponLoadout().size() - 1; i++) {
					String stg = (" [" + "\""
							+ unit.getRuckWeaponLoadout().get(i).getClassname()
							+ "\"" + ","
							+ unit.getRuckWeaponLoadout().get(i).getQuantity() + "]");
					fWo.print(stg + ",");
				}
				fWo.print(" ["
						+ "\""
						+ unit.getRuckWeaponLoadout().get(
								unit.getRuckWeaponLoadout().size() - 1)
								.getClassname()
						+ "\""
						+ ","
						+ unit.getRuckWeaponLoadout().get(
								unit.getRuckWeaponLoadout().size() - 1)
								.getQuantity() + "]");
			}

			if ((unit.getRuckWeaponLoadout().size() != 0)
					&& (unit.getRuckMagazineLoadout().size() != 0)) {
				fWo.print(",");
			}

			if (unit.getRuckMagazineLoadout().size() != 0) {
				for (int i = 0; i < unit.getRuckMagazineLoadout().size() - 1; i++) {
					String stg = (" ["
							+ "\""
							+ unit.getRuckMagazineLoadout().get(i)
									.getClassname()
							+ "\""
							+ ","
							+ unit.getRuckMagazineLoadout().get(i)
									.getQuantity() + "]");
					fWo.print(stg + ",");
				}
				fWo.print(" ["
						+ "\""
						+ unit.getRuckMagazineLoadout().get(
								unit.getRuckMagazineLoadout().size() - 1)
								.getClassname()
						+ "\""
						+ ","
						+ unit.getRuckMagazineLoadout().get(
								unit.getRuckMagazineLoadout().size() - 1)
								.getQuantity() + "]");

			}

			fWo.print(" ];");

			fWo.println("");
			if ((unit.getRuckWeaponLoadout().size() != 0
					|| unit.getRuckMagazineLoadout().size() != 0)&&!noACE) {
				fWo.println(" " + unit.getEditorName() + " setvariable ["
						+ "\"" + "ACE_RuckMagContents" + "\"" + ", " + "_"
						+ unit.getEditorName() + "airuck];");
			}
			
			if (!noACE) {
				fWo.println(" " + unit.getEditorName() + " setvariable [" + "\""
						+ "ACE_weapononback" + "\"" + ", " + "_"
						+ unit.getEditorName() + "aiweapback];");
			}
			fWo.println(" {" + unit.getEditorName()
					+ " addmagazine _x} foreach " + "_" + unit.getEditorName()
					+ "aiammo;");
			fWo.println(" {" + unit.getEditorName() + " addWeapon _x} foreach "
					+ "_" + unit.getEditorName() + "aiweap;");
			fWo.println("");
		}
	}

	private void writePlayableUnits(Collection<Unit> units, PrintWriter fWo) {
		
		for (Unit unit : units) {

			// Sac  dos est en secondaryWeapon si il n'y a pas de
			// secondaryWeapon
			if (unit.getBackWeaponLoadout().getType() != null) {
				if (unit.getBackWeaponLoadout().getType().equals("Ruck")) {
					if (unit.getSecondaryWeaponLoadout().getClassname().equals(
							"")) {
						Weapon tempWeapon = unit.getSecondaryWeaponLoadout();
						unit.setSecondaryWeaponLoadout(unit
								.getBackWeaponLoadout());
						unit.setBackWeaponLoadout(tempWeapon);
					}
				}
			}

			unit.getListWeaponClassname().clear();
			if (!unit.getPrimaryWeaponLoadout().getClassname().equals(""))
				unit.getListWeaponClassname().add(
						unit.getPrimaryWeaponLoadout().getClassname());
			if (!unit.getSecondaryWeaponLoadout().getClassname().equals(""))
				unit.getListWeaponClassname().add(
						unit.getSecondaryWeaponLoadout().getClassname());
			if (!unit.getPistolLoadout().getClassname().equals(""))
				unit.getListWeaponClassname().add(
						unit.getPistolLoadout().getClassname());

			for (Weapon weapon : unit.getOpticalLoadout())
				unit.getListWeaponClassname().add(weapon.getClassname());
			for (Weapon weapon : unit.getItemLoadout())
				unit.getListWeaponClassname().add(weapon.getClassname());

			unit.getListMagazineClassname().clear();
			for (Magazine magazine : unit.getPrimaryMagazineLoadout())
				unit.getListMagazineClassname().add(magazine.getClassname());
			for (Magazine magazine : unit.getSecondaryMagazineLoadout())
				unit.getListMagazineClassname().add(magazine.getClassname());
			for (Magazine magazine : unit.getFirstAidMagazineLoadout()) {
				for (int i = 0; i < magazine.getQuantity(); i++)
					unit.getListMagazineClassname()
							.add(magazine.getClassname());
			}

			fWo.println("");
			fWo.println("  removeallweapons " + unit.getEditorName() + ";");
			fWo.println("  removeallitems " + unit.getEditorName() + ";");
			fWo.println("  removeBackpack " + unit.getEditorName() + ";");
			fWo.println("");

			fWo.println("  " + "_" + unit.getEditorName() + "playerweap = [");
			if (unit.getListWeaponClassname().size() != 0) {
				for (int i = 0; i < unit.getListWeaponClassname().size() - 1; i++) {
					String stg = ("\t " + "\""
							+ unit.getListWeaponClassname().get(i) + "\"");
					fWo.println(stg + ",");
				}
				fWo.println("\t  "
						+ "\""
						+ unit.getListWeaponClassname().get(
								unit.getListWeaponClassname().size() - 1)
						+ "\"");
			}
			fWo.println("\t " + "];");
			fWo.println("");
			fWo.println("  " + "_" + unit.getEditorName() + "playerammo = [");
			if (unit.getListMagazineClassname().size() != 0) {
				for (int i = 0; i < unit.getListMagazineClassname().size() - 1; i++) {
					String stg = ("\t " + "\""
							+ unit.getListMagazineClassname().get(i) + "\"");
					fWo.println(stg + ",");
				}
				fWo.println("\t  "
						+ "\""
						+ unit.getListMagazineClassname().get(
								unit.getListMagazineClassname().size() - 1)
						+ "\"");
			}
			fWo.println("\t " + "];");
			fWo.println("");
			fWo.println("  " + "_" + unit.getEditorName() + "playerweapback = "
					+ "\"" + unit.getBackWeaponLoadout().getClassname() + "\""
					+ ";");
			fWo.println("");

			fWo.print("  " + "_" + unit.getEditorName() + "playeruck = [");

			if (unit.getRuckWeaponLoadout().size() != 0) {

				for (int i = 0; i < unit.getRuckWeaponLoadout().size() - 1; i++) {
					String stg = (" [" + "\""
							+ unit.getRuckWeaponLoadout().get(i).getClassname()
							+ "\"" + ","
							+ unit.getRuckWeaponLoadout().get(i).getQuantity() + "]");
					fWo.print(stg + ",");
				}
				fWo.print("  ["
						+ "\""
						+ unit.getRuckWeaponLoadout().get(
								unit.getRuckWeaponLoadout().size() - 1)
								.getClassname()
						+ "\""
						+ ","
						+ unit.getRuckWeaponLoadout().get(
								unit.getRuckWeaponLoadout().size() - 1)
								.getQuantity() + "]");
			}

			if ((unit.getRuckWeaponLoadout().size() != 0)
					&& (unit.getRuckMagazineLoadout().size() != 0)) {
				fWo.print(",");
			}

			if (unit.getRuckMagazineLoadout().size() != 0) {
				for (int i = 0; i < unit.getRuckMagazineLoadout().size() - 1; i++) {
					String stg = ("  ["
							+ "\""
							+ unit.getRuckMagazineLoadout().get(i)
									.getClassname()
							+ "\""
							+ ","
							+ unit.getRuckMagazineLoadout().get(i)
									.getQuantity() + "]");
					fWo.print(stg + ",");
				}
				fWo.print("  ["
						+ "\""
						+ unit.getRuckMagazineLoadout().get(
								unit.getRuckMagazineLoadout().size() - 1)
								.getClassname()
						+ "\""
						+ ","
						+ unit.getRuckMagazineLoadout().get(
								unit.getRuckMagazineLoadout().size() - 1)
								.getQuantity() + "]");
			}
			fWo.print("];");
			fWo.println("");
			if ((unit.getRuckWeaponLoadout().size() != 0
					|| unit.getRuckMagazineLoadout().size() != 0)&&!noACE) {
				fWo.println("   " + unit.getEditorName() + " setvariable ["
						+ "\"" + "ACE_RuckMagContents" + "\"" + ", " + "_"
						+ unit.getEditorName() + "playeruck];");
			}
			
			if (!noACE) {
				fWo.println("   " + unit.getEditorName() + " setvariable [" + "\""
						+ "ACE_weapononback" + "\"" + ", " + "_"
						+ unit.getEditorName() + "playerweapback];");
			}
			fWo.println("  {" + unit.getEditorName()
					+ " addmagazine _x} foreach " + "_" + unit.getEditorName()
					+ "playerammo;");
			fWo.println("  {" + unit.getEditorName()
					+ " addWeapon _x} foreach " + "_" + unit.getEditorName()
					+ "playerweap;");
			fWo.println("");
		}
	}
}
