package com.lea.dao;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.nio.channels.FileChannel;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import com.lea.domain.Group;
import com.lea.domain.Mission;
import com.lea.domain.Unit;

public class MissionDAO extends AbstractMissionDAO {

	/** The number of playable units in this mission **/
	private int numberPlayers;

	/** The path to "lea" folder in a mission folder if any **/
	private String pathToLeaFolder;

	/** True if it is a new extracted unit from the mission.sqm file **/
	private boolean isNewUnit;

	@Override
	public Mission load(String missionFilePath) throws FileNotFoundException,
			IOException, ClassNotFoundException {

		ObjectInputStream fRo = new ObjectInputStream(new GZIPInputStream(
				new FileInputStream(missionFilePath)));
		Mission mission = (Mission) fRo.readObject();
		fRo.close();
		return mission;
	}

	@Override
	public void save(Mission mission, String missionsFolderPath)
			throws FileNotFoundException, IOException {

		String missionPath = missionsFolderPath + "/"
				+ mission.getMissionFileName() + ".lea.mission";

		ObjectOutputStream fWo = new ObjectOutputStream(new GZIPOutputStream(
				new FileOutputStream(missionPath)));

		if (mission != null)
			fWo.writeObject(mission);
		fWo.close();
	}

	@Override
	public void copyMission(String sourceLocationPath,
			String destinationFolderPath) throws IOException {

		File sourceLocation = new File(sourceLocationPath);
		File destinationLocation = new File(destinationFolderPath);
		copyFile(sourceLocation, destinationLocation);
	}

	@Override
	public void deleteMissionFile(Mission mission, String missionsFolderPath) {

		String missionPath = missionsFolderPath + "/"
				+ mission.getMissionFileName() + ".lea.mission";

		File file = new File(missionPath);
		file.delete();

	}

	@Override
	public Mission parse(String missionSQFfilePath)
			throws FileNotFoundException, IOException, ClassNotFoundException {

		DataInputStream fRo = new DataInputStream(new FileInputStream(
				missionSQFfilePath));
		BufferedReader d = new BufferedReader(new InputStreamReader(fRo));

		String ligne = "";
		List<String> listRecordingVehicles = new ArrayList<String>();
		Pattern patternBriefingName = Pattern.compile("briefingName");
		Pattern patternBriefingDescription = Pattern
				.compile("briefingDescription");

		boolean recordingVehicles = false;

		Mission mission = new Mission();
		File file = new File(missionSQFfilePath);
		mission.setMissionPath(missionSQFfilePath);
		mission.setMissionFileName(file.getParentFile().getName());
		mission.setMissionFolderPath(file.getParentFile().getAbsolutePath());

		int ligneIndice = 0;
		int ligneStartRecording = 0;
		String briefingName = null;
		String briefingDescription = null;

		/*
		 * Extract lignes corresponding to "vehicles" in mission.sqm as a list
		 * of String (listRecordingVehicles) Put ligne number in the mission.sqm
		 * file into listLigneIndices
		 */
		while (true) {

			ligne = d.readLine();

			if (ligne == null)
				break;

			if (ligne.contains("class Intro")
					|| (ligne.contains("class Sensors"))
					|| (ligne.contains("class Markers"))) {
				recordingVehicles = false;
				// Adding buffer values to list to prevent from exception
				for (int i = 0; i < 5; i++)
					listRecordingVehicles.add("");
				break;
			}

			Matcher matcherBriefingName = patternBriefingName.matcher(ligne);
			Matcher matcherBriefingDescription = patternBriefingDescription
					.matcher(ligne);

			if (matcherBriefingName.find()) {
				briefingName = (String) ligne.trim().subSequence(14,
						(ligne.trim().length() - 2));
			}
			if (matcherBriefingDescription.find()) {
				briefingDescription = (String) ligne.trim().subSequence(21,
						(ligne.trim().length() - 2));
				mission.setDescription(briefingDescription);
			}
			if (ligne.contains("class Vehicles")) {
				recordingVehicles = true;
				ligneStartRecording = ligneIndice;
			}
			if (recordingVehicles)
				listRecordingVehicles.add(ligne.trim());

			ligneIndice++;
		}

		fRo.close();
		/* End of extraction */

		if (briefingName != null)
			mission.setName(briefingName);
		else
			mission.setName("/");

		if (briefingDescription != null)
			mission.setDescription(briefingDescription);
		else
			mission.setDescription("/");

		/* Parsing contain of listRecordingVehicles and creation of groups */
		List<Group> listGroups = new ArrayList<Group>();
		mission.setListGroup(listGroups);
		int index = -1;
		Group g = null;
		for (String s : listRecordingVehicles) {
			index++;
			if (s.contains("class Item0")) {
				String numberVehiclePattern = (String) listRecordingVehicles
						.get(index - 1)
						.subSequence(
								6,
								listRecordingVehicles.get(index - 1).length() - 1);
				int numberVehicles = Integer.parseInt(numberVehiclePattern);
				g = new Group(numberVehicles);
				g.setGroupIndex(index);
				g.addVehicleIndex(index);
				g.addVehicleLigneIndex(index + ligneStartRecording);
				listGroups.add(g);
			}
			if ((s.contains("class Item"))
					&& (!s.contains("class Item0"))
					&& (g.getListIndexVehicles().size() < g.getNumberVehicles())) {
				g.addVehicleIndex(index);
				g.addVehicleLigneIndex(index + ligneStartRecording);
			}
		}

		/* Building units */
		int Id = 0;// Id of unit. This id is unique.
		numberPlayers = 0;

		for (int i = 0; i < listGroups.size(); i++) {
			Group group = listGroups.get(i);
			for (int j = 0; j < group.getNumberVehicles(); j++) {
				index = group.getVehicleIndex(j);
				Unit unit = new Unit();
				unit.setGroupNumber(i);
				unit.setId(Id);

				if (index < (group
						.getVehicleIndex(group.getNumberVehicles() - 1))) {
					int indexStart = group.getVehicleIndex(j);
					int indexEnd = group.getVehicleIndex(j + 1);
					isNewUnit = true;
					for (int k = indexStart; k <= indexEnd; k++) {
						String stg = listRecordingVehicles.get(k);
						setUnit(stg, unit);
					}

				}

				else if (index == (group.getVehicleIndex(group
						.getNumberVehicles() - 1))) {
					int indexStart = group.getVehicleIndex(j);
					int indexEnd = group.getVehicleIndex(j) + 13;
					isNewUnit = true;
					for (int k = indexStart; k <= indexEnd; k++) {
						String stg = listRecordingVehicles.get(k);
						setUnit(stg, unit);
					}
				}

				boolean ok = evaluateUnit(unit);
				if (ok) {
					mission.getMapUnit().put(Id, unit);
				}
				Id++;
			}
		}
		mission.setNumberPlayers(numberPlayers);
		numberPlayers = 0;
		return mission;
	}

	private boolean evaluateUnit(Unit unit) {

		boolean res = true;

		if (unit.getSide() == null || unit.getClassname() == null
				|| unit.getType() == null || unit.getEditorName() == null)
			res = false;

		if (res == true) {

			if (unit.getSide().equals("") || unit.getClassname().equals("")
					|| unit.getType().equals(""))
				res = false;

			else if (unit.getSide().equals("LOGIC"))
				res = false;
		}
		return res;
	}

	private void setUnit(String stg, Unit unit) {

		// beginning of a next unit => exit
		if (stg.contains("side=") && !isNewUnit) {
			return;
		}

		if (stg.contains("side=")) {
			String side = (String) stg.subSequence(6, stg.length() - 2);
			unit.setSide(side);
			if (side.equals("EMPTY")) {
				unit.setType("EMPTY");
			}
			isNewUnit = false;
		} else if (stg.contains("vehicle=")) {
			String classname = (String) stg.subSequence(9, stg.length() - 2);
			unit.setClassname(classname);
		} else if (stg.contains("player=")) {
			unit.setType("PLAYABLE");
			numberPlayers++;
		} else if (stg.contains("leader=")) {
			unit.setLeader(true);
		} else if (stg.contains("text=")) {
			String editorName = (String) stg.subSequence(6, stg.length() - 2);
			unit.setEditorName(editorName);
		} else if (stg.contains("description=")) {
			String editorDescription = (String) stg.subSequence(13, stg
					.length() - 2);
			if (editorDescription.equals("")) {
				unit.setEditorDescription("Empty");
			} else {
				unit.setEditorDescription(editorDescription);
			}
		}
	}

	@Override
	public void extractPBO(File sourceFile, File destinationFile)
			throws IOException, InterruptedException {

		File sourceLocation = sourceFile;
		File targetLocation = new File(PBO_FOLDER + "/" + sourceFile.getName());
		copyFile(sourceLocation, targetLocation);
		String command = "resources\\bin\\cpbo.exe -y -e " + PBO_FOLDER_WIN
				+ "\\" + sourceFile.getName();
		Process process = Runtime.getRuntime().exec(command);

		count = 0;
		boolean wait = true;
		for (int i = 0; i < 20; i++) {
			System.out.println(i);
			Thread.sleep(250);
			count++;
			this.updateObservateur();
			try {
				System.out.println("return value = " + process.exitValue());
				wait = false;
				break;
			} catch (IllegalThreadStateException e) {
				System.out.println("exception in state cpbo.exe");
			}
		}

		if (wait) {
			process.destroy();
		}

		this.updateObservateur();

		String missionFolderName = targetLocation.getName().replaceAll(".pbo","");
		File sourceLocation2 = new File(PBO_FOLDER + "/" + missionFolderName);

		File targetLocation2 = new File(destinationFile.getAbsoluteFile() + "/"
				+ missionFolderName);
		copyDirectory(sourceLocation2, targetLocation2);
		deleteFile(targetLocation);
		deleteDirectory(sourceLocation2);

	}

	@Override
	public void makePBO(String installPath, String missionFolderPath,
			String missionFileName) throws IOException, InterruptedException {

		File sourceLocation = new File(missionFolderPath);
		File targetLocation = new File(PBO_FOLDER + "/" + missionFileName);
		copyDirectory(sourceLocation, targetLocation);
		String command = "resources\\bin\\cpbo.exe -y -p " + PBO_FOLDER_WIN
				+ "\\" + missionFileName;
		Process process = Runtime.getRuntime().exec(command);

		count = 0;
		boolean wait = true;
		for (int i = 0; i < 20; i++) {
			System.out.println(i);
			Thread.sleep(250);
			count++;
			this.updateObservateur();
			try {
				System.out.println("return value = " + process.exitValue());
				wait = false;
				break;
			} catch (IllegalThreadStateException e) {
				System.out.println("exception in state cpbo.exe");
			}
		}

		if (wait) {
			process.destroy();
		}

		this.updateObservateur();

		File sourceLocation2 = new File(PBO_FOLDER + "/" + missionFileName
				+ ".pbo");

		File targetLocation2 = new File(PBOS_FOLDER + "/" + missionFileName
				+ ".pbo");

		copyFile(sourceLocation2, targetLocation2);
		deleteDirectory(targetLocation);
		deleteFile(sourceLocation2);
	}

	@Override
	public void backupMissionSQM(String missionSQMPath) throws IOException {
		File sourceLocation = new File(missionSQMPath);
		File targetLocation = new File(missionSQMPath + ".bck");
		File targetLocation0 = new File(missionSQMPath + ".bck0");
		if (!targetLocation0.exists()) {
			copyFile(sourceLocation, targetLocation0);
			copyFile(sourceLocation, targetLocation);
		} else {
			copyFile(sourceLocation, targetLocation);
		}
	}

	@Override
	public void deleteMissionSQM(String missionSQMPath) {
		File sourceLocation = new File(missionSQMPath);
		deleteFile(sourceLocation);
	}

	@Override
	public void rollBackMissionSQM(String missionSQMPath) throws IOException {
		File sourceLocation = new File(missionSQMPath + ".bck");
		File targetLocation = new File(missionSQMPath);
		copyFile(sourceLocation, targetLocation);
	}

	@Override
	public void generateMissionSQM(Mission mission, String missionFolderPath)
			throws IOException {

		DataInputStream fRo = new DataInputStream(new FileInputStream(
				missionFolderPath + "/" + "mission.sqm.bck"));
		BufferedReader reader = new BufferedReader(new InputStreamReader(fRo));

		DataOutputStream fWo = new DataOutputStream(new FileOutputStream(
				missionFolderPath + "/" + "mission.sqm"));
		BufferedWriter writter = new BufferedWriter(new OutputStreamWriter(fWo));

		String ligne = "";

		Set<Map.Entry<Integer, Unit>> entrees = mission.getMapUnit().entrySet();
		Iterator<Map.Entry<Integer, Unit>> iter = entrees.iterator();
		boolean isLogic = false;

		while (true) {
			ligne = reader.readLine();
			if (ligne == null)
				break;

			writter.write(ligne);
			writter.newLine();

			if (ligne.contains("LOGIC")) {
				isLogic = true;
			}

			if (ligne.contains("vehicle=") && !isLogic) {

				String buffer = ligne.substring(0, ligne.indexOf("vehicle="));

				Map.Entry<Integer, Unit> entree = (Map.Entry) iter.next();
				System.out.println("Ligne = " + ligne.trim());
				Unit unit = entree.getValue();
				String classnameName = unit.getClassname();
				System.out.println("Unit Classname = " + classnameName);

				String editorName = unit.getEditorName();
				if (editorName.equals("not assigned")) {
					editorName = "";
				}
				String editorNameLigne = buffer + "text=" + "\"" + editorName
						+ "\"" + ";";

				String editorDescription = unit.getEditorDescription();
				if (editorDescription.equals("Empty")) {
					editorDescription = "";
				}
				String editorDescriptionLigne = buffer + "description=" + "\""
						+ editorDescription + "\"" + ";";

				List<String> list = new ArrayList<String>();
				while (!ligne.contains("class Item")
						&& !ligne.contains("class Vehicles")
						&& !ligne.contains("class Intro")
						&& !ligne.contains("class Sensors")
						&& !ligne.contains("class Markers")) {
					ligne = reader.readLine();
					list.add(ligne);
				}

				int testIndex = -1;
				int descriptionIndex = -1;
				int numberOfOpenParentheses = 0;
				int numberOfCloseParentheses = 0;

				for (int i = 0; i < list.size(); i++) {
					if (list.get(i).contains("text=")) {
						testIndex = i;
					} else if (list.get(i).contains("description=")) {
						descriptionIndex = i;
					} else if (list.get(i).contains("{")) {
						numberOfOpenParentheses++;
					} else if (list.get(i).contains("};")) {
						numberOfCloseParentheses++;
					}
				}

				if (testIndex != -1) {
					list.remove(testIndex);
					list.add(testIndex, editorNameLigne);
				} 
				else if (testIndex == -1 && !editorName.equals("")) {
					list.add(list.size()- 1 - (numberOfCloseParentheses - numberOfOpenParentheses),editorNameLigne);
				}

				if (descriptionIndex != -1) {
					list.remove(descriptionIndex);
					list.add(descriptionIndex, editorDescriptionLigne);
				} 
				else if (descriptionIndex == -1&& !editorDescription.equals("")) {
					list
							.add(
									list.size()
											- 1
											- (numberOfCloseParentheses - numberOfOpenParentheses),
									editorDescriptionLigne);
				}

				for (int i = 0; i < list.size(); i++) {
					writter.write(list.get(i));
					writter.newLine();
				}
			} else if (ligne.contains("skill=") && isLogic) {
				isLogic = false;
			}
		}
		writter.flush();
		reader.close();
		writter.close();
		fRo.close();
		fWo.close();
	}

	// If targetLocation does not exist, it will be created.
	private void copyDirectory(File sourceLocation, File targetLocation)
			throws IOException {

		if (sourceLocation.isDirectory()) {
			if (!targetLocation.exists()) {
				targetLocation.mkdir();
			}

			String[] children = sourceLocation.list();
			for (int i = 0; i < children.length; i++) {
				copyDirectory(new File(sourceLocation, children[i]), new File(
						targetLocation, children[i]));
			}
		} else {
			InputStream in = new FileInputStream(sourceLocation);
			OutputStream out = new FileOutputStream(targetLocation);
			// Copy the bits from instream to outstream
			byte[] buf = new byte[1024];
			int len;
			while ((len = in.read(buf)) > 0) {
				out.write(buf, 0, len);
			}
			in.close();
			out.close();
		}
	}

	private void copyFile(File sourceLocation, File targetLocation)
			throws IOException {

		FileChannel source = null;
		FileChannel destination = null;
		try {
			source = new FileInputStream(sourceLocation).getChannel();
			destination = new FileOutputStream(targetLocation).getChannel();
			destination.transferFrom(source, 0, source.size());
		} finally {
			if (source != null) {
				source.close();
			}
			if (destination != null) {
				destination.close();
			}
		}
	}

	private boolean deleteDirectory(File file) {
		if (file.exists()) {
			File[] files = file.listFiles();
			for (int i = 0; i < files.length; i++) {
				if (files[i].isDirectory()) {
					deleteDirectory(files[i]);
				} else {
					files[i].delete();
				}
			}
		}
		return (file.delete());
	}

	private boolean deleteFile(File file) {
		boolean response = false;
		if (file.exists()) {
			response = file.delete();
		}
		return response;
	}

	public String findLeaFolder(String initialPath) {
		pathToLeaFolder = null;
		find(initialPath);
		return pathToLeaFolder;
	}

	private void find(String initialPath) {
		File file = new File(initialPath);
		if (file.exists()) {
			File[] files = file.listFiles();
			for (int i = 0; i < files.length; i++) {
				if (files[i].isDirectory()) {
					if (files[i].getName().equals("lea")) {
						pathToLeaFolder = files[i].getAbsolutePath();
					} else {
						find(files[i].getAbsolutePath());
					}
				}
			}
		}
	}
}
