package com.lea.dao;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.Collection;
import java.util.HashMap;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

import com.lea.domain.Profile;
import com.lea.exception.LoadingException;


public class ProfileDAO extends AbstractProfileDAO {

	private static HashMap<Integer, Profile> mapProfiles = new HashMap<Integer, Profile>();

	public void save(Profile profile, String profilePath)
			throws FileNotFoundException, IOException {

		mapProfiles.put(profile.getId(), profile);

		ObjectOutputStream fWo = new ObjectOutputStream(new GZIPOutputStream(
				new FileOutputStream(profilePath)));
		if (profile != null)
			fWo.writeObject(profile);
		fWo.close();

	}

	// public Collection<Profile> loadAll() {

	// ObjectInputStream fRo = new ObjectInputStream(new FileInputStream(
	// missionFilePath));
	// Mission mission = (Mission) fRo.readObject();
	// fRo.close();
	// return mission;
	//		
	// return null;
	// }

	@Override
	public Collection<Profile> loadAll(String profilesFolderPath){

		File directory = new File(profilesFolderPath);
		File[] subfiles = directory.listFiles();
		mapProfiles.clear();

		for (int i = 0; i < subfiles.length; i++) {
			try {
				if (!subfiles[i].isDirectory()) {
					String profilePath = subfiles[i].getAbsolutePath();
					ObjectInputStream fRo = new ObjectInputStream(
							new GZIPInputStream(
									new FileInputStream(profilePath)));
					Profile profile = (Profile) fRo.readObject();
					if (profile != null) {
						profile.setId(i);
						mapProfiles.put(i, profile);
					}
					fRo.close();
				}
			} catch (Exception e) {
				//If can't load a profile keep going
				e.printStackTrace();
			}
		}
		return mapProfiles.values();
	}

	@Override
	public Profile getProfile(int profileId) {
		return mapProfiles.get(profileId);
	}

	@Override
	public HashMap<Integer, Profile> getMap() {
		return mapProfiles;
	}

	@Override
	public int add(String profileName, String profilePath, boolean value)
			throws FileNotFoundException, IOException {
		int profileId = mapProfiles.size();
		Profile profile = new Profile();
		profile.setProfileName(profileName);
		profile.setId(profileId);
		profile.setVehicle(value);
		mapProfiles.put(profileId, profile);

		ObjectOutputStream fWo = new ObjectOutputStream(new GZIPOutputStream(
				new FileOutputStream(profilePath)));
		if (profile != null)
			fWo.writeObject(profile);
		fWo.close();

		return profileId;
	}

	@Override
	public void remove(int profileId, String profilePath) {
		mapProfiles.remove(profileId);
		File file = new File(profilePath);
		if (file.exists()){
			file.delete();
		}
	}

	@Override
	public void load(String path, String installationPath)
			throws FileNotFoundException, IOException, ClassNotFoundException {
		ObjectInputStream fRo = new ObjectInputStream(new GZIPInputStream(
				new FileInputStream(path)));
		Profile profile = (Profile) fRo.readObject();
		if (profile != null) {

			int profileId = mapProfiles.size();
			profile.setId(profileId);
			mapProfiles.put(profileId, profile);

			String profilePath = installationPath + "/profiles/"
					+ profile.getProfileName() + ".lea.profile";

			ObjectOutputStream fWo = new ObjectOutputStream(
					new GZIPOutputStream(new FileOutputStream(profilePath)));
			fWo.writeObject(profile);
			fWo.close();
		}
	}
}
