package com.lea.main;

import com.lea.ui.Facade;
import com.lea.ui.mission.AmmoPanel;
import com.lea.ui.mission.LoadoutACEPanel;
import com.lea.ui.mission.MissionEditor;
import com.lea.ui.mission.RuckandModsPanel;
import com.lea.ui.mission.UnitsPanel;

public class Lea {

	/**
	 * This is the entry point for LEA. LEA is free software and agree the GNU
	 * Global Public Licence in version 3. This software is to be used with the
	 * game ArmA 2 Combined Operation by Bohemia Interactive Studio (BIS) and
	 * Advanced Combat Environment 2 (ACE2) addon made by the game community.
	 * For non commercial purpose only.
	 * 
	 * @author Major_Shepard for the [S.o.E] Team, visit us at
	 *         www.sonsofexiled.fr and BIS forum.
	 * 
	 * @param args
	 *            command line parameters
	 * 
	 */

	public static void main(String[] args) {

		try {
			String osName = System.getProperty("os.name");
			if (osName.contains("Windows")) {
				javax.swing.UIManager
						.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
			} else if (osName.contains("Linux")) {
				javax.swing.UIManager
						.setLookAndFeel("com.sun.java.swing.plaf.gtk.GTKLookAndFeel");
			}
		} catch (Exception e) {
		} 

		String path = "";
		if (args.length != 0) {
			path = args[0].trim();
		}
		/* Display version in console */
		if (path.contains("-version")) {
			System.out
					.println("Loadout Editor for ArmA 2 Combined Operations & ACE 2 (LEA)");
			System.out.println("Version " + Version.getName());
			System.out.println("Build " + Version.getVersion() + " ("
					+ Version.getYear() + ")");
			System.out.println("LEA by the [S.o.E] team, www.sonsofexiled.fr");
		} else if (path.contains("-help")) {
			System.out
					.println("LEA [path to file.lea.mission]   open mission in editor");
			System.out
					.println("LEA [path to file.lea.profile]   open profile in editor");
		} else {
			Facade facade = new Facade();
			new UnitsPanel(facade);
			new AmmoPanel(facade);
			new LoadoutACEPanel(facade);
			new RuckandModsPanel(facade);
			MissionEditor missionEditor = new MissionEditor(facade);
			missionEditor.init();

			/* Open mission in editor */
			if (path.contains(".lea.mission")) {
				missionEditor.loadMission(path.replace("\\", "/"));
			}
			/* Open profile in editor */
			else if (path.contains(".lea.profile")) {
				missionEditor.loadProfile(path.replace("\\", "/"));
			}
		}
	}
}
