package com.lea.service;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.crypto.Cipher;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;

import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;
import org.apache.commons.net.ftp.FTPReply;

import com.lea.dao.AbstractCommonDAO;
import com.lea.dao.AbstractFtpDAO;
import com.lea.dao.CommonDAO;
import com.lea.dao.FtpDAO;
import com.lea.domain.Ftp;
import com.lea.dto.FtpDTO;
import com.lea.exception.CheckException;
import com.lea.exception.FtpException;
import com.lea.exception.LoadingException;
import com.lea.exception.WritingException;
import com.lea.main.Version;

public class FtpService {

	private static final AbstractFtpDAO ftpDAO = new FtpDAO();
	private static final AbstractCommonDAO commonDAO = new CommonDAO();
	private static FTPClient ftpClient = null;
	private static Ftp ftp = null;
	private static final String UPDTATE_REPOSITORY_ADRESS = "www.sonsofexiled.fr";
	private static final int UPDTATE_REPOSITORY_PORT = 21;
	private static final String UPDTATE_REPOSITORY_LOGIN = "anonymous";
	private static final String UPDTATE_REPOSITORY_PASS = "";

	private static final byte[] secreteKey = new byte[] { 0x01, 0x72, 0x43,
			0x3E, 0x1C, 0x7A, 0x55, 0, 0x01, 0x72, 0x43, 0x3E, 0x1C, 0x7A,
			0x55, 0x4F };

	public FTPClient connect(String url, String port, String login,
			String password) throws FtpException {

		try {
			ftp = new Ftp(url, port, login, password);
			ftpClient = new FTPClient();
			ftpClient.connect(url, Integer.parseInt(port));
			ftpClient.login(login, password);
			ftpClient.setFileType(FTP.BINARY_FILE_TYPE);// binary transfert mode
			ftpClient.enterLocalPassiveMode();// passive mode
			int reply = ftpClient.getReplyCode();

			if (FTPReply.isPositiveCompletion(reply)) {
				System.out.println("Connected Success");
			} else {
				System.out.println("Connection Failed");
				ftpClient.disconnect();
				throw new FtpException("Fail to connect to " + url + ":" + port);
			}

		} catch (Exception ex) {
			ftp = null;
			throw new FtpException("Fail to connect to " + url + ":" + port);
		}
		return ftpClient;
	}

	public FTPClient reconnect() throws FtpException {

		try {
			System.out.println("Reconnect");
			ftpClient = new FTPClient();
			ftpClient.connect(ftp.getUrl(), Integer.parseInt(ftp.getPort()));
			ftpClient.login(ftp.getLogin(), ftp.getPassword());
			ftpClient.setFileType(FTP.BINARY_FILE_TYPE);// binary transfert mode
			ftpClient.enterLocalPassiveMode();// passive mode
			int reply = ftpClient.getReplyCode();

			if (FTPReply.isPositiveCompletion(reply)) {
				System.out.println("Connected Success");
			} else {
				System.out.println("Connection Failed");
				ftpClient.disconnect();
				throw new FtpException("Fail to connect to " + ftp.getUrl()
						+ ":" + ftp.getPort());
			}
			System.out.println("Reconnect Success");
		} catch (Exception ex) {
			throw new FtpException("Fail to connect to " + ftp.getUrl() + ":"
					+ ftp.getPort());
		}
		return ftpClient;
	}

	public void disconnect() {
		try {
			ftpClient.disconnect();
			ftpClient = null;
			ftp = null;
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void setUpload(File file) throws FileNotFoundException {
		ftpDAO.setUpload(file, ftpClient);
	}

	public void upload(String fileName) throws FtpException {
		try {
			ftpDAO.upload(fileName, ftpClient);
		} catch (Exception e) {
			e.printStackTrace();
			throw new FtpException("Fail to upload file(s).");
		}
	}

	public long getFtpFilesSize(Vector selectedFiles) throws IOException {
		long size = ftpDAO.getFtpFilesSize(selectedFiles, ftpClient);
		return size;
	}

	public void setDownload(String fileName, String localActualPath)
			throws FileNotFoundException {
		ftpDAO.setDownload(fileName, localActualPath, ftpClient);
	}

	public void download(String fileName) throws FtpException {
		try {
			ftpDAO.download(fileName, ftpClient);
		} catch (Exception e) {
			e.printStackTrace();
			throw new FtpException("Fail to download file(s).");
		}
	}

	public void delete(FTPFile ftpFile, String filePath) throws FtpException {
		try {
			if (ftpFile.isFile()) {
				ftpClient.deleteFile(filePath);
			} else if (ftpFile.isDirectory()) {
				ftpClient.removeDirectory(filePath);
			}
		} catch (IOException e) {
			e.printStackTrace();
			throw new FtpException("Fail to delete " + ftpFile.getName());
		}
	}

	public void rename(FTPFile ftpFile, String actualPath, String name)
			throws FtpException {
		try {
			ftpClient.rename(actualPath + "/" + ftpFile.getName(), actualPath
					+ "/" + name);
		} catch (IOException e) {
			e.printStackTrace();
			throw new FtpException("Fail to rename " + ftpFile.getName());
		}
	}

	public void makeDirectory(String actualPath, String name)
			throws FtpException {
		try {
			ftpClient.makeDirectory(name);
		} catch (IOException e) {
			e.printStackTrace();
			throw new FtpException("Fail to create directory " + name);
		}
	}

	public void checkData(String url, String port, String login, String password)
			throws CheckException {
		Ftp ftp = new Ftp(url, port, login, password);
		ftp.checkData();
		System.out.println("data checked");
	}

	public void generateAutoConfig(String url, String port, String login,
			String password, String path, String remoteMissionsFolderPath,
			String remoteProfilesFolderPath, String remotePbosFolderPath)
			throws WritingException {

		Ftp ftp = new Ftp(url, port, login, password, remoteMissionsFolderPath,
				remoteProfilesFolderPath, remotePbosFolderPath);
		try {
			Cipher cipher = getEncryptionCipher();
			ftpDAO.writeAutoConfig(cipher, ftp, path);
		} catch (Exception e) {
			e.printStackTrace();
			throw new WritingException("Can't create auto-config file " + "\n"
					+ e.getMessage());
		}
	}

	public FtpDTO importAutoConfig(String path) throws LoadingException {
		try {
			Cipher cipher = getDecryptionCipher();
			Ftp ftp = ftpDAO.readAutoConfig(cipher, path);
			FtpDTO ftpDTO = transform2FtpDTO(ftp);
			if (ftp == null) {
				throw new Exception();
			}
			return ftpDTO;

		} catch (Exception e) {
			e.printStackTrace();
			throw new LoadingException("Can't load auto-config " + "\n"
					+ e.getMessage());
		}
	}

	public Collection<FtpDTO> getFtpHistory() {
		Collection<FtpDTO> ftpDTOs = null;
		try {
			Cipher cipher = getDecryptionCipher();
			String intallationPath = commonDAO.getInstallationPath();
			Collection<Ftp> ftps = ftpDAO.readFtpHistory(cipher,
					intallationPath);
			ftpDTOs = transformFtp2DTOs(ftps);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return ftpDTOs;
	}

	public void updateFtpHistory(String url, String port, String login,
			String password, String remoteMissionsPath,
			String remoteProfilesPath, String remotePbosFolderPath) {
		try {
			Cipher cipher = getEncryptionCipher();
			String intallationPath = commonDAO.getInstallationPath();
			Ftp ftp = new Ftp(url, port, login, password, remoteMissionsPath,
					remoteProfilesPath, remotePbosFolderPath);
			ftpDAO.writeFtpHistory(cipher, ftp, intallationPath);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public boolean removeFtpHistory(FtpDTO ftpDTO) {
		String intallationPath = commonDAO.getInstallationPath();
		Ftp ftp = transformDTO2Ftp(ftpDTO);
		return ftpDAO.removeFtpHistory(ftp, intallationPath);
	}

	public String checkForUpdate() throws FtpException {
		String response = null;
		try {
			FTPClient ftpClientUpdate = new FTPClient();
			ftpClientUpdate.connect(UPDTATE_REPOSITORY_ADRESS,
					UPDTATE_REPOSITORY_PORT);
			ftpClientUpdate.login(UPDTATE_REPOSITORY_LOGIN,
					UPDTATE_REPOSITORY_PASS);
			ftpClientUpdate.setFileType(FTP.ASCII_FILE_TYPE);// ascii transfert
			// mode
			ftpClientUpdate.enterLocalPassiveMode();// passive mode
			int reply = ftpClientUpdate.getReplyCode();

			if (FTPReply.isPositiveCompletion(reply)) {
				System.out.println("Connected Success");
			} else {
				System.out.println("Connection Failed");
				ftpClientUpdate.disconnect();
				throw new FtpException("Fail to connect to remote repository.");
			}
			String installPath = commonDAO.getInstallationPath();
			String updateVersionName = ftpDAO.downloadXMLupdateFile(installPath, ftpClientUpdate);
			if (updateVersionName != null) {
				System.out.println(updateVersionName);
				ftpClientUpdate.disconnect();
				StringTokenizer stringTokenizer = new StringTokenizer(
						updateVersionName, ".");

				if (stringTokenizer.countTokens() == 3) {
					String major = stringTokenizer.nextToken();
					String minor = stringTokenizer.nextToken();
					String build = stringTokenizer.nextToken();

					int updateBuild = Integer.parseInt(build);
					int actualBuild = Version.getBuild();

					if (updateBuild > actualBuild) {
						response = updateVersionName;
					}
				}
			}else {
				response = null;
			}
		} catch (Exception ex) {
			ex.printStackTrace();
			ftp = null;
			throw new FtpException("Fail to connect to remote repository.");
		}
		return response;
	}

	public FTPClient getFtpClient() {
		return ftpClient;
	}

	public FtpDAO getFtpDAO() {
		return ftpDAO.getFtpDAO();
	}

	public Ftp getFtp() {
		return ftp;
	}

	/* Business methodes */
	private Cipher getEncryptionCipher() throws NoSuchAlgorithmException,
			NoSuchPaddingException, InvalidKeyException {
		Cipher cipher = Cipher.getInstance("AES");
		SecretKey key = new SecretKeySpec(secreteKey, "AES");
		cipher.init(Cipher.ENCRYPT_MODE, key);
		return cipher;
	}

	private Cipher getDecryptionCipher() throws NoSuchAlgorithmException,
			NoSuchPaddingException, InvalidKeyException {
		Cipher cipher = Cipher.getInstance("AES");
		SecretKey key = new SecretKeySpec(secreteKey, "AES");
		cipher.init(Cipher.DECRYPT_MODE, key);
		return cipher;
	}

	private Collection transformFtp2DTOs(final Collection ftps) {
		final Collection ftpsDTO = new ArrayList();
		for (Iterator iterator = ftps.iterator(); iterator.hasNext();) {
			final Ftp ftp = (Ftp) iterator.next();
			ftpsDTO.add(transform2FtpDTO(ftp));
		}
		return ftpsDTO;
	}

	private FtpDTO transform2FtpDTO(Ftp ftp) {
		FtpDTO ftpDTO = new FtpDTO();
		ftpDTO.setUrl(ftp.getUrl());
		ftpDTO.setPort(ftp.getPort());
		ftpDTO.setLogin(ftp.getLogin());
		ftpDTO.setPassword(ftp.getPassword());
		ftpDTO.setRemoteMissionsFolderPath(ftp.getRemoteMissionsFolderPath());
		ftpDTO.setRemoteProfilesFolderPath(ftp.getRemoteProfilesFolderPath());
		ftpDTO.setRemotePbosFolderPath(ftp.getRemotePbosFolderPath());
		return ftpDTO;
	}

	private Ftp transformDTO2Ftp(FtpDTO ftpDTO) {
		Ftp ftp = new Ftp();
		ftp.setUrl(ftpDTO.getUrl());
		ftp.setPort(ftpDTO.getPort());
		ftp.setLogin(ftpDTO.getLogin());
		ftp.setPassword(ftpDTO.getPassword());
		ftp.setRemoteMissionsFolderPath(ftpDTO.getRemoteMissionsFolderPath());
		ftp.setRemoteProfilesFolderPath(ftpDTO.getRemoteProfilesFolderPath());
		ftp.setRemotePbosFolderPath(ftpDTO.getRemotePbosFolderPath());
		return ftp;
	}
}
