package com.lea.service;

import java.lang.ref.Reference;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.swing.ImageIcon;

import com.lea.dao.AbstractCommonDAO;
import com.lea.dao.AbstractMagazineDAO;
import com.lea.dao.AmmoMagDAO;
import com.lea.dao.CommonDAO;
import com.lea.dao.CrewservedMagDAO;
import com.lea.dao.EquipMagDAO;
import com.lea.dao.ExplosiveMagDAO;
import com.lea.dao.MagazineDAO;
import com.lea.domain.Magazine;
import com.lea.dto.MagazineDTO;
import com.lea.exception.FinderException;

public class MagazineService {

	private static final AbstractMagazineDAO ammoMagDAO = new AmmoMagDAO();
	private static final AbstractMagazineDAO equipMagDAO = new EquipMagDAO();
	private static final AbstractMagazineDAO explosiveDAO = new ExplosiveMagDAO();
	private static final AbstractMagazineDAO crewservedDAO = new CrewservedMagDAO();
	private static final AbstractMagazineDAO magazineDAO = new MagazineDAO();
	private static final AbstractCommonDAO commonDAO = new CommonDAO();

	public MagazineDTO FindMagazineDescription(String magazineClassname)
			throws FinderException {

		// Finds the object
		final Magazine magazine = (Magazine) magazineDAO
				.findByClassname(magazineClassname);

		if (magazine == null)
			throw new FinderException("can't find magazine classname "
					+ magazineClassname);

		// Transforms domain objects into DTO
		final MagazineDTO magazineDTO = transformMagazine2DTO(magazine);

		return magazineDTO;
	}

	public Collection<MagazineDTO> FindAmmoMags() throws FinderException {

		// Finds all the objects
		final Collection magazines = ammoMagDAO.findAll();

		if (magazines == null)
			throw new FinderException("can't find all magazines");

		// Transforms domain objects into DTOs
		final Collection magazinesDTO = transformMagazines2DTOs(magazines);

		return magazinesDTO;
	}

	public Collection<MagazineDTO> FindEquipMags(boolean noACE) throws FinderException {

		// Finds all the objects
		final Collection<Magazine> magazines = equipMagDAO.findAll();

		if (magazines == null)
			throw new FinderException("can't find all magazines");

		List<Magazine> listmagazinesFaction = new ArrayList<Magazine>();
		for (Magazine magazine : magazines) {
			if (noACE) {
				if (magazine.getAce() == false) {
					listmagazinesFaction.add(magazine);
				}
			} else {
				listmagazinesFaction.add(magazine);
			}
		}
		Collections.sort(listmagazinesFaction);

		// Transforms domain objects into DTOs
		final Collection magazinesDTO = transformMagazines2DTOs(listmagazinesFaction);

		return magazinesDTO;
	}

	public Collection<MagazineDTO> FindExplosives(boolean noACE)
			throws FinderException {

		// Finds all the objects
		final Collection<Magazine> magazines = explosiveDAO.findAll();

		if (magazines == null)
			throw new FinderException("can't find all explosives");

		List<Magazine> listmagazinesFaction = new ArrayList<Magazine>();
		for (Magazine magazine : magazines) {
			if (noACE) {
				if (magazine.getAce() == false) {
					listmagazinesFaction.add(magazine);
				}
			} else {
				listmagazinesFaction.add(magazine);
			}
		}
		Collections.sort(listmagazinesFaction);

		// Transforms domain objects into DTOs
		final Collection magazinesDTO = transformMagazines2DTOs(listmagazinesFaction);

		return magazinesDTO;
	}

	public Collection<MagazineDTO> FindCrewServeds(boolean noACE)
			throws FinderException {

		// Finds all the objects
		final Collection<Magazine> magazines = crewservedDAO.findAll();

		if (magazines == null)
			throw new FinderException("can't find all magazines");

		List<Magazine> listmagazinesFaction = new ArrayList<Magazine>();
		for (Magazine magazine : magazines) {
			if (noACE) {
				if (magazine.getAce() == false) {
					listmagazinesFaction.add(magazine);
				}
			} else {
				listmagazinesFaction.add(magazine);
			}
		}
		Collections.sort(listmagazinesFaction);

		// Transforms domain objects into DTOs
		final Collection magazinesDTO = transformMagazines2DTOs(listmagazinesFaction);

		return magazinesDTO;
	}

	public Collection<MagazineDTO> FindExplosives(String factionName,
			boolean noACE) throws FinderException {

		// Finds all the objects
		final Collection<Magazine> magazines = explosiveDAO
				.findByFaction(factionName);

		List<Magazine> listmagazinesFaction = new ArrayList<Magazine>();
		for (Magazine magazine : magazines) {
			if (noACE) {
				if (magazine.getAce() == false) {
					listmagazinesFaction.add(magazine);
				}
			} else {
				listmagazinesFaction.add(magazine);
			}
		}

		final Collection<Magazine> magazinesDefault = explosiveDAO
				.findByFaction("Default");

		List<Magazine> listmagazinesDefault = new ArrayList<Magazine>();
		for (Magazine magazine : magazinesDefault) {
			if (!listmagazinesFaction.contains(magazine)) {
				if (noACE) {
					if (magazine.getAce() == false) {
						listmagazinesFaction.add(magazine);
					}
				} else {
					listmagazinesFaction.add(magazine);
				}
			}
		}

		if (listmagazinesFaction.size() == 0)
			throw new FinderException("faction" + factionName
					+ " has no explosives");

		Collections.sort(listmagazinesFaction);

		// Transforms domain objects into DTOs
		final Collection<MagazineDTO> magazinesFactionDTO = transformMagazines2DTOs(listmagazinesFaction);

		return magazinesFactionDTO;
	}

	public Collection<MagazineDTO> FindEquipMags(String factionName, boolean noACE)
			throws FinderException {

		// Finds all the objects
		final Collection<Magazine> magazines = equipMagDAO
				.findByFaction(factionName);

		List<Magazine> listmagazinesFaction = new ArrayList<Magazine>();
		for (Magazine magazine : magazines) {
			if (noACE) {
				if (magazine.getAce() == false) {
					listmagazinesFaction.add(magazine);
				}
			} else {
				listmagazinesFaction.add(magazine);
			}
		}

		final Collection<Magazine> magazinesDefault = equipMagDAO
				.findByFaction("Default");

		// List<Magazine> listmagazinesDefault = new ArrayList<Magazine>();
		for (Magazine magazine : magazinesDefault) {
			if (!listmagazinesFaction.contains(magazine)) {
				if (noACE) {
					if (magazine.getAce() == false) {
						listmagazinesFaction.add(magazine);
					}
				} else {
					listmagazinesFaction.add(magazine);
				}
			}
		}

		if (listmagazinesFaction.size() == 0)
			throw new FinderException("faction" + factionName
					+ " has no equip magazines");

		Collections.sort(listmagazinesFaction);

		// Transforms domain objects into DTOs
		final Collection<MagazineDTO> magazinesFactionDTO = transformMagazines2DTOs(listmagazinesFaction);

		return magazinesFactionDTO;
	}

	public Collection<MagazineDTO> FindCrewServeds(String factionName,
			boolean noACE) throws FinderException {

		// Finds all the objects
		final Collection<Magazine> magazines = crewservedDAO
				.findByFaction(factionName);

		List<Magazine> listmagazinesFaction = new ArrayList<Magazine>();
		for (Magazine magazine : magazines) {
			if (noACE) {
				if (magazine.getAce() == false) {
					listmagazinesFaction.add(magazine);
				}
			} else {
				listmagazinesFaction.add(magazine);
			}
		}

		final Collection<Magazine> magazinesDefault = crewservedDAO
				.findByFaction("Default");

		for (Magazine magazine : magazinesDefault) {
			if (!listmagazinesFaction.contains(magazine)) {
				if (noACE) {
					if (magazine.getAce() == false) {
						listmagazinesFaction.add(magazine);
					}
				} else {
					listmagazinesFaction.add(magazine);
				}
			}
		}

		if (listmagazinesFaction.size() == 0)
			throw new FinderException("faction" + factionName
					+ " has no crew served magazines");

		Collections.sort(listmagazinesFaction);

		// Transforms domain objects into DTOs
		final Collection<MagazineDTO> magazinesFactionDTO = transformMagazines2DTOs(listmagazinesFaction);

		return magazinesFactionDTO;
	}

	// Business methods

	public Set<String> getAvailableMods() {
		return magazineDAO.getAvailableMods();
	}

	public int readMagazinesClasslist() {
		int numberMagazines = magazineDAO.readMagazinesClasslist(commonDAO
				.getInstallationPath());
		return numberMagazines;
	}

	public void loadMagazines() {
		magazineDAO.createDBMagazines();
	}

	public AbstractMagazineDAO getMagazineDAO() {
		return magazineDAO;
	}

	private Collection transformMagazines2DTOs(final Collection magazines) {
		final Collection magazinesDTO = new ArrayList();
		for (Iterator iterator = magazines.iterator(); iterator.hasNext();) {
			final Magazine magazine = (Magazine) iterator.next();
			magazinesDTO.add(transformMagazine2DTO(magazine));
		}
		return magazinesDTO;
	}

	private MagazineDTO transformMagazine2DTO(Magazine magazine) {
		final MagazineDTO magazineDTO = new MagazineDTO(magazine.getClassname());
		magazineDTO.setName(magazine.getName());
		magazineDTO.setAce(magazine.getAce());
		magazineDTO.setAceWeight(magazine.getAceWeight());
		magazineDTO.setCaliber(magazine.getCaliber());
		magazineDTO.setMod(magazine.getMod());
		magazineDTO.setType(magazine.getType());
		magazineDTO.setCategory(magazine.getCategory());
		magazineDTO.setSlot(magazine.getSlot());
		return magazineDTO;
	}

	public Reference<ImageIcon> FindMagazineImageIcon(String classname) {
		String installPath = commonDAO.getInstallationPath();
		return magazineDAO.findImageIcon(installPath, classname);
	}

	public Reference<ImageIcon> FindMagazinesImageIconInIconesFolder(
			String classname) throws FinderException {
		String installPath = commonDAO.getInstallationPath();
		return magazineDAO.FindMagazinesImageIconInIconesFolder(installPath,
				classname);
	}
}
