package com.lea.service;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.lea.dao.AbstractCommonDAO;
import com.lea.dao.AbstractMissionDAO;
import com.lea.dao.AbstractScriptDAO;
import com.lea.dao.AbstractUnitDAO;
import com.lea.dao.CommonDAO;
import com.lea.dao.InitDAO;
import com.lea.dao.LoadoutInitDAO;
import com.lea.dao.LoadoutMultiDAO;
import com.lea.dao.LoadoutSoloDAO;
import com.lea.dao.MissionDAO;
import com.lea.dao.UnitDAO;
import com.lea.domain.DomainObject;
import com.lea.domain.Magazine;
import com.lea.domain.Mission;
import com.lea.domain.Unit;
import com.lea.domain.Weapon;
import com.lea.dto.DomainObjectDTO;
import com.lea.dto.MagazineDTO;
import com.lea.dto.MissionDTO;
import com.lea.dto.UnitDTO;
import com.lea.dto.WeaponDTO;
import com.lea.exception.CheckException;
import com.lea.exception.FinderException;
import com.lea.exception.LoadingException;
import com.lea.exception.WritingException;



public class MissionService {

	private static Mission mission = new Mission();
	private static final AbstractMissionDAO missionDAO = new MissionDAO();
	private static final AbstractScriptDAO initDAO = new InitDAO();
	private static final AbstractScriptDAO loadoutInitDAO = new LoadoutInitDAO();
	private static final AbstractScriptDAO loadoutSoloDAO = new LoadoutSoloDAO();
	private static final AbstractScriptDAO loadoutMultiDAO = new LoadoutMultiDAO();
	private static final AbstractUnitDAO unitDAO = new UnitDAO();
	private static final AbstractCommonDAO commonDAO = new CommonDAO();

	private List<String> listEditorNames = null;

	public MissionDTO loadMissionInfos() {

		MissionDTO missionDTO = transformMission2DTO();
		return missionDTO;
	}

	public void eraseUnitsLoadout() {
		Collection<Unit> units = mission.getMapUnit().values();
		for (Unit unit : units) {
			Weapon weapon = new Weapon();
			weapon.setClassname("");
			unit.setPrimaryWeaponLoadout(weapon);
			unit.setSecondaryWeaponLoadout(weapon);
			unit.setBackWeaponLoadout(weapon);
			unit.setPistolLoadout(weapon);
			unit.getPrimaryMagazineLoadout().clear();
			unit.getSecondaryMagazineLoadout().clear();
			unit.getRuckWeaponLoadout().clear();
			unit.getRuckMagazineLoadout().clear();
			unit.getOpticalLoadout().clear();
			for (Magazine magazine : unit.getFirstAidMagazineLoadout()) {
				magazine.setQuantity(0);
			}
			unit.getItemLoadout().clear();
			unit.getModsLoadout().clear();
			unit.setAceWeight(0.0);
			unit.setUnitProfileName("none");
		}
	}

	public Collection<UnitDTO> loadAllUnits() throws FinderException {

		// Find the objects
		Collection<Unit> units = mission.getMapUnit().values();

		if (units == null)
			throw new FinderException(
					"There is no unit to edit! Check if you have given a name to them in the editor of the mission.sqm.");

		// List<Unit> list = new ArrayList<Unit>();

		// Resolve unit from classname
		for (Unit unit : units) {

			final Unit unitResolved = (Unit) unitDAO.findByClassname(unit
					.getClassname());

			if (unitResolved != null) {
				unit.setName(unitResolved.getName());
				unit.setAce(unitResolved.getAce());
				unit.setMod(unitResolved.getMod());
				unit.setVehicle(unitResolved.isVehicle());
				unit.setNumberPrimarySlot(unitResolved.getNumberPrimarySlot());
				unit.setNumberSecondarySlot(unitResolved
						.getNumberSecondarySlot());
			} else
				unit.setName("Unknown");

			// if (unit.getEditorName().equals("")){
			// unit.setEditorName("not assigned");
			// }

			// list.add(unit);
		}

		// mission.getMapUnit().clear();
		// int id = 0;
		// for (Unit unit : list) {
		//
		// if (unit.getEditorName().equals("test")) {
		// System.out.println("test contenu dans list");
		// id = unit.getId();
		// System.out.println("id: " + id);
		// System.out.println(unit.getClassname());
		// System.out.println(unit.getGroupNumber());
		// }
		// if (unit.getId() == id) {
		// System.out.println("id identique: " + id);
		// System.out.println(unit.getEditorName());
		// System.out.println(unit.getClassname());
		// System.out.println(unit.getGroupNumber());
		// }
		// mission.getMapUnit().put(unit.getId(), unit);
		// }

		// for (Unit unit : mission.getMapUnit().values()) {
		// if (unit.getEditorName().equals("test")) {
		// System.out.println("test contenu dans units");
		// }
		// }

		// Transforms domain objects into DTO
		final Collection<UnitDTO> unitsDTO = transformUnits2DTOs(units);

		return unitsDTO;
	}

	public UnitDTO loadUnit(int unitId) throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		// Transforms domain objects into DTO
		final UnitDTO unitDTO = transformUnit2DTO(unit);

		return unitDTO;
	}

	public Collection<UnitDTO> loadUnits(int groupNumber)
			throws FinderException {

		Collection<UnitDTO> unitsDTO = loadAllUnits();

		if (unitsDTO == null)
			throw new FinderException("can't find the unit");

		Collection<UnitDTO> groupUnitsDTO = new ArrayList<UnitDTO>();

		for (UnitDTO unitDTO : unitsDTO) {
			if (unitDTO.getGroupNumber() == groupNumber)
				groupUnitsDTO.add(unitDTO);
		}

		return groupUnitsDTO;
	}

	public Collection<UnitDTO> loadUnits(String side) throws FinderException {

		Collection<UnitDTO> unitsDTO = loadAllUnits();

		if (unitsDTO == null)
			throw new FinderException("can't find the unit");

		Collection<UnitDTO> sideUnitsDTO = new ArrayList<UnitDTO>();

		for (UnitDTO unitDTO : unitsDTO) {
			if (unitDTO.getSide().equals(side))
				sideUnitsDTO.add(unitDTO);
		}
		return sideUnitsDTO;
	}

	public int loadUnitNumberPrimarySlot(int unitId) throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		return unit.getNumberPrimarySlot();
	}

	public int loadUnitNumberSecondarySlot(int unitId) throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		return unit.getNumberSecondarySlot();
	}

	public WeaponDTO loadUnitPrimaryWeaponLoadout(int unitId)
			throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		final Weapon weapon = unit.getPrimaryWeaponLoadout();
		if (weapon == null)
			throw new FinderException("unit primary weapon is empty");
		if (weapon.getClassname().equals(""))
			throw new FinderException("unit primary weapon is empty");

		// Transforms domain objects into DTOs
		final WeaponDTO weaponDTO = transformWeapon2DTO(weapon);

		return weaponDTO;
	}

	public WeaponDTO loadUnitSecondaryWeaponLoadout(int unitId)
			throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		final Weapon weapon = unit.getSecondaryWeaponLoadout();

		if (weapon == null)
			throw new FinderException("unit primary weapon is empty");
		if (weapon.getClassname().equals(""))
			throw new FinderException("unit primary weapon is empty");

		// Transforms domain objects into DTOs
		final WeaponDTO weaponDTO = transformWeapon2DTO(weapon);

		return weaponDTO;
	}

	public WeaponDTO loadUnitBackWeaponLoadout(int unitId)
			throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		final Weapon weapon = unit.getBackWeaponLoadout();

		if (weapon == null)
			throw new FinderException("unit primary weapon is empty");
		if (weapon.getClassname().equals(""))
			throw new FinderException("unit primary weapon is empty");

		// Transforms domain objects into DTOs
		final WeaponDTO weaponDTO = transformWeapon2DTO(weapon);

		return weaponDTO;
	}

	public WeaponDTO loadUnitPistolLoadout(int unitId) throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		final Weapon weapon = unit.getPistolLoadout();

		if (weapon == null)
			throw new FinderException("unit primary weapon is empty");
		if (weapon.getClassname().equals(""))
			throw new FinderException("unit primary weapon is empty");

		// Transforms domain objects into DTOs
		final WeaponDTO weaponDTO = transformWeapon2DTO(weapon);

		return weaponDTO;
	}

	public Collection<WeaponDTO> loadUnitOpticalLoadout(int unitId)
			throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		final Collection weapons = unit.getOpticalLoadout();

		if (weapons == null || weapons.isEmpty())
			throw new FinderException("can't find the unit opticals");

		// Transforms domain objects into DTOs
		final Collection weaponsDTO = transformWeapons2DTOs(weapons);

		return weaponsDTO;
	}

	public Collection<WeaponDTO> loadUnitItemLoadout(int unitId)
			throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		final Collection weapons = unit.getItemLoadout();

		if (weapons == null || weapons.isEmpty())
			throw new FinderException("can't find the unit items");

		// Transforms domain objects into DTOs
		final Collection weaponsDTO = transformWeapons2DTOs(weapons);

		return weaponsDTO;
	}

	public Collection<MagazineDTO> loadUnitPrimaryMagazineLoadout(int unitId)
			throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		final Collection magazines = unit.getPrimaryMagazineLoadout();

		if (magazines == null || magazines.isEmpty())
			throw new FinderException("can't find the unit primary magazines");

		// Transforms domain objects into DTOs
		final Collection magazinesDTO = transformMagazines2DTOs(magazines);

		return magazinesDTO;
	}

	public Collection<MagazineDTO> loadUnitSecondaryMagazineLoadout(int unitId)
			throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		final Collection magazines = unit.getSecondaryMagazineLoadout();

		if (magazines == null || magazines.size() == 0)
			throw new FinderException("can't find the unit secondary magazines");

		// Transforms domain objects into DTOs
		final Collection magazinesDTO = transformMagazines2DTOs(magazines);

		return magazinesDTO;
	}

	public Collection<MagazineDTO> loadUnitFirstAidMagazineLoadout(int unitId)
			throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		final Collection magazines = unit.getFirstAidMagazineLoadout();

		if (magazines == null || magazines.size() == 0)
			throw new FinderException("can't find the unit First Aid magazines");

		// Transforms domain objects into DTOs
		final Collection magazinesDTO = transformMagazines2DTOs(magazines);

		return magazinesDTO;
	}

	public double loadUnitWeight(int unitId) throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		return unit.getAceWeight();
	}

	public Collection<WeaponDTO> loadUnitRuckWeapons(int unitId)
			throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		final Collection weapons = unit.getRuckWeaponLoadout();

		if (weapons == null || weapons.size() == 0)
			throw new FinderException("can't find the unit ruck weapons");

		// Transforms domain objects into DTOs
		final Collection<WeaponDTO> weaponsDTO = transformWeapons2DTOs(weapons);

		return weaponsDTO;
	}

	public Collection<MagazineDTO> loadUnitRuckMagazines(int unitId)
			throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		final Collection magazines = unit.getRuckMagazineLoadout();

		if (magazines == null || magazines.size() == 0)
			throw new FinderException("can't find the unit ruck magazines");

		// Transforms domain objects into DTOs
		final Collection<MagazineDTO> magazinesDTO = transformMagazines2DTOs(magazines);

		return magazinesDTO;
	}

	public Collection<WeaponDTO> loadMissionListItemsPreferences()
			throws FinderException {

		Collection<Weapon> listItemsPreferences = mission
				.getListItemsPreferences();

		if (listItemsPreferences == null || listItemsPreferences.size() == 0)
			throw new FinderException("can't find item preferences");

		// Transforms domain objects into DTOs
		final Collection<WeaponDTO> listItemsPreferencesDTO = transformWeapons2DTOs(listItemsPreferences);

		return listItemsPreferencesDTO;
	}

	public Collection<WeaponDTO> loadMissionListOpticalsPreferences()
			throws FinderException {

		Collection<Weapon> listOpticalsPreferences = mission
				.getListOpticalsPreferences();

		if (listOpticalsPreferences == null
				|| listOpticalsPreferences.size() == 0)
			throw new FinderException("can't find opticals preferences");

		// Transforms domain objects into DTOs
		final Collection<WeaponDTO> listOpticalsPreferencesDTO = transformWeapons2DTOs(listOpticalsPreferences);

		return listOpticalsPreferencesDTO;
	}

	public Collection<MagazineDTO> loadMissionListEquipPreferences()
			throws FinderException {

		Collection<Magazine> listEquipPreferences = mission
				.getListEquipPreferences();

		if (listEquipPreferences == null || listEquipPreferences.size() == 0)
			throw new FinderException("can't find equip preferences");

		// Transforms domain objects into DTOs
		final Collection<MagazineDTO> listEquipPreferencesDTO = transformMagazines2DTOs(listEquipPreferences);

		return listEquipPreferencesDTO;
	}

	public String loadUnitProfileName(int unitId) throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		return unit.getUnitProfileName();
	}

	public boolean UnitIsVehicle(int unitId) throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		return unit.isVehicle();
	}

	public void saveUnitPrimaryWeaponLoadout(WeaponDTO weaponDTO, int unitId)
			throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		if (weaponDTO != null) {
			Weapon weapon = transform2Weapon(weaponDTO);
			unit.setPrimaryWeaponLoadout(weapon);
		}
		if (weaponDTO == null) {
			Weapon weapon = new Weapon();
			weapon.setClassname("");
			unit.setPrimaryWeaponLoadout(weapon);
		}
	}

	public void saveUnitSecondaryWeaponLoadout(WeaponDTO weaponDTO, int unitId)
			throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		if (weaponDTO != null) {
			Weapon weapon = transform2Weapon(weaponDTO);
			unit.setSecondaryWeaponLoadout(weapon);
		}
		if (weaponDTO == null) {
			Weapon weapon = new Weapon();
			weapon.setClassname("");
			unit.setSecondaryWeaponLoadout(weapon);
		}
	}

	public void saveUnitBackWeaponLoadout(WeaponDTO weaponDTO, int unitId)
			throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		if (weaponDTO != null) {
			Weapon weapon = transform2Weapon(weaponDTO);
			unit.setBackWeaponLoadout(weapon);
		}
		if (weaponDTO == null) {
			Weapon weapon = new Weapon();
			weapon.setClassname("");
			unit.setBackWeaponLoadout(weapon);
		}
	}

	public void saveUnitPistolLoadout(WeaponDTO weaponDTO, int unitId)
			throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		if (weaponDTO != null) {
			Weapon weapon = transform2Weapon(weaponDTO);
			unit.setPistolLoadout(weapon);
		}
		if (weaponDTO == null) {
			Weapon weapon = new Weapon();
			weapon.setClassname("");
			unit.setPistolLoadout(weapon);
		}
	}

	public void saveUnitPrimaryMagazines(Collection<MagazineDTO> magazinesDTO,
			int unitId) throws FinderException {
		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		unit.getPrimaryMagazineLoadout().clear();
		if (magazinesDTO != null) {
			for (MagazineDTO magazineDTO : magazinesDTO) {
				Magazine magazine = transform2Magazine(magazineDTO);
				unit.getPrimaryMagazineLoadout().add(magazine);
			}
		}
	}

	public void saveUnitSecondaryMagazines(
			Collection<MagazineDTO> magazinesDTO, int unitId)
			throws FinderException {
		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		// Clear the magazine classnames list of the unit
		unit.getSecondaryMagazineLoadout().clear();

		if (magazinesDTO != null) {
			for (MagazineDTO magazineDTO : magazinesDTO) {
				Magazine magazine = transform2Magazine(magazineDTO);
				unit.getSecondaryMagazineLoadout().add(magazine);
			}
		}
	}

	public void saveUnitItems(Collection<WeaponDTO> weaponsDTO, int unitId)
			throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		// Clear the weapon classnames list of the unit
		unit.getItemLoadout().clear();

		if (weaponsDTO != null) {
			for (WeaponDTO weaponDTO : weaponsDTO) {
				Weapon weapon = transform2Weapon(weaponDTO);
				unit.getItemLoadout().add(weapon);
			}
		}
	}

	public void saveUnitOpticalsLoadout(Collection<WeaponDTO> weaponsDTO,
			int unitId) throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		// Clear the weapon classnames list of the unit
		unit.getOpticalLoadout().clear();

		if (weaponsDTO != null) {
			for (WeaponDTO weaponDTO : weaponsDTO) {
				Weapon weapon = transform2Weapon(weaponDTO);
				unit.getOpticalLoadout().add(weapon);
			}
		}
	}

	public void saveUnitMods(Set<String> mods, int unitId)
			throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		// Clear the weapon classnames list of the unit
		unit.getModsLoadout().clear();

		if (mods != null) {
			unit.getModsLoadout().addAll(mods);
		}

	}

	public void saveUnitWeight(double weight, int unitId)
			throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		unit.setAceWeight(weight);

	}

	public void saveUnitRuck(List<DomainObjectDTO> ruckDTOs, int unitId)
			throws FinderException {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		// Clear the weapon and magazines classnames list of the unit
		unit.getRuckWeaponLoadout().clear();
		unit.getRuckMagazineLoadout().clear();

		if (ruckDTOs != null) {
			for (DomainObjectDTO domainObjectDTO : ruckDTOs) {
				if (domainObjectDTO instanceof WeaponDTO) {
					Weapon weapon = transform2Weapon((WeaponDTO) domainObjectDTO);
					unit.getRuckWeaponLoadout().add(weapon);
				}
				if (domainObjectDTO instanceof MagazineDTO) {
					Magazine magazine = transform2Magazine((MagazineDTO) domainObjectDTO);
					unit.getRuckMagazineLoadout().add(magazine);
				}
			}
		}
	}

	public void saveUnitRuckWeapons(Collection<WeaponDTO> ruckDTOs, int unitId)
			throws FinderException {
		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		unit.getRuckWeaponLoadout().clear();

		if (ruckDTOs != null) {
			for (WeaponDTO weaponDTO : ruckDTOs) {
				Weapon weapon = transform2Weapon(weaponDTO);
				unit.getRuckWeaponLoadout().add(weapon);
			}
		}
	}

	public void saveUnitRuckMagazines(Collection<MagazineDTO> ruckDTOs,
			int unitId) throws FinderException {
		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		unit.getRuckMagazineLoadout().clear();

		if (ruckDTOs != null) {
			for (MagazineDTO magazineDTO : ruckDTOs) {
				Magazine magazine = transform2Magazine(magazineDTO);
				unit.getRuckMagazineLoadout().add(magazine);
			}
		}
	}

	public void saveUnitFirstAid(Collection<MagazineDTO> firstAid, int unitId)
			throws FinderException {
		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);
		if (unit == null)
			throw new FinderException("can't find the unit");

		unit.getFirstAidMagazineLoadout().clear();

		if (firstAid != null) {
			for (MagazineDTO magazineDTO : firstAid) {
				Magazine magazine = transform2Magazine(magazineDTO);
				unit.getFirstAidMagazineLoadout().add(magazine);
			}
		}
	}

	public void saveUnitProfileName(String profileName, int unitId) {
		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);
		unit.setUnitProfileName(profileName);
		mission.getMapUnit().put(unitId, unit);
	}

	public void setUnitVehicle(boolean value, int unitId) {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);
		unit.setVehicle(value);
		mission.getMapUnit().put(unitId, unit);
	}

	public void saveUnitEditorName(int unitId, String name) {

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		unit.setEditorName(name);
		mission.getMapUnit().put(unitId, unit);
	}

	public void saveEditorDescription(int unitId, String description) {

		if (description.equals("")) {
			description = "Empty";
		}

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		unit.setEditorDescription(description);
		mission.getMapUnit().put(unitId, unit);
	}

	public void checkUnitEditorName(String name, int unitId)
			throws CheckException {

		if (name.equals("")) {
			throw new CheckException("Unit name can't be null!");
		} else if (name.toUpperCase().equals("LEADER")) {
			throw new CheckException("Unit name can't be named " + "\""
					+ "leader" + "\"" + " !");
		}

		String first = Character.toString(name.charAt(0));

		Pattern pattern = Pattern.compile("[^a-zA-Z]");
		Matcher matcher = pattern.matcher(first);
		boolean found = matcher.matches();

		if (found) {
			throw new CheckException("Unit name must start with a letter!");
		}

		pattern = Pattern.compile("[^a-zA-Z_0-9]");

		for (int i = 0; i < name.length(); i++) {
			String a = Character.toString(name.charAt(i));
			matcher = pattern.matcher(a);
			found = matcher.matches();
			if (found) {
				throw new CheckException(
						"Unit name must only contains letters and figures!");
			}
		}

		listEditorNames = new ArrayList<String>();
		Set<Map.Entry<Integer, Unit>> entrees = mission.getMapUnit().entrySet();
		Iterator<Map.Entry<Integer, Unit>> iter = entrees.iterator();
		while (iter.hasNext()) {
			Map.Entry<Integer, Unit> entree = (Map.Entry) iter.next();
			Unit u = entree.getValue();
			String editorName = u.getEditorName();
			int id = u.getId();
			if (!editorName.equals("not assigned")) {
				listEditorNames.add(u.getEditorName());
			}
		}

		Unit unit = mission.getMapUnit().get(unitId);
		String unitEditorName = unit.getEditorName();

		if (listEditorNames.contains(name) && (!name.equals(unitEditorName))) {
			throw new CheckException("Unit name already exists!");
		}
	}

	public void setUnitEditable(boolean value, int unitId) {
		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);
		if (unit != null) {
			unit.setEditable(value);
		}
	}

	public boolean unitIsEditable(int unitId) {
		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);
		return unit.isEditable();
	}

	public void saveMissionListMods(boolean noACE) {
		mission.getListMods().clear();
		for (Unit unit : mission.getMapUnit().values()) {
			mission.getListMods().addAll(unit.getModsLoadout());
		}
		if (!noACE){
			mission.getListMods().add("ACE");
		}
	}
	
	public Set<String> getListMods(){
		return mission.getListMods();
	}

	public void saveMissionListItemsPreferences(
			Collection<WeaponDTO> listItemsPreferences) {
		mission.getListItemsPreferences().clear();
		for (WeaponDTO weaponDTO : listItemsPreferences) {
			Weapon weapon = transform2Weapon(weaponDTO);
			mission.getListItemsPreferences().add(weapon);
		}
	}

	public void saveMissionListOpticalsPreferences(
			Collection<WeaponDTO> listOpticalsPreferences) {
		mission.getListOpticalsPreferences().clear();
		for (WeaponDTO weaponDTO : listOpticalsPreferences) {
			Weapon weapon = transform2Weapon(weaponDTO);
			mission.getListOpticalsPreferences().add(weapon);
		}
	}

	public void saveMissionListEquipPreferences(
			Collection<MagazineDTO> listEquipPreferences) {
		mission.getListEquipPreferences().clear();
		for (MagazineDTO magazineDTO : listEquipPreferences) {
			Magazine magazine = transform2Magazine(magazineDTO);
			mission.getListEquipPreferences().add(magazine);
		}
	}

	public void applyMissionListItemsPreferences() {
		for (Unit unit : mission.getMapUnit().values()) {
			if (!unit.isVehicle() && unit.getType().equals("PLAYABLE")) {
				for (Weapon weapon : mission.getListItemsPreferences()) {
					if (!unit.getItemLoadout().contains(weapon))
						unit.getItemLoadout().add(weapon);
				}
			}
		}
	}

	public void applyMissionListOpticalsPreferences() {
		for (Unit unit : mission.getMapUnit().values()) {
			if (!unit.isVehicle() && unit.getType().equals("PLAYABLE")) {
				for (Weapon weapon : mission.getListOpticalsPreferences()) {
					if (!unit.getOpticalLoadout().contains(weapon))
						unit.getOpticalLoadout().add(weapon);
				}
			}
		}
	}

	public void applyMissionListEquipPreferences() {

		List<Magazine> listEquipPreferences = mission.getListEquipPreferences();

		int nbEquipPreferencesBandage = 0;
		int nbEquipPreferencesMorphine = 0;
		int nbEquipPreferencesEpinephrine = 0;
		int nbEquipPreferencesMedkit = 0;

		for (Magazine magazine : listEquipPreferences) {
			if (magazine.getClassname().equals("ACE_Bandage")) {
				nbEquipPreferencesBandage++;
			} else if (magazine.getClassname().equals("ACE_Morphine")) {
				nbEquipPreferencesMorphine++;
			} else if (magazine.getClassname().equals("ACE_Epinephrine")) {
				nbEquipPreferencesEpinephrine++;
			} else if (magazine.getClassname().equals("ACE_Medkit")) {
				nbEquipPreferencesMedkit++;
			}
		}

		for (Unit unit : mission.getMapUnit().values()) {
			if (!unit.isVehicle() && unit.getType().equals("PLAYABLE")) {

				List<Magazine> firstAidMagazineLoadout = unit
						.getFirstAidMagazineLoadout();

				for (Magazine magazine : firstAidMagazineLoadout) {
					if (magazine.getClassname().equals("ACE_Bandage")
							&& (nbEquipPreferencesBandage - magazine
									.getQuantity()) > 0) {
						magazine.setQuantity(nbEquipPreferencesBandage);
					} else if (magazine.getClassname().equals("ACE_Morphine")
							&& (nbEquipPreferencesMorphine - magazine
									.getQuantity()) > 0) {
						magazine.setQuantity(nbEquipPreferencesMorphine);
					} else if (magazine.getClassname()
							.equals("ACE_Epinephrine")
							&& (nbEquipPreferencesEpinephrine - magazine
									.getQuantity()) > 0) {
						magazine.setQuantity(nbEquipPreferencesEpinephrine);
					} else if (magazine.getClassname().equals("ACE_Medkit")
							&& (nbEquipPreferencesMedkit - magazine
									.getQuantity()) > 0) {
						magazine.setQuantity(nbEquipPreferencesMedkit);
					}
				}
			}
		}
	}

	public void saveMission() throws WritingException {

		String missionFolderPath = commonDAO.getInstallationPath()
				+ "/missions";
		try {
			missionDAO.deleteMissionFile(mission, missionFolderPath);
		} catch (Exception e) {
			throw new WritingException("Failed to save the mission!" + "\n"
					+ e.getMessage());
		}
		try {
			missionDAO.save(mission, missionFolderPath);
		} catch (Exception e) {
			throw new WritingException("Failed to save the mission!");
		}
	}

	public boolean isMissionExists(File missionFolder) {

		String leaFolderPath = missionDAO.findLeaFolder(missionFolder
				.getAbsolutePath());

		if (leaFolderPath == null) {
			return false;
		}

		String sourceLocationPath = leaFolderPath + "/"
				+ missionFolder.getName() + ".lea.mission";

		File file = new File(sourceLocationPath);
		if (!file.exists()) {
			return false;
		} else {
			return true;
		}
	}

	public void copyMissionFromLEA(String destinationPath)
			throws WritingException {

		try {
			CommonService commonService = new CommonService();
			String sourceLocationPath = commonService.getMissionsFolderPath()
					+ "/" + mission.getMissionFileName() + ".lea.mission";
			String destinationLocationPath = destinationPath + "/lea/"
					+ mission.getMissionFileName() + ".lea.mission";
			missionDAO.copyMission(sourceLocationPath, destinationLocationPath);
		} catch (Exception e) {
			e.printStackTrace();
			throw new WritingException(
					"Failed to transfert LEA mission to ArmA 2 mission folder!");
		}
	}

	public void copyMissionToLEA(String missionFolderPath)
			throws WritingException {

		try {
			String leaFolderPath = missionDAO.findLeaFolder(missionFolderPath);

			if (leaFolderPath == null) {
				return;
			}

			CommonService commonService = new CommonService();
			String sourceLocationPath = leaFolderPath + "/"
					+ mission.getMissionFileName() + ".lea.mission";

			File file = new File(sourceLocationPath);
			if (!file.exists()) {
				return;
			}

			String destinationLocationPath = commonService
					.getMissionsFolderPath()
					+ "/" + mission.getMissionFileName() + ".lea.mission";

			missionDAO.copyMission(sourceLocationPath, destinationLocationPath);

		} catch (Exception e) {
			e.printStackTrace();
			throw new WritingException(
					"Failed to transfert LEA mission from ArmA 2 mission folder!");
		}
	}

	public void applyMission(String missionFilePath) throws LoadingException {

		if (!missionFilePath.contains(".lea.mission"))
			throw new LoadingException("File format is not valid!");

		Mission missionLoaded = null;

		try {
			missionLoaded = missionDAO.load(missionFilePath);

			if (missionLoaded != null) {

				Set<Map.Entry<Integer, Unit>> entrees = missionLoaded
						.getMapUnit().entrySet();
				Iterator<Map.Entry<Integer, Unit>> iter = entrees.iterator();
				Collection<Unit> unitsLoaded = new ArrayList<Unit>();
				while (iter.hasNext()) {
					Map.Entry<Integer, Unit> entree = (Map.Entry) iter.next();
					unitsLoaded.add(entree.getValue());
				}

				Set<Map.Entry<Integer, Unit>> entrees2 = mission.getMapUnit()
						.entrySet();
				Iterator<Map.Entry<Integer, Unit>> iter2 = entrees2.iterator();

				while (iter2.hasNext()) {
					boolean flag = false;
					Map.Entry<Integer, Unit> entree2 = (Map.Entry) iter2.next();
					String unitEditorName = entree2.getValue().getEditorName();
					int Id = entree2.getValue().getId();
					for (Unit unitLoaded : unitsLoaded) {
						if (unitLoaded.getEditorName().equals(unitEditorName)
								&& unitLoaded.isVehicle() == entree2.getValue()
										.isVehicle()
								&& !unitLoaded.getEditorName().equals(
										"not assigned")) {
							unitLoaded
									.setLeader(entree2.getValue().getLeader());
							unitLoaded.setEditorDescription(entree2.getValue()
									.getEditorDescription());
							unitLoaded.setType(entree2.getValue().getType());
							unitLoaded.setSide(entree2.getValue().getSide());
							unitLoaded.setGroupNumber(entree2.getValue()
									.getGroupNumber());
							unitLoaded.setId(entree2.getValue().getId());
							mission.getMapUnit().put(Id, unitLoaded);
							flag = true;
							break;
						}
					}
					if (flag == false) {
						mission.getMapUnit().put(entree2.getKey(),
								entree2.getValue());
					}
				}
				for (Weapon weapon : missionLoaded.getListItemsPreferences()) {
					mission.getListItemsPreferences().add(weapon);
				}
				for (Weapon weapon : missionLoaded.getListOpticalsPreferences()) {
					mission.getListOpticalsPreferences().add(weapon);
				}
				for (Magazine magazine : missionLoaded
						.getListEquipPreferences()) {
					mission.getListEquipPreferences().add(magazine);
				}
			}
		} catch (Exception e) {
			throw new LoadingException("Failed to load mission file!");
		}
	}

	public void loadMission(String missionFilePath) throws LoadingException {

		if (!missionFilePath.contains(".lea.mission"))
			throw new LoadingException("File format is not valid!");

		mission = new Mission();
		Mission missionLoaded = null;
		try {
			missionLoaded = missionDAO.load(missionFilePath);

			if (missionLoaded != null) {
				mission.setName(missionLoaded.getName());
				mission.setDescription(missionLoaded.getDescription());
				mission.setNumberPlayers(missionLoaded.getNumberPlayers());
				mission.setMissionPath(missionLoaded.getMissionPath());
				mission.setMissionFileName(missionLoaded.getMissionFileName());
				mission.setMissionFolderPath(missionLoaded
						.getMissionFolderPath());

				Set<Map.Entry<Integer, Unit>> entrees = missionLoaded
						.getMapUnit().entrySet();
				Iterator<Map.Entry<Integer, Unit>> iter = entrees.iterator();
				while (iter.hasNext()) {
					Map.Entry<Integer, Unit> entree = (Map.Entry) iter.next();
					mission.getMapUnit()
							.put(entree.getKey(), entree.getValue());
				}
				for (Weapon weapon : missionLoaded.getListItemsPreferences()) {
					mission.getListItemsPreferences().add(weapon);
				}
				for (Weapon weapon : missionLoaded.getListOpticalsPreferences()) {
					mission.getListOpticalsPreferences().add(weapon);
				}
				for (Magazine magazine : missionLoaded
						.getListEquipPreferences()) {
					mission.getListEquipPreferences().add(magazine);
				}
			}

		} catch (Exception e) {
			throw new LoadingException("Failed to load mission file!");
		}
	}

	public void importMissionSQM(String missionSQMfilePath)
			throws LoadingException {

		if (!missionSQMfilePath.contains("mission.sqm"))
			throw new LoadingException("File format is not valid!");

		mission = new Mission();
		Mission missionParsed = null;
		Mission missionLoaded = null;

		try {
			missionParsed = missionDAO.parse(missionSQMfilePath);
			checkNonAssignedEditorName(missionParsed);

			mission.setName(missionParsed.getName());
			mission.setDescription(missionParsed.getDescription());
			mission.setNumberPlayers(missionParsed.getNumberPlayers());
			mission.setMissionFileName(missionParsed.getMissionFileName());
			mission.setMissionPath(missionParsed.getMissionPath());
			mission.setMissionFolderPath(missionParsed.getMissionFolderPath());

			try {
				String missionFilePath = commonDAO.getInstallationPath()
						+ "/missions/" + missionParsed.getMissionFileName()
						+ ".lea.mission";

				File file = new File(missionFilePath);
				if (file.exists()) {
					missionLoaded = missionDAO.load(missionFilePath);
				}
			} catch (Exception e) {
				e.printStackTrace();
			}

			if (missionLoaded == null) {
				Set<Map.Entry<Integer, Unit>> entrees = missionParsed
						.getMapUnit().entrySet();
				Iterator<Map.Entry<Integer, Unit>> iter = entrees.iterator();
				while (iter.hasNext()) {
					Map.Entry<Integer, Unit> entree = (Map.Entry) iter.next();
					mission.getMapUnit()
							.put(entree.getKey(), entree.getValue());
				}
			} else {
				Set<Map.Entry<Integer, Unit>> entrees = missionLoaded
						.getMapUnit().entrySet();
				Iterator<Map.Entry<Integer, Unit>> iter = entrees.iterator();
				Collection<Unit> unitsLoaded = new ArrayList<Unit>();
				while (iter.hasNext()) {
					Map.Entry<Integer, Unit> entree = (Map.Entry) iter.next();
					unitsLoaded.add(entree.getValue());
				}

				Set<Map.Entry<Integer, Unit>> entrees2 = missionParsed.getMapUnit().entrySet();
				Iterator<Map.Entry<Integer, Unit>> iter2 = entrees2.iterator();

				while (iter2.hasNext()) {
					boolean flag = false;
					Map.Entry<Integer, Unit> entree2 = (Map.Entry) iter2.next();
					String unitEditorName = entree2.getValue().getEditorName();
					int Id = entree2.getValue().getId();
					for (Unit unitLoaded : unitsLoaded) {
						if (unitLoaded.getEditorName().equals(unitEditorName)
								&& !unitLoaded.getEditorName().equals(
										"not assigned")) {
							unitLoaded.setClassname(entree2.getValue().getClassname());
							unitLoaded
									.setLeader(entree2.getValue().getLeader());
							unitLoaded.setEditorDescription(entree2.getValue()
									.getEditorDescription());
							unitLoaded.setType(entree2.getValue().getType());
							unitLoaded.setSide(entree2.getValue().getSide());
							unitLoaded.setGroupNumber(entree2.getValue()
									.getGroupNumber());
							unitLoaded.setId(entree2.getValue().getId());
							mission.getMapUnit().put(Id, unitLoaded);
							flag = true;
							break;
						}
					}
					if (flag == false) {
						mission.getMapUnit().put(entree2.getKey(),
								entree2.getValue());
					}
				}
				for (Weapon weapon : missionLoaded.getListItemsPreferences()) {
					mission.getListItemsPreferences().add(weapon);
				}
				for (Weapon weapon : missionLoaded.getListOpticalsPreferences()) {
					mission.getListOpticalsPreferences().add(weapon);
				}
				for (Magazine magazine : missionLoaded
						.getListEquipPreferences()) {
					mission.getListEquipPreferences().add(magazine);
				}
			}
		} catch (Exception e) {
			throw new LoadingException("Failed to load mission.sqm file" + "\n"
					+ e.getMessage());
		}
	}

	private void checkNonAssignedEditorName(Mission missionParsed) {

		Set<Map.Entry<Integer, Unit>> entrees = missionParsed.getMapUnit()
				.entrySet();
		Iterator<Map.Entry<Integer, Unit>> iter = entrees.iterator();
		while (iter.hasNext()) {
			Map.Entry<Integer, Unit> entree = (Map.Entry) iter.next();
			Unit unit = entree.getValue();
			String editorName = unit.getEditorName();
			if (editorName.equals("")) {
				unit.setEditorName("not assigned");
				unit.setEditable(false);
			}
		}
	}

	public boolean generateEditorName(int unitId) throws FinderException {

		listEditorNames = new ArrayList<String>();
		Set<Map.Entry<Integer, Unit>> entrees = mission.getMapUnit().entrySet();
		Iterator<Map.Entry<Integer, Unit>> iter = entrees.iterator();
		while (iter.hasNext()) {
			Map.Entry<Integer, Unit> entree = (Map.Entry) iter.next();
			Unit u = entree.getValue();
			String editorName = u.getEditorName();
			if (!editorName.equals("not assigned")) {
				listEditorNames.add(u.getEditorName());
			}
		}

		// Find the object
		Unit unit = mission.getMapUnit().get(unitId);

		if (unit == null)
			throw new FinderException("can't find the unit");

		if (!unit.getEditorName().equals("not assigned")) {
			return false;
		}

		String editorName = unit.getGroupNumber() + "_" + unit.getId();
		int number = unit.getId();
		String name = determineEditorName(editorName, number);
		unit.setEditorName("unit_" + name);
		mission.getMapUnit().put(unit.getId(), unit);
		return true;
	}

	private String determineEditorName(String editorName, int number) {
		String response = "";
		if (!listEditorNames.contains(editorName)) {
			response = editorName;
		} else {
			number++;
			editorName = editorName.substring(0, editorName.indexOf("_")
					+ number);
			determineEditorName(editorName, number);
		}
		return response;
	}

	public void generateInitSQFScript(String path,boolean noACE) throws WritingException,
			IOException {
		initDAO.writeScript(mission, path,noACE);
	}

	public void generateLoadoutSQFScript(String path,boolean noACE) throws WritingException,
			IOException {
		loadoutInitDAO.writeScript(mission, path,noACE);
		loadoutSoloDAO.writeScript(mission, path,noACE);
		loadoutMultiDAO.writeScript(mission, path,noACE);
	}

	public void extractPBO(File sourceFile, File destinationFile)
			throws WritingException {
		try {
			missionDAO.extractPBO(sourceFile, destinationFile);
		} catch (Exception e) {
			e.printStackTrace();
			throw new WritingException(e.getMessage());
		}
	}

	public void makePBO() throws WritingException {
		String installPath = commonDAO.getInstallationPath();
		String missionFolderPath = mission.getMissionFolderPath();
		String missionFileName = mission.getMissionFileName();
		try {
			missionDAO.makePBO(installPath, missionFolderPath, missionFileName);
		} catch (Exception e) {
			e.printStackTrace();
			throw new WritingException(e.getMessage());
		}
	}

	public void checkMissionSQMExists() throws CheckException {
		String missionSQMPath = mission.getMissionPath();
		File file = new File(missionSQMPath);
		if (!file.exists()) {
			throw new CheckException(missionSQMPath + " does not exists!");
		}
	}

	public void backupMissionSQM() throws WritingException {
		try {
			String missionSQMPath = mission.getMissionPath();
			missionDAO.backupMissionSQM(missionSQMPath);
		} catch (IOException e) {
			e.printStackTrace();
			throw new WritingException(e.getMessage());
		}
	}

	public void deleteMissionSQM() {
		String missionSQMPath = mission.getMissionPath();
		missionDAO.deleteMissionSQM(missionSQMPath);
	}

	public void generateMissionSQM() throws WritingException {
		String missionFolderPath = mission.getMissionFolderPath();
		try {
			missionDAO.generateMissionSQM(mission, missionFolderPath);
		} catch (Exception e) {
			e.printStackTrace();
			throw new WritingException(e.getMessage());
		}
	}

	public void rollBackMissionSQM() throws WritingException {
		String missionSQMPath = mission.getMissionPath();
		try {
			missionDAO.rollBackMissionSQM(missionSQMPath);
		} catch (IOException e) {
			e.printStackTrace();
			throw new WritingException(e.getMessage());
		}
	}

	private Collection transformUnits2DTOs(Collection units) {
		final Collection<UnitDTO> unitsDTO = new ArrayList<UnitDTO>();
		for (Iterator iterator = units.iterator(); iterator.hasNext();) {
			final Unit unit = (Unit) iterator.next();
			unitsDTO.add(transformUnit2DTO(unit));
		}
		return unitsDTO;
	}

	private UnitDTO transformUnit2DTO(Unit unit) {
		final UnitDTO unitDTO = new UnitDTO();
		unitDTO.setEditorName(unit.getEditorName());
		unitDTO.setEditorDescription(unit.getEditorDescription());
		unitDTO.setSide(unit.getSide());
		unitDTO.setGroupNumber(unit.getGroupNumber());
		unitDTO.setName(unit.getName());
		unitDTO.setAce(unit.getAce());
		unitDTO.setType(unit.getType());
		unitDTO.setId(unit.getId());
		unitDTO.setClassname(unit.getClassname());
		unitDTO.setName(unit.getName());
		unitDTO.setLeader(unit.getLeader());
		unitDTO.setMod(unit.getMod());
		unitDTO.setVehicle(unit.isVehicle());
		unitDTO.setUnitProfileName(unit.getUnitProfileName());
		unitDTO.setNumberPrimarySlot(unit.getNumberPrimarySlot());
		unitDTO.setNumberSecondarySlot(unit.getNumberSecondarySlot());
		return unitDTO;
	}

	private DomainObjectDTO transformDomain2DTO(DomainObject domainObject) {
		DomainObjectDTO domainObjectDTO = null;
		if (domainObject instanceof Weapon)
			domainObjectDTO = transformWeapon2DTO((Weapon) domainObject);
		if (domainObject instanceof Magazine)
			domainObjectDTO = transformMagazine2DTO((Magazine) domainObject);
		return domainObjectDTO;
	}

	private Collection transformWeapons2DTOs(final Collection weapons) {
		final Collection weaponsDTO = new ArrayList();
		for (Iterator iterator = weapons.iterator(); iterator.hasNext();) {
			final Weapon weapon = (Weapon) iterator.next();
			weaponsDTO.add(transformWeapon2DTO(weapon));
		}
		return weaponsDTO;
	}

	private WeaponDTO transformWeapon2DTO(Weapon weapon) {
		final WeaponDTO weaponDTO = new WeaponDTO(weapon.getClassname());
		weaponDTO.setName(weapon.getName());
		weaponDTO.setAce(weapon.getAce());
		weaponDTO.setAceWeight(weapon.getAceWeight());
		weaponDTO.setCaliber(weapon.getCaliber());
		weaponDTO.setMod(weapon.getMod());
		weaponDTO.setType(weapon.getType());
		weaponDTO.setQuantity(weapon.getQuantity());
		return weaponDTO;
	}

	private Collection transformMagazines2DTOs(final Collection magazines) {
		final Collection magazinesDTO = new ArrayList();
		for (Iterator iterator = magazines.iterator(); iterator.hasNext();) {
			final Magazine magazine = (Magazine) iterator.next();
			magazinesDTO.add(transformMagazine2DTO(magazine));
		}
		return magazinesDTO;
	}

	private MagazineDTO transformMagazine2DTO(Magazine magazine) {
		final MagazineDTO magazineDTO = new MagazineDTO(magazine.getClassname());
		magazineDTO.setName(magazine.getName());
		magazineDTO.setAce(magazine.getAce());
		magazineDTO.setAceWeight(magazine.getAceWeight());
		magazineDTO.setCaliber(magazine.getCaliber());
		magazineDTO.setMod(magazine.getMod());
		magazineDTO.setType(magazine.getType());
		magazineDTO.setCategory(magazine.getCategory());
		magazineDTO.setSlot(magazine.getSlot());
		magazineDTO.setQuantity(magazine.getQuantity());
		return magazineDTO;
	}

	private MissionDTO transformMission2DTO() {
		MissionDTO missionDTO = new MissionDTO();
		missionDTO.setMissionName(mission.getName());
		missionDTO.setMissionDescription(mission.getDescription());
		missionDTO.setNumberPlayers(mission.getNumberPlayers());
		missionDTO.setMissionFileName(mission.getMissionFileName());
		missionDTO.setMissionFolderPath(mission.getMissionFolderPath());
		return missionDTO;
	}

	private Weapon transform2Weapon(WeaponDTO weaponDTO) {
		final Weapon weapon = new Weapon(weaponDTO.getClassname());
		weapon.setName(weaponDTO.getName());
		weapon.setAce(weaponDTO.getAce());
		weapon.setAceWeight(weaponDTO.getAceWeight());
		weapon.setCaliber(weaponDTO.getCaliber());
		weapon.setMod(weaponDTO.getMod());
		weapon.setType(weaponDTO.getType());
		weapon.setQuantity(weaponDTO.getQuantity());
		return weapon;
	}

	private Magazine transform2Magazine(MagazineDTO magazineDTO) {
		final Magazine magazine = new Magazine(magazineDTO.getClassname());
		magazine.setName(magazineDTO.getName());
		magazine.setAce(magazineDTO.getAce());
		magazine.setAceWeight(magazineDTO.getAceWeight());
		magazine.setCaliber(magazineDTO.getCaliber());
		magazine.setMod(magazineDTO.getMod());
		magazine.setType(magazineDTO.getType());
		magazine.setCategory(magazineDTO.getCategory());
		magazine.setSlot(magazineDTO.getSlot());
		magazine.setQuantity(magazineDTO.getQuantity());
		return magazine;
	}

	public static Mission getMission() {
		return mission;
	}

	public AbstractMissionDAO getMissionDAO() {
		return missionDAO;
	}
}
