package com.lea.service;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.StringTokenizer;

import com.lea.dao.AbstractCommonDAO;
import com.lea.dao.AbstractMagazineDAO;
import com.lea.dao.AbstractProfileDAO;
import com.lea.dao.AbstractScriptDAO;
import com.lea.dao.AbstractWeaponDAO;
import com.lea.dao.CommonDAO;
import com.lea.dao.LoadoutAddonDAO;
import com.lea.dao.MagazineDAO;
import com.lea.dao.ProfileDAO;
import com.lea.dao.WeaponDAO;
import com.lea.domain.Magazine;
import com.lea.domain.Profile;
import com.lea.domain.Unit;
import com.lea.domain.Weapon;
import com.lea.dto.DomainObjectDTO;
import com.lea.dto.MagazineDTO;
import com.lea.dto.ProfileDTO;
import com.lea.dto.WeaponDTO;
import com.lea.exception.FinderException;
import com.lea.exception.LoadingException;
import com.lea.exception.WritingException;

public class ProfileService {

	private static final AbstractProfileDAO profileDAO = new ProfileDAO();
	private static final AbstractCommonDAO commonDAO = new CommonDAO();
	private static final AbstractScriptDAO scriptDAO = new LoadoutAddonDAO();
	private static final AbstractWeaponDAO weaponDAO = new WeaponDAO();
	private static final AbstractMagazineDAO magazineDAO = new MagazineDAO();

	public void saveUnitAsProfile(int unitId, String profileName)
			throws WritingException {
		MissionService missionService = new MissionService();
		Unit unit = missionService.getMission().getMapUnit().get(unitId);
		Profile profile = transformUnit2Profile(unit);
		profile.setProfileName(profileName);
		saveProfile(profile);
	}

	public void saveAllProfiles() throws WritingException {
		Collection<Profile> profiles = profileDAO.getMap().values();
		for (Profile profile : profiles)
			saveProfile(profile);
	}

	public void saveProfile(int profileId) throws WritingException {
		// Find the object
		Profile profile = profileDAO.getMap().get(profileId);
		saveProfile(profile);
	}

	public void saveProfile(Profile profile) throws WritingException {

		try {

			Collection<Profile> profiles = profileDAO.getMap().values();

			if (profiles == null) {
				profile.setId(0);
			} else {
				boolean cointains = false;
				for (Profile p : profiles) {
					if (p.getProfileName().equals(profile.getProfileName())) {
						profile.setId(p.getId());
						cointains = true;
					}
				}
				if (!cointains) {
					profile.setId(profiles.size());
				}
			}

			String path = commonDAO.getInstallationPath();
			String profilePath = path + "/profiles/" + profile.getProfileName()
					+ ".lea.profile";

			profileDAO.save(profile, profilePath);

		} catch (Exception e) {
			throw new WritingException("Failed to save the profile "
					+ profile.getProfileName() + "!");
		}
	}

	public Collection<ProfileDTO> loadProfiles() throws FinderException {

		String profilesFolderPath = commonDAO.getInstallationPath()
				+ "/profiles";
		Collection<Profile> profiles = profileDAO.loadAll(profilesFolderPath);

		if (profiles == null || profiles.size() == 0) {
			throw new FinderException("Profiles list is empty.");
		}

		List<Profile> listProfiles = new ArrayList();
		for (Profile profile : profiles) {
			listProfiles.add(profile);
		}
		Collections.sort(listProfiles);

		final Collection<ProfileDTO> profilesDTO = transformProfles2DTOs(listProfiles);

		return profilesDTO;
	}

	public void saveProfilePrimaryWeaponLoadout(WeaponDTO weaponDTO,
			int profileId) throws FinderException {

		// Find the object
		Profile profile = profileDAO.getMap().get(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		if (weaponDTO != null) {
			Weapon weapon = transform2Weapon(weaponDTO);
			profile.setPrimaryWeaponLoadout(weapon);
		}
		if (weaponDTO == null) {
			Weapon weapon = new Weapon();
			weapon.setClassname("");
			profile.setPrimaryWeaponLoadout(weapon);
		}
	}

	public void saveProfileSecondaryWeaponLoadout(WeaponDTO weaponDTO,
			int profileId) throws FinderException {

		// Find the object
		Profile profile = profileDAO.getMap().get(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		if (weaponDTO != null) {
			Weapon weapon = transform2Weapon(weaponDTO);
			profile.setSecondaryWeaponLoadout(weapon);
		}
		if (weaponDTO == null) {
			Weapon weapon = new Weapon();
			weapon.setClassname("");
			profile.setSecondaryWeaponLoadout(weapon);
		}

	}

	public void saveProfileBackWeaponLoadout(WeaponDTO weaponDTO, int profileId)
			throws FinderException {

		// Find the object
		Profile profile = profileDAO.getMap().get(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		if (weaponDTO != null) {
			Weapon weapon = transform2Weapon(weaponDTO);
			profile.setBackWeaponLoadout(weapon);
		}
		if (weaponDTO == null) {
			Weapon weapon = new Weapon();
			weapon.setClassname("");
			profile.setBackWeaponLoadout(weapon);
		}

	}

	public void saveProfilePistolLoadout(WeaponDTO weaponDTO, int profileId)
			throws FinderException {

		// Find the object
		Profile profile = profileDAO.getMap().get(profileId);

		if (profile == null)
			throw new FinderException("can't find the unit");

		if (weaponDTO != null) {
			Weapon weapon = transform2Weapon(weaponDTO);
			profile.setPistolLoadout(weapon);
		}
		if (weaponDTO == null) {
			Weapon weapon = new Weapon();
			weapon.setClassname("");
			profile.setPistolLoadout(weapon);
		}
	}

	public void saveProfileItems(Collection<WeaponDTO> weaponsDTO, int profileId)
			throws FinderException {

		// Find the object
		Profile profile = profileDAO.getMap().get(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		if (weaponsDTO != null) {
			// Clear the weapon classnames list of the unit
			profile.getItemLoadout().clear();

			for (WeaponDTO weaponDTO : weaponsDTO) {
				Weapon weapon = transform2Weapon(weaponDTO);
				profile.getItemLoadout().add(weapon);
			}
		}
	}

	public void saveProfilePrimaryMagazines(
			Collection<MagazineDTO> magazinesDTO, int profileId)
			throws FinderException {

		// Find the object
		Profile profile = profileDAO.getMap().get(profileId);

		if (profile == null)
			throw new FinderException("can't find the unit");

		if (magazinesDTO != null) {
			profile.getPrimaryMagazineLoadout().clear();
			for (MagazineDTO magazineDTO : magazinesDTO) {
				Magazine magazine = transform2Magazine(magazineDTO);
				profile.getPrimaryMagazineLoadout().add(magazine);
			}
		}
	}

	public void saveProfileSecondaryMagazines(
			Collection<MagazineDTO> magazinesDTO, int profileId)
			throws FinderException {

		// Find the object
		Profile profile = profileDAO.getMap().get(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		if (magazinesDTO != null) {
			// Clear the magazine classnames list of the unit
			profile.getSecondaryMagazineLoadout().clear();

			for (MagazineDTO magazineDTO : magazinesDTO) {
				Magazine magazine = transform2Magazine(magazineDTO);
				profile.getSecondaryMagazineLoadout().add(magazine);
			}
		}
	}

	public void saveProfileOpticalsLoadout(Collection<WeaponDTO> weaponsDTO,
			int profileId) throws FinderException {

		// Find the object
		Profile profile = profileDAO.getMap().get(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		if (weaponsDTO != null) {
			// Clear the weapon classnames list of the unit
			profile.getOpticalLoadout().clear();

			for (WeaponDTO weaponDTO : weaponsDTO) {
				Weapon weapon = transform2Weapon(weaponDTO);
				profile.getOpticalLoadout().add(weapon);
			}
		}
	}

	public void saveProfileMods(Set<String> mods, int profileId)
			throws FinderException {

		// Find the object
		Profile profile = profileDAO.getMap().get(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		if (mods != null) {
			// Clear the weapon classnames list of the unit
			profile.getModsLoadout().clear();
			profile.getModsLoadout().addAll(mods);
		}
	}

	public void saveProfileWeight(double weight, int profileId)
			throws FinderException {

		// Find the object
		Profile profile = profileDAO.getMap().get(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		profile.setAceWeight(weight);

	}

	public void saveProfileRuck(List<DomainObjectDTO> ruckDTOs, int profileId)
			throws FinderException {

		// Find the object
		Profile profile = profileDAO.getMap().get(profileId);

		if (profile == null)
			throw new FinderException("can't find the unit");

		if (ruckDTOs != null) {
			profile.getRuckWeaponLoadout().clear();
			profile.getRuckMagazineLoadout().clear();

			for (DomainObjectDTO domainObjectDTO : ruckDTOs) {
				if (domainObjectDTO instanceof WeaponDTO) {
					Weapon weapon = transform2Weapon((WeaponDTO) domainObjectDTO);
					profile.getRuckWeaponLoadout().add(weapon);
				}
				if (domainObjectDTO instanceof MagazineDTO) {
					Magazine magazine = transform2Magazine((MagazineDTO) domainObjectDTO);
					profile.getRuckMagazineLoadout().add(magazine);
				}
			}
		}
	}

	public void saveProfileRuckWeapons(Collection<WeaponDTO> ruckDTOs,
			int profileId) throws FinderException {

		// Find the object
		Profile profile = profileDAO.getMap().get(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");
		if (ruckDTOs != null) {
			profile.getRuckWeaponLoadout().clear();

			for (WeaponDTO weaponDTO : ruckDTOs) {
				Weapon weapon = transform2Weapon(weaponDTO);
				profile.getRuckWeaponLoadout().add(weapon);
			}
		}
	}

	public void saveProfileRuckMagazines(Collection<MagazineDTO> ruckDTOs,
			int profileId) throws FinderException {

		// Find the object
		Profile profile = profileDAO.getMap().get(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");
		if (ruckDTOs != null) {
			profile.getRuckMagazineLoadout().clear();

			for (MagazineDTO magazineDTO : ruckDTOs) {
				Magazine magazine = transform2Magazine(magazineDTO);
				profile.getRuckMagazineLoadout().add(magazine);
			}
		}
	}

	public void saveProfileName(String profileName, int profileId)
			throws FinderException {

		// Find the object
		Profile profile = profileDAO.getMap().get(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		profile.setProfileName(profileName);

	}

	public void saveProfileFirstAid(List<MagazineDTO> firstAid, int profileId)
			throws FinderException {

		// Find the object
		Profile profile = profileDAO.getMap().get(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		profile.getFirstAidMagazineLoadout().clear();
		for (MagazineDTO magazineDTO : firstAid) {
			Magazine magazine = transform2Magazine(magazineDTO);
			profile.getFirstAidMagazineLoadout().add(magazine);
		}
	}

	public WeaponDTO loadProfilePrimaryWeaponLoadout(int profileId)
			throws FinderException {

		// Find the object
		Profile profile = profileDAO.getProfile(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		final Weapon weapon = profile.getPrimaryWeaponLoadout();
		if (weapon == null)
			throw new FinderException("profile primary weapon is empty");
		if (weapon.getClassname().equals(""))
			throw new FinderException("profile primary weapon is empty");

		// Transforms domain objects into DTOs
		final WeaponDTO weaponDTO = transformWeapon2DTO(weapon);

		return weaponDTO;
	}

	public WeaponDTO loadProfileSecondaryWeaponLoadout(int profileId)
			throws FinderException {

		// Find the object
		Profile profile = profileDAO.getProfile(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		final Weapon weapon = profile.getSecondaryWeaponLoadout();

		if (weapon == null)
			throw new FinderException("profile primary weapon is empty");
		if (weapon.getClassname().equals(""))
			throw new FinderException("profile primary weapon is empty");

		// Transforms domain objects into DTOs
		final WeaponDTO weaponDTO = transformWeapon2DTO(weapon);

		return weaponDTO;
	}

	public WeaponDTO loadProfileBackWeaponLoadout(int profileId)
			throws FinderException {

		// Find the object
		Profile profile = profileDAO.getProfile(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		final Weapon weapon = profile.getBackWeaponLoadout();

		if (weapon == null)
			throw new FinderException("profile primary weapon is empty");
		if (weapon.getClassname().equals(""))
			throw new FinderException("profile primary weapon is empty");

		// Transforms domain objects into DTOs
		final WeaponDTO weaponDTO = transformWeapon2DTO(weapon);

		return weaponDTO;
	}

	public WeaponDTO loadProfilePistolLoadout(int profileId)
			throws FinderException {

		// Find the object
		Profile profile = profileDAO.getProfile(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		final Weapon weapon = profile.getPistolLoadout();

		if (weapon == null)
			throw new FinderException("profile primary weapon is empty");
		if (weapon.getClassname().equals(""))
			throw new FinderException("profile primary weapon is empty");

		// Transforms domain objects into DTOs
		final WeaponDTO weaponDTO = transformWeapon2DTO(weapon);

		return weaponDTO;
	}

	public Collection<WeaponDTO> loadProfileOpticalLoadout(int profileId)
			throws FinderException {

		// Find the object
		Profile profile = profileDAO.getProfile(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		final Set<Weapon> weapons = profile.getOpticalLoadout();

		if (weapons == null || weapons.size() == 0)
			throw new FinderException("can't find the profile opticals");

		// Transforms domain objects into DTOs
		final Collection weaponsDTO = transformWeapons2DTOs(weapons);

		return weaponsDTO;
	}

	public Collection<WeaponDTO> loadProfileItemLoadout(int profileId)
			throws FinderException {

		// Find the object
		Profile profile = profileDAO.getProfile(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		final Set<Weapon> weapons = profile.getItemLoadout();

		if (weapons == null || weapons.size() == 0)
			throw new FinderException("can't find the profile items");

		// Transforms domain objects into DTOs
		final Collection weaponsDTO = transformWeapons2DTOs(weapons);

		return weaponsDTO;
	}

	public Collection<MagazineDTO> loadProfilePrimaryMagazineLoadout(
			int profileId) throws FinderException {

		// Find the object
		Profile profile = profileDAO.getProfile(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		final Collection magazines = profile.getPrimaryMagazineLoadout();

		if (magazines == null || magazines.size() == 0)
			throw new FinderException(
					"can't find the profile primary magazines");

		// Transforms domain objects into DTOs
		final Collection magazinesDTO = transformMagazines2DTOs(magazines);

		return magazinesDTO;
	}

	public Collection<MagazineDTO> loadProfileSecondaryMagazineLoadout(
			int profileId) throws FinderException {

		// Find the object
		Profile profile = profileDAO.getProfile(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		final Collection magazines = profile.getSecondaryMagazineLoadout();

		if (magazines == null || magazines.size() == 0)
			throw new FinderException(
					"can't find the profile secondary magazines");

		// Transforms domain objects into DTOs
		final Collection magazinesDTO = transformMagazines2DTOs(magazines);

		return magazinesDTO;
	}

	public double loadProfileWeight(int profileId) throws FinderException {

		// Find the object
		Profile profile = profileDAO.getProfile(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		return profile.getAceWeight();
	}

	public Collection loadProfileRuckWeapons(int profileId)
			throws FinderException {

		// Find the object
		Profile profile = profileDAO.getProfile(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		final Collection weapons = profile.getRuckWeaponLoadout();

		if (weapons == null || weapons.size() == 0)
			throw new FinderException("can't find the unit ruck weapons");

		// Transforms domain objects into DTOs
		final Collection<WeaponDTO> weaponsDTO = transformWeapons2DTOs(weapons);

		return weaponsDTO;
	}

	public Collection loadProfileRuckMagazines(int profileId)
			throws FinderException {

		// Find the object
		Profile profile = profileDAO.getProfile(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		final Collection magazines = profile.getRuckMagazineLoadout();

		if (magazines == null || magazines.size() == 0)
			throw new FinderException("can't find the unit ruck magazines");

		// Transforms domain objects into DTOs
		final Collection<MagazineDTO> magazinesDTO = transformMagazines2DTOs(magazines);

		return magazinesDTO;
	}

	public Collection<MagazineDTO> loadProfileFirstAidMagazineLoadout(
			int profileId) throws FinderException {

		// Find the object
		Profile profile = profileDAO.getProfile(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		final Collection magazines = profile.getFirstAidMagazineLoadout();

		if (magazines == null || magazines.size() == 0)
			throw new FinderException("can't find the unit First Aid magazines");

		// Transforms domain objects into DTOs
		final Collection magazinesDTO = transformMagazines2DTOs(magazines);

		return magazinesDTO;
	}

	public ProfileDTO findProfile(int profileId) throws FinderException {
		Profile profile = profileDAO.getProfile(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile.");

		ProfileDTO profileDTO = transformProfile2DTO(profile);
		return profileDTO;
	}

	public ProfileDTO findProfile(String profileName) throws FinderException {

		Collection<Profile> profiles = profileDAO.getMap().values();

		if (profiles == null)
			throw new FinderException("can't find the profile.");

		ProfileDTO profileDTO = null;

		for (Profile profile : profiles) {
			if (profile.getProfileName().equals(profileName)) {
				profileDTO = transformProfile2DTO(profile);
			}
		}

		if (profileDTO == null)
			throw new FinderException("can't find the profile.");

		return profileDTO;
	}

	public int createNewProfile(String profileName, boolean value)
			throws WritingException {
		String path = commonDAO.getInstallationPath();
		String profilePath = path + "/profiles/" + profileName + ".lea.profile";
		if (new File(profilePath).exists()) {
			throw new WritingException("Profile already exists!");
		}
		int profileId = 0;
		try {
			profileId = profileDAO.add(profileName, profilePath, value);
		} catch (Exception e) {
			throw new WritingException("Can't create profile!");
		}
		return profileId;
	}

	public void renameProfile(int profileId, String newProfileName)
			throws WritingException {

		Profile profile = profileDAO.getMap().get(profileId);
		String path = commonDAO.getInstallationPath();
		String profilePath = path + "/profiles/" + profile.getProfileName()
				+ ".lea.profile";
		String newProfilePath = path + "/profiles/" + newProfileName
				+ ".lea.profile";

		if (new File(newProfilePath).exists()) {
			throw new WritingException("Profile already exists!");
		}

		profileDAO.remove(profileId, profilePath);
		profile.setProfileName(newProfileName);
		saveProfile(profile);
	}

	public void removeProfile(int profileId) {

		Profile profile = profileDAO.getMap().get(profileId);
		String path = commonDAO.getInstallationPath();
		String profilePath = path + "/profiles/" + profile.getProfileName()
				+ ".lea.profile";

		profileDAO.remove(profileId, profilePath);
	}

	public void importProfile(String path) throws LoadingException {

		String installationPath = commonDAO.getInstallationPath();

		try {
			profileDAO.load(path, installationPath);

		} catch (Exception e) {
			throw new LoadingException("Can't load profile!");
		}
	}

	public void initLeaAddon(int profileId, String path, boolean noACE)
			throws WritingException, IOException {

		Profile profile = profileDAO.getMap().get(profileId);
		scriptDAO.writeScript(profile, path,noACE);
	}

	public Collection<WeaponDTO> loadProfilesListItemsPreferences()
			throws FinderException {

		Collection<WeaponDTO> listItemsPreferences = new ArrayList<WeaponDTO>();
		String itemsPreferences = commonDAO.getProfilesListItemsPreferences();

		if (itemsPreferences == null) {
			throw new FinderException("can't find item preferences");
		}

		StringTokenizer stringTokenizer = new StringTokenizer(itemsPreferences,
				",");

		while (stringTokenizer.hasMoreTokens()) {
			String weaponClassname = stringTokenizer.nextToken();

			// Finds the object
			final Weapon weapon = (Weapon) weaponDAO
					.findByClassname(weaponClassname);

			if (weapon == null)
				throw new FinderException("weapon classname " + weaponClassname
						+ " not found");

			// Transforms domain objects into DTO
			final WeaponDTO weaponDTO = transformWeapon2DTO(weapon);

			listItemsPreferences.add(weaponDTO);
		}

		if (listItemsPreferences.size() == 0)
			throw new FinderException("can't find item preferences");

		return listItemsPreferences;
	}

	public Collection<WeaponDTO> loadProfilesListOpticalsPreferences()
			throws FinderException {

		Collection<WeaponDTO> listOpticalsPreferences = new ArrayList<WeaponDTO>();
		String opticalsPreferences = commonDAO
				.getProfilesListOpticalsPreferences();

		if (opticalsPreferences == null) {
			throw new FinderException("can't find optical preferences");
		}

		StringTokenizer stringTokenizer = new StringTokenizer(
				opticalsPreferences, ",");

		while (stringTokenizer.hasMoreTokens()) {
			String weaponClassname = stringTokenizer.nextToken();

			// Finds the object
			final Weapon weapon = (Weapon) weaponDAO
					.findByClassname(weaponClassname);

			if (weapon == null)
				throw new FinderException("weapon classname " + weaponClassname
						+ " not found");

			// Transforms domain objects into DTO
			final WeaponDTO weaponDTO = transformWeapon2DTO(weapon);

			listOpticalsPreferences.add(weaponDTO);
		}

		if (listOpticalsPreferences.size() == 0)
			throw new FinderException("can't find item preferences");

		return listOpticalsPreferences;
	}

	public Collection<MagazineDTO> loadProfilesListEquipPreferences()
			throws FinderException {

		Collection<MagazineDTO> listEquipPreferences = new ArrayList<MagazineDTO>();
		String equipPreferences = commonDAO.getProfilesListEquipPreferences();

		if (equipPreferences == null) {
			throw new FinderException("can't find optical preferences");
		}

		StringTokenizer stringTokenizer = new StringTokenizer(equipPreferences,
				",");

		while (stringTokenizer.hasMoreTokens()) {
			String magazineClassname = stringTokenizer.nextToken();

			// Finds the object
			final Magazine magazine = (Magazine) magazineDAO
					.findByClassname(magazineClassname);

			if (magazine == null)
				throw new FinderException("magazine classname "
						+ magazineClassname + " not found");

			// Transforms domain objects into DTO
			final MagazineDTO magazineDTO = transformMagazine2DTO(magazine);
			listEquipPreferences.add(magazineDTO);
		}

		if (listEquipPreferences.size() == 0)
			throw new FinderException("can't find item preferences");

		return listEquipPreferences;
	}

	public void saveProfilesListItemsPreferences(
			Collection<WeaponDTO> listItemsPreferences) {

		StringBuffer stringBuffer = new StringBuffer();
		for (WeaponDTO weaponDTO : listItemsPreferences) {
			stringBuffer.append(weaponDTO.getClassname());
			stringBuffer.append(",");
		}
		commonDAO.setProfilesListItemsPreferences(stringBuffer.toString());
	}

	public void saveProfilesListOpticalsPreferences(
			Collection<WeaponDTO> listOpticalsPreferences) {

		StringBuffer stringBuffer = new StringBuffer();
		for (WeaponDTO weaponDTO : listOpticalsPreferences) {
			stringBuffer.append(weaponDTO.getClassname());
			stringBuffer.append(",");
		}
		commonDAO.setProfilesListOpticalsPreferences(stringBuffer.toString());
	}

	public void saveProfilesListEquipPreferences(
			Collection<MagazineDTO> listEquipPreferences) {

		StringBuffer stringBuffer = new StringBuffer();
		for (MagazineDTO magazineDTO : listEquipPreferences) {
			stringBuffer.append(magazineDTO.getClassname());
			stringBuffer.append(",");
		}
		commonDAO.setProfilesListEquipPreferences(stringBuffer.toString());
	}

	public void applyProfileListItemsPreferences(int profileId)
			throws FinderException {

		Collection<WeaponDTO> weaponsDTO = null;
		try {
			weaponsDTO = loadProfilesListItemsPreferences();
		} catch (FinderException e) {
		}

		// Find the object
		Profile profile = profileDAO.getMap().get(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		if (weaponsDTO != null) {
			for (WeaponDTO weaponDTO : weaponsDTO) {
				Weapon weapon = transform2Weapon(weaponDTO);
				if (!profile.getItemLoadout().contains(weapon)) {
					profile.getItemLoadout().add(weapon);
				}
			}
		}
	}

	public void applyProfileListOpticalsPreferences(int profileId)
			throws FinderException {

		Collection<WeaponDTO> weaponsDTO = null;
		try {
			weaponsDTO = loadProfilesListOpticalsPreferences();
		} catch (FinderException e) {
		}

		// Find the object
		Profile profile = profileDAO.getMap().get(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		if (weaponsDTO != null) {
			for (WeaponDTO weaponDTO : weaponsDTO) {
				Weapon weapon = transform2Weapon(weaponDTO);
				if (!profile.getOpticalLoadout().contains(weapon)) {
					profile.getOpticalLoadout().add(weapon);
				}
			}
		}
	}

	public void applyProfileListEquipPreferences(int profileId)
			throws FinderException {

		Collection<MagazineDTO> magazinesDTO = null;
		try {
			magazinesDTO = loadProfilesListEquipPreferences();
		} catch (FinderException e) {
		}

		// Find the object
		Profile profile = profileDAO.getMap().get(profileId);

		if (profile == null)
			throw new FinderException("can't find the profile");

		if (magazinesDTO != null) {

			int nbEquipPreferencesBandage = 0;
			int nbEquipPreferencesMorphine = 0;
			int nbEquipPreferencesEpinephrine = 0;
			int nbEquipPreferencesMedkit = 0;

			for (MagazineDTO magazineDTO : magazinesDTO) {
				if (magazineDTO.getClassname().equals("ACE_Bandage")) {
					nbEquipPreferencesBandage++;
				} else if (magazineDTO.getClassname().equals("ACE_Morphine")) {
					nbEquipPreferencesMorphine++;
				} else if (magazineDTO.getClassname().equals("ACE_Epinephrine")) {
					nbEquipPreferencesEpinephrine++;
				} else if (magazineDTO.getClassname().equals("ACE_Medkit")) {
					nbEquipPreferencesMedkit++;
				}
			}

			List<Magazine> firstAidMagazineLoadout = profile
					.getFirstAidMagazineLoadout();

			for (Magazine magazine : firstAidMagazineLoadout) {
				if (magazine.getClassname().equals("ACE_Bandage")
						&& (nbEquipPreferencesBandage - magazine.getQuantity()) > 0) {
					magazine.setQuantity(nbEquipPreferencesBandage);
				} else if (magazine.getClassname().equals("ACE_Morphine")
						&& (nbEquipPreferencesMorphine - magazine.getQuantity()) > 0) {
					magazine.setQuantity(nbEquipPreferencesMorphine);
				} else if (magazine.getClassname().equals("ACE_Epinephrine")
						&& (nbEquipPreferencesEpinephrine - magazine
								.getQuantity()) > 0) {
					magazine.setQuantity(nbEquipPreferencesEpinephrine);
				} else if (magazine.getClassname().equals("ACE_Medkit")
						&& (nbEquipPreferencesMedkit - magazine.getQuantity()) > 0) {
					magazine.setQuantity(nbEquipPreferencesMedkit);
				}
			}
		}
	}

	private MagazineDTO transformMagazine2DTO(Magazine magazine) {
		final MagazineDTO magazineDTO = new MagazineDTO(magazine.getClassname());
		magazineDTO.setName(magazine.getName());
		magazineDTO.setAce(magazine.getAce());
		magazineDTO.setAceWeight(magazine.getAceWeight());
		magazineDTO.setCaliber(magazine.getCaliber());
		magazineDTO.setMod(magazine.getMod());
		magazineDTO.setType(magazine.getType());
		magazineDTO.setCategory(magazine.getCategory());
		magazineDTO.setSlot(magazine.getSlot());
		magazineDTO.setQuantity(magazine.getQuantity());
		return magazineDTO;
	}

	private Collection<ProfileDTO> transformProfles2DTOs(
			Collection<Profile> profiles) {
		final Collection profilesDTO = new ArrayList();
		for (Iterator iterator = profiles.iterator(); iterator.hasNext();) {
			final Profile profile = (Profile) iterator.next();
			profilesDTO.add(transformProfile2DTO(profile));
		}
		return profilesDTO;
	}

	private ProfileDTO transformProfile2DTO(Profile profile) {
		final ProfileDTO profileDTO = new ProfileDTO();
		profileDTO.setProfileName(profile.getProfileName());
		profileDTO.setVehicle(profile.isVehicle());
		profileDTO.setProfileId(profile.getId());
		return profileDTO;
	}

	private Profile transformUnit2Profile(Unit unit) {
		Profile profile = new Profile();
		profile.setModsLoadout(unit.getModsLoadout());
		profile.setOpticalLoadout(unit.getOpticalLoadout());
		profile.setPistolLoadout(unit.getPistolLoadout());
		profile.setPrimaryWeaponLoadout(unit.getPrimaryWeaponLoadout());
		profile.setSecondaryWeaponLoadout(unit.getSecondaryWeaponLoadout());
		profile.setBackWeaponLoadout(unit.getBackWeaponLoadout());
		profile.setPrimaryMagazineLoadout(unit.getPrimaryMagazineLoadout());
		profile.setSecondaryMagazineLoadout(unit.getSecondaryMagazineLoadout());
		profile.setFirstAidMagazineLoadout(unit.getFirstAidMagazineLoadout());
		profile.setItemLoadout(unit.getItemLoadout());
		profile.setVehicle(unit.isVehicle());
		profile.setAceWeight(unit.getAceWeight());
		profile.setRuckWeaponLoadout(unit.getRuckWeaponLoadout());
		profile.setRuckMagazineLoadout(unit.getRuckMagazineLoadout());
		profile.setVehicle(unit.isVehicle());
		return profile;
	}

	private Collection transformWeapons2DTOs(final Collection weapons) {
		final Collection weaponsDTO = new ArrayList();
		for (Iterator iterator = weapons.iterator(); iterator.hasNext();) {
			final Weapon weapon = (Weapon) iterator.next();
			weaponsDTO.add(transformWeapon2DTO(weapon));
		}
		return weaponsDTO;
	}

	private WeaponDTO transformWeapon2DTO(Weapon weapon) {
		final WeaponDTO weaponDTO = new WeaponDTO(weapon.getClassname());
		weaponDTO.setName(weapon.getName());
		weaponDTO.setAce(weapon.getAce());
		weaponDTO.setAceWeight(weapon.getAceWeight());
		weaponDTO.setCaliber(weapon.getCaliber());
		weaponDTO.setMod(weapon.getMod());
		weaponDTO.setType(weapon.getType());
		weaponDTO.setQuantity(weapon.getQuantity());
		return weaponDTO;
	}

	private Collection transformMagazines2DTOs(final Collection magazines) {
		final Collection magazinesDTO = new ArrayList();
		for (Iterator iterator = magazines.iterator(); iterator.hasNext();) {
			final Magazine magazine = (Magazine) iterator.next();
			magazinesDTO.add(transformMagazine2DTO(magazine));
		}
		return magazinesDTO;
	}

	private Weapon transform2Weapon(WeaponDTO weaponDTO) {
		final Weapon weapon = new Weapon(weaponDTO.getClassname());
		weapon.setName(weaponDTO.getName());
		weapon.setAce(weaponDTO.getAce());
		weapon.setAceWeight(weaponDTO.getAceWeight());
		weapon.setCaliber(weaponDTO.getCaliber());
		weapon.setMod(weaponDTO.getMod());
		weapon.setType(weaponDTO.getType());
		weapon.setQuantity(weaponDTO.getQuantity());
		return weapon;
	}

	private Magazine transform2Magazine(MagazineDTO magazineDTO) {
		final Magazine magazine = new Magazine(magazineDTO.getClassname());
		magazine.setName(magazineDTO.getName());
		magazine.setAce(magazineDTO.getAce());
		magazine.setAceWeight(magazineDTO.getAceWeight());
		magazine.setCaliber(magazineDTO.getCaliber());
		magazine.setMod(magazineDTO.getMod());
		magazine.setType(magazineDTO.getType());
		magazine.setCategory(magazineDTO.getCategory());
		magazine.setSlot(magazineDTO.getSlot());
		magazine.setQuantity(magazineDTO.getQuantity());
		return magazine;
	}

}
