package com.lea.service;

import java.util.ArrayList;
import java.util.Collection;

import com.lea.dao.AbstractCommonDAO;
import com.lea.dao.AbstractMagazineDAO;
import com.lea.dao.AbstractUnitDAO;
import com.lea.dao.AbstractWeaponDAO;
import com.lea.dao.CommonDAO;
import com.lea.dao.MagazineDAO;
import com.lea.dao.UnitDAO;
import com.lea.dao.WeaponDAO;
import com.lea.domain.Magazine;
import com.lea.domain.Unit;
import com.lea.domain.Weapon;
import com.lea.dto.MagazineDTO;
import com.lea.dto.UnitDTO;
import com.lea.dto.WeaponDTO;
import com.lea.exception.FinderException;



public class UnitService {

	private static final AbstractUnitDAO unitDAO = new UnitDAO();
	private static final AbstractWeaponDAO weaponDAO = new WeaponDAO();
	private static final AbstractMagazineDAO magazineDAO = new MagazineDAO();
	private static final AbstractCommonDAO commonDAO = new CommonDAO();

	public UnitDTO FindUnitDescription(String unitClassname)
			throws FinderException {

		// Finds the object
		final Unit unit = (Unit) unitDAO.findByClassname(unitClassname);

		if (unit == null)
			throw new FinderException("can't find unit");

		// Transforms domain objects into DTO
		final UnitDTO unitDTO = transformUnit2DTO(unit);

		return unitDTO;
	}

	public Collection<WeaponDTO> findUnitWeapons(String classname)
			throws FinderException {

		final Unit unit = unitDAO.getMap().get(classname);

		if (unit == null)
			throw new FinderException("can't find unit");

		Collection<String> weaponClassnames = unit.getListWeaponClassname();
		Collection<WeaponDTO> weaponsDTO = new ArrayList<WeaponDTO>();
		for (String weaponClassname : weaponClassnames) {
			if (!weaponClassname.equals("Put")
					|| !weaponClassname.equals("Throw")) {
				Weapon weapon = weaponDAO.findByClassname(weaponClassname);
				if (weapon != null) {
					WeaponDTO weaponDTO = transformWeapon2DTO(weapon);
					weaponsDTO.add(weaponDTO);
				}
			}
		}
		return weaponsDTO;
	}

	public Collection<MagazineDTO> findUnitMagazines(String classname)
			throws FinderException {

		final Unit unit = unitDAO.getMap().get(classname);

		if (unit == null)
			throw new FinderException("can't find unit");

		Collection<String> magazineClassnames = unit.getListMagazineClassname();
		Collection<MagazineDTO> magazinesDTO = new ArrayList<MagazineDTO>();
		for (String magazineClassname : magazineClassnames) {
			Magazine magazine = magazineDAO.findByClassname(magazineClassname);
			if (magazine != null) {
				MagazineDTO magazineDTO = transformMagazine2DTO(magazine);
				magazinesDTO.add(magazineDTO);
			}
		}
		return magazinesDTO;

	}

	private MagazineDTO transformMagazine2DTO(Magazine magazine) {
		final MagazineDTO magazineDTO = new MagazineDTO(magazine.getClassname());
		magazineDTO.setName(magazine.getName());
		magazineDTO.setAce(magazine.getAce());
		magazineDTO.setAceWeight(magazine.getAceWeight());
		magazineDTO.setCaliber(magazine.getCaliber());
		magazineDTO.setMod(magazine.getMod());
		magazineDTO.setType(magazine.getType());
		magazineDTO.setCategory(magazine.getCategory());
		magazineDTO.setSlot(magazine.getSlot());
		return magazineDTO;
	}

	private WeaponDTO transformWeapon2DTO(Weapon weapon) {
		final WeaponDTO weaponDTO = new WeaponDTO(weapon.getClassname());
		weaponDTO.setName(weapon.getName());
		weaponDTO.setAce(weapon.getAce());
		weaponDTO.setAceWeight(weapon.getAceWeight());
		weaponDTO.setCaliber(weapon.getCaliber());
		weaponDTO.setMod(weapon.getMod());
		weaponDTO.setType(weapon.getType());
		return weaponDTO;
	}

	private UnitDTO transformUnit2DTO(Unit unit) {
		final UnitDTO unitDTO = new UnitDTO(unit.getClassname());
		unitDTO.setName(unit.getName());
		unitDTO.setAce(unit.getAce());
		unitDTO.setFaction(unit.getFaction());
		unitDTO.setVehicle(unit.isVehicle());
		unitDTO.setMod(unit.getMod());
		return unitDTO;
	}

	public int readUnitsClasslist() {
		int numberUnits = unitDAO.readUnitsClasslist(commonDAO.getInstallationPath());
		return numberUnits;
	}

	public AbstractUnitDAO getUnitDAO() {
		return unitDAO;
	}

	public void loadUnits() {
		unitDAO.createDBUnits();
	}

}
