package com.lea.service;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.StringTokenizer;

import javax.swing.ImageIcon;

import com.lea.dao.AbstractCommonDAO;
import com.lea.dao.AbstractMagazineDAO;
import com.lea.dao.AbstractWeaponDAO;
import com.lea.dao.ArDAO;
import com.lea.dao.CommonDAO;
import com.lea.dao.EquipDAO;
import com.lea.dao.ItemDAO;
import com.lea.dao.LauncherDAO;
import com.lea.dao.MagazineDAO;
import com.lea.dao.MgDAO;
import com.lea.dao.PistolDAO;
import com.lea.dao.RifleDAO;
import com.lea.dao.RuckDAO;
import com.lea.dao.SniperDAO;
import com.lea.dao.WeaponDAO;
import com.lea.domain.Magazine;
import com.lea.domain.Weapon;
import com.lea.dto.MagazineDTO;
import com.lea.dto.WeaponDTO;
import com.lea.exception.FinderException;

public final class WeaponService {

	private static final AbstractWeaponDAO rifleDAO = new RifleDAO();
	private static final AbstractWeaponDAO sniperDAO = new SniperDAO();
	private static final AbstractWeaponDAO arDAO = new ArDAO();
	private static final AbstractWeaponDAO mgDAO = new MgDAO();
	private static final AbstractWeaponDAO launcherDAO = new LauncherDAO();
	private static final AbstractWeaponDAO pistolDAO = new PistolDAO();
	private static final AbstractWeaponDAO equipDAO = new EquipDAO();
	private static final AbstractWeaponDAO itemDAO = new ItemDAO();
	private static final AbstractWeaponDAO ruckDAO = new RuckDAO();
	private static final AbstractWeaponDAO weaponDAO = new WeaponDAO();
	private static final AbstractMagazineDAO magazineDAO = new MagazineDAO();
	private static final AbstractCommonDAO commonDAO = new CommonDAO();

	// --- Rifle ---//

	public Collection<WeaponDTO> FindRifles() throws FinderException {

		// Finds all the objects
		CommonService commonService = new CommonService();
		Collection<String> selectedMods = commonService.getSelectedMods();

		final Collection<Weapon> weapons = rifleDAO.findAll();

		if (weapons == null)
			throw new FinderException("can't find all classnames");

		// Sort weapons in weapon name alphabetic order
		List<Weapon> listWeapons = new ArrayList<Weapon>();
		for (Weapon weapon : weapons) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}
		Collections.sort(listWeapons);

		// Transforms domain objects into DTOs
		final Collection<WeaponDTO> weaponsDTO = transformWeapons2DTOs(listWeapons);

		return weaponsDTO;
	}

	public Collection<WeaponDTO> FindRifles(String factionName)
			throws FinderException {

		// Finds all the objects
		final Collection<Weapon> weaponsFaction = rifleDAO
				.findByFaction(factionName);

		// Sort weapons in weapon name alphabetic order

		final Collection<Weapon> weaponsDefault = rifleDAO
				.findByFaction("Default");

		CommonService commonService = new CommonService();
		Collection<String> selectedMods = commonService.getSelectedMods();

		List<Weapon> listWeapons = new ArrayList<Weapon>();
		for (Weapon weapon : weaponsFaction) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}
		for (Weapon weapon : weaponsDefault) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}

		if (listWeapons.size() == 0)
			throw new FinderException("faction" + factionName + " has no rifle");

		Collections.sort(listWeapons);

		// Transforms domain objects into DTOs
		final Collection<WeaponDTO> weaponsFactionDTO = transformWeapons2DTOs(listWeapons);

		return weaponsFactionDTO;
	}

	public Collection<MagazineDTO> FindRifleMagazines(String weaponClassname,
			boolean noACE) throws FinderException {

		// Finds all the magazines to a given weapon
		final Collection<String> magazines = rifleDAO
				.findMagazines(weaponClassname);

		if (magazines.size() == 0 || magazines == null)
			throw new FinderException("weapon classname " + weaponClassname
					+ " not found");

		final Collection<Magazine> magazineList = new ArrayList();

		for (String magazineClassname : magazines) {
			Magazine mag = magazineDAO.findByClassname(magazineClassname);
			if (mag == null)
				throw new FinderException("magazine classname "
						+ magazineClassname + " not found");

			if (noACE) {
				if (mag.getAce() == false) {
					magazineList.add(mag);
				}
			} else {
				magazineList.add(mag);
			}

		}

		// Transforms domain objects into DTOs
		final Collection magazinesDTO = transformMagazines2DTOs(magazineList);

		return magazinesDTO;
	}

	public WeaponDTO FindRifleDescription(String weaponClassname)
			throws FinderException {

		// Finds the object
		final Weapon weapon = (Weapon) rifleDAO
				.findByClassname(weaponClassname);

		if (weapon == null)
			throw new FinderException("weapon classname " + weaponClassname
					+ " not found");

		// Transforms domain objects into DTO
		final WeaponDTO weaponDTO = transformWeapon2DTO(weapon);

		return weaponDTO;
	}

	// --- Sniper ---//

	public Collection<WeaponDTO> FindSnipers() throws FinderException {

		// Finds all the objects
		final Collection<Weapon> weapons = sniperDAO.findAll();

		if (weapons == null)
			throw new FinderException("can't find all classnames");

		CommonService commonService = new CommonService();
		Collection<String> selectedMods = commonService.getSelectedMods();

		// Sort weapons in weapon name alphabetic order
		List<Weapon> listWeapons = new ArrayList<Weapon>();
		for (Weapon weapon : weapons) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}
		Collections.sort(listWeapons);

		// Transforms domain objects into DTOs
		final Collection weaponsDTO = transformWeapons2DTOs(listWeapons);

		return weaponsDTO;
	}

	public Collection<WeaponDTO> FindSnipers(String factionName)
			throws FinderException {

		// Finds all the objects
		final Collection<Weapon> weaponsFaction = sniperDAO
				.findByFaction(factionName);

		final Collection<Weapon> weaponsDefault = sniperDAO
				.findByFaction("Default");

		CommonService commonService = new CommonService();
		Collection<String> selectedMods = commonService.getSelectedMods();

		List<Weapon> listWeapons = new ArrayList<Weapon>();
		for (Weapon weapon : weaponsFaction) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}
		for (Weapon weapon : weaponsDefault) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}

		if (listWeapons.size() == 0)
			throw new FinderException("faction" + factionName
					+ " has no sniper");

		Collections.sort(listWeapons);

		// Transforms domain objects into DTOs
		final Collection<WeaponDTO> weaponsFactionDTO = transformWeapons2DTOs(listWeapons);

		return weaponsFactionDTO;
	}

	public Collection<MagazineDTO> FindSniperMagazines(String weaponClassname,
			boolean noACE) throws FinderException {

		// Finds all the magazines to a given weapon
		final Collection<String> magazines = sniperDAO
				.findMagazines(weaponClassname);

		if (magazines.size() == 0 || magazines == null)
			throw new FinderException("weapon classname " + weaponClassname
					+ " not found");

		final Collection<Magazine> magazineList = new ArrayList();

		for (String magazineClassname : magazines) {
			Magazine mag = magazineDAO.findByClassname(magazineClassname);
			if (mag == null)
				throw new FinderException("magazine classname "
						+ magazineClassname + " not found");
			if (noACE) {
				if (mag.getAce() == false) {
					magazineList.add(mag);
				}
			} else {
				magazineList.add(mag);
			}
		}

		// Transforms domain objects into DTOs
		final Collection magazinesDTO = transformMagazines2DTOs(magazineList);

		return magazinesDTO;
	}

	public WeaponDTO FindSniperDescription(String weaponClassname)
			throws FinderException {

		// Finds the object
		final Weapon weapon = (Weapon) sniperDAO
				.findByClassname(weaponClassname);

		if (weapon == null)
			throw new FinderException("weapon classname " + weaponClassname
					+ " not found");

		// Transforms domain objects into DTO
		final WeaponDTO weaponDTO = transformWeapon2DTO(weapon);

		return weaponDTO;
	}

	// ---AR---//

	public Collection<WeaponDTO> FindARs() throws FinderException {

		// Finds all the objects
		final Collection<Weapon> weapons = arDAO.findAll();

		if (weapons == null)
			throw new FinderException("can't find all classnames");

		CommonService commonService = new CommonService();
		Collection<String> selectedMods = commonService.getSelectedMods();

		// Sort weapons in weapon name alphabetic order
		List<Weapon> listWeapons = new ArrayList<Weapon>();
		for (Weapon weapon : weapons) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}
		Collections.sort(listWeapons);

		// Transforms domain objects into DTOs
		final Collection weaponsDTO = transformWeapons2DTOs(listWeapons);

		return weaponsDTO;
	}

	public Collection<WeaponDTO> FindARs(String factionName)
			throws FinderException {

		// Finds all the objects
		final Collection<Weapon> weaponsFaction = arDAO
				.findByFaction(factionName);

		final Collection<Weapon> weaponsDefault = arDAO
				.findByFaction("Default");

		CommonService commonService = new CommonService();
		Collection<String> selectedMods = commonService.getSelectedMods();

		List<Weapon> listWeapons = new ArrayList<Weapon>();
		for (Weapon weapon : weaponsFaction) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}
		for (Weapon weapon : weaponsDefault) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}

		if (listWeapons.size() == 0)
			throw new FinderException("faction" + factionName + " has no AR");

		Collections.sort(listWeapons);

		// Transforms domain objects into DTOs
		final Collection<WeaponDTO> weaponsFactionDTO = transformWeapons2DTOs(listWeapons);

		return weaponsFactionDTO;
	}

	public Collection<MagazineDTO> FindARMagazines(String weaponClassname,
			boolean noACE) throws FinderException {

		// Finds all the magazines to a given weapon
		final Collection<String> magazines = arDAO
				.findMagazines(weaponClassname);

		if (magazines.size() == 0 || magazines == null)
			throw new FinderException("weapon classname " + weaponClassname
					+ " not found");

		final Collection<Magazine> magazineList = new ArrayList();

		for (String magazineClassname : magazines) {
			Magazine mag = magazineDAO.findByClassname(magazineClassname);
			if (mag == null)
				throw new FinderException("magazine classname "
						+ magazineClassname + " not found");

			if (noACE) {
				if (mag.getAce() == false) {
					magazineList.add(mag);
				}
			} else {
				magazineList.add(mag);
			}
		}

		// Transforms domain objects into DTOs
		final Collection magazinesDTO = transformMagazines2DTOs(magazineList);

		return magazinesDTO;
	}

	public WeaponDTO FindARDescription(String weaponClassname)
			throws FinderException {

		// Finds the object
		final Weapon weapon = (Weapon) arDAO.findByClassname(weaponClassname);

		if (weapon == null)
			throw new FinderException("weapon classname " + weaponClassname
					+ " not found");

		// Transforms domain objects into DTO
		final WeaponDTO weaponDTO = transformWeapon2DTO(weapon);

		return weaponDTO;
	}

	// ---MG--//

	public Collection<WeaponDTO> FindMGs() throws FinderException {

		// Finds all the objects
		final Collection<Weapon> weapons = mgDAO.findAll();

		if (weapons == null)
			throw new FinderException("can't find all classnames");

		CommonService commonService = new CommonService();
		Collection<String> selectedMods = commonService.getSelectedMods();

		// Sort weapons in weapon name alphabetic order
		List<Weapon> listWeapons = new ArrayList<Weapon>();
		for (Weapon weapon : weapons) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}
		Collections.sort(listWeapons);

		// Transforms domain objects into DTOs
		final Collection weaponsDTO = transformWeapons2DTOs(listWeapons);

		return weaponsDTO;
	}

	public Collection<WeaponDTO> FindMGs(String factionName)
			throws FinderException {

		// Finds all the objects
		final Collection<Weapon> weaponsFaction = mgDAO
				.findByFaction(factionName);

		final Collection<Weapon> weaponsDefault = mgDAO
				.findByFaction("Default");

		CommonService commonService = new CommonService();
		Collection<String> selectedMods = commonService.getSelectedMods();

		List<Weapon> listWeapons = new ArrayList<Weapon>();
		for (Weapon weapon : weaponsFaction) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}
		for (Weapon weapon : weaponsDefault) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}

		if (listWeapons.size() == 0)
			throw new FinderException("faction" + factionName + " has no MG");

		Collections.sort(listWeapons);

		// Transforms domain objects into DTOs
		final Collection<WeaponDTO> weaponsFactionDTO = transformWeapons2DTOs(listWeapons);

		return weaponsFactionDTO;
	}

	public Collection<MagazineDTO> FindMGMagazines(String weaponClassname)
			throws FinderException {

		// Finds all the magazines to a given weapon
		final Collection<String> magazines = mgDAO
				.findMagazines(weaponClassname);

		if (magazines.size() == 0 || magazines == null)
			throw new FinderException("weapon classname " + weaponClassname
					+ " not found");

		final Collection<Magazine> magazineList = new ArrayList();

		for (String magazineClassname : magazines) {
			Magazine mag = magazineDAO.findByClassname(magazineClassname);
			if (mag == null)
				throw new FinderException("magazine classname "
						+ magazineClassname + " not found");
			magazineList.add(mag);
		}

		// Transforms domain objects into DTOs
		final Collection magazinesDTO = transformMagazines2DTOs(magazineList);

		return magazinesDTO;
	}

	public WeaponDTO FindMGDescription(String weaponClassname)
			throws FinderException {

		// Finds the object
		final Weapon weapon = (Weapon) mgDAO.findByClassname(weaponClassname);

		if (weapon == null)
			throw new FinderException("weapon classname " + weaponClassname
					+ " not found");

		// Transforms domain objects into DTO
		final WeaponDTO weaponDTO = transformWeapon2DTO(weapon);

		return weaponDTO;
	}

	// ---Launcher---//

	public Collection<WeaponDTO> FindLaunchers() throws FinderException {

		// Finds all the objects
		final Collection<Weapon> weapons = launcherDAO.findAll();

		if (weapons == null)
			throw new FinderException("can't find all classnames");

		CommonService commonService = new CommonService();
		Collection<String> selectedMods = commonService.getSelectedMods();

		// Sort weapons in weapon name alphabetic order
		List<Weapon> listWeapons = new ArrayList<Weapon>();
		for (Weapon weapon : weapons) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}
		Collections.sort(listWeapons);

		// Transforms domain objects into DTOs
		final Collection weaponsDTO = transformWeapons2DTOs(listWeapons);

		return weaponsDTO;
	}

	public Collection<WeaponDTO> FindLaunchers(String factionName)
			throws FinderException {

		// Finds all the objects
		final Collection<Weapon> weaponsFaction = launcherDAO
				.findByFaction(factionName);

		final Collection<Weapon> weaponsDefault = launcherDAO
				.findByFaction("Default");

		CommonService commonService = new CommonService();
		Collection<String> selectedMods = commonService.getSelectedMods();

		List<Weapon> listWeapons = new ArrayList<Weapon>();
		for (Weapon weapon : weaponsFaction) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}
		for (Weapon weapon : weaponsDefault) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}

		if (listWeapons.size() == 0)
			throw new FinderException("faction" + factionName
					+ " has no launcher");

		Collections.sort(listWeapons);

		// Transforms domain objects into DTOs
		final Collection<WeaponDTO> weaponsFactionDTO = transformWeapons2DTOs(listWeapons);

		return weaponsFactionDTO;
	}

	public Collection<MagazineDTO> FindLauncherMagazines(
			String weaponClassname, boolean noACE) throws FinderException {

		// Finds all the magazines to a given weapon
		final Collection<String> magazines = launcherDAO
				.findMagazines(weaponClassname);

		if (magazines.size() == 0 || magazines == null)
			throw new FinderException("weapon classname " + weaponClassname
					+ " not found");

		final Collection<Magazine> magazineList = new ArrayList();

		for (String magazineClassname : magazines) {
			Magazine mag = magazineDAO.findByClassname(magazineClassname);
			if (mag == null)
				throw new FinderException("magazine classname "
						+ magazineClassname + " not found");

			if (noACE) {
				if (mag.getAce() == false) {
					magazineList.add(mag);
				}
			} else {
				magazineList.add(mag);
			}
		}

		// Transforms domain objects into DTOs
		final Collection magazinesDTO = transformMagazines2DTOs(magazineList);

		return magazinesDTO;
	}

	public WeaponDTO FindLauncherDescription(String weaponClassname)
			throws FinderException {

		// Finds the object
		final Weapon weapon = (Weapon) launcherDAO
				.findByClassname(weaponClassname);

		if (weapon == null)
			throw new FinderException("weapon classname " + weaponClassname
					+ " not found");

		// Transforms domain objects into DTO
		final WeaponDTO weaponDTO = transformWeapon2DTO(weapon);

		return weaponDTO;
	}

	// ---Pistol---//

	public Collection<WeaponDTO> FindPistols() throws FinderException {

		// Finds all the objects
		final Collection<Weapon> weapons = pistolDAO.findAll();

		if (weapons == null)
			throw new FinderException("can't find all classnames");

		CommonService commonService = new CommonService();
		Collection<String> selectedMods = commonService.getSelectedMods();

		// Sort weapons in weapon name alphabetic order
		List<Weapon> listWeapons = new ArrayList<Weapon>();
		for (Weapon weapon : weapons) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}
		Collections.sort(listWeapons);

		// Transforms domain objects into DTOs
		final Collection weaponsDTO = transformWeapons2DTOs(listWeapons);

		return weaponsDTO;
	}

	public Collection<WeaponDTO> FindPistols(String factionName)
			throws FinderException {

		// Finds all the objects
		final Collection<Weapon> weaponsFaction = pistolDAO
				.findByFaction(factionName);

		final Collection<Weapon> weaponsDefault = pistolDAO
				.findByFaction("Default");

		CommonService commonService = new CommonService();
		Collection<String> selectedMods = commonService.getSelectedMods();

		List<Weapon> listWeapons = new ArrayList<Weapon>();
		for (Weapon weapon : weaponsFaction) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}
		for (Weapon weapon : weaponsDefault) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}

		Collections.sort(listWeapons);

		if (listWeapons.size() == 0)
			throw new FinderException("faction" + factionName
					+ " has no pistol");

		// Transforms domain objects into DTOs
		final Collection<WeaponDTO> weaponsFactionDTO = transformWeapons2DTOs(listWeapons);

		return weaponsFactionDTO;
	}

	public Collection<MagazineDTO> FindPistolMagazines(String weaponClassname, boolean noACE)
			throws FinderException {

		// Finds all the magazines to a given weapon
		final Collection<String> magazines = pistolDAO
				.findMagazines(weaponClassname);

		if (magazines.size() == 0 || magazines == null)
			throw new FinderException("weapon classname " + weaponClassname
					+ " not found");

		final Collection<Magazine> magazineList = new ArrayList();

		for (String magazineClassname : magazines) {
			Magazine mag = magazineDAO.findByClassname(magazineClassname);
			if (mag == null)
				throw new FinderException("magazine classname "
						+ magazineClassname + " not found");
			
			if (noACE) {
				if (mag.getAce() == false) {
					magazineList.add(mag);
				}
			} else {
				magazineList.add(mag);
			}
		}

		// Transforms domain objects into DTOs
		final Collection magazinesDTO = transformMagazines2DTOs(magazineList);

		return magazinesDTO;
	}

	public WeaponDTO FindPistolDescription(String weaponClassname)
			throws FinderException {

		// Finds the object
		final Weapon weapon = (Weapon) pistolDAO
				.findByClassname(weaponClassname);

		if (weapon == null)
			throw new FinderException("weapon classname " + weaponClassname
					+ " not found");

		// Transforms domain objects into DTO
		final WeaponDTO weaponDTO = transformWeapon2DTO(weapon);

		return weaponDTO;
	}

	// ---Optical---//

	public Collection<WeaponDTO> FindEquips() throws FinderException {

		// Finds all the objects
		final Collection<Weapon> weapons = equipDAO.findAll();

		if (weapons == null)
			throw new FinderException("can't find all classnames");

		CommonService commonService = new CommonService();
		Collection<String> selectedMods = commonService.getSelectedMods();

		// Sort weapons in weapon name alphabetic order
		List<Weapon> listWeapons = new ArrayList<Weapon>();
		for (Weapon weapon : weapons) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}
		Collections.sort(listWeapons);

		// Transforms domain objects into DTOs
		final Collection weaponsDTO = transformWeapons2DTOs(listWeapons);

		return weaponsDTO;
	}

	public Collection<WeaponDTO> FindEquips(String factionName)
			throws FinderException {

		// Finds all the objects
		final Collection<Weapon> weaponsFaction = equipDAO
				.findByFaction(factionName);

		final Collection<Weapon> weaponsDefault = equipDAO
				.findByFaction("Default");

		CommonService commonService = new CommonService();
		Collection<String> selectedMods = commonService.getSelectedMods();

		List<Weapon> listWeapons = new ArrayList<Weapon>();
		for (Weapon weapon : weaponsFaction) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}
		for (Weapon weapon : weaponsDefault) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}

		if (listWeapons.size() == 0)
			throw new FinderException("faction" + factionName + " has no equip");

		Collections.sort(listWeapons);

		// Transforms domain objects into DTOs
		final Collection<WeaponDTO> weaponsDefaultDTO = transformWeapons2DTOs(listWeapons);

		return weaponsDefaultDTO;

	}

	public WeaponDTO FindEquipDescription(String weaponClassname)
			throws FinderException {

		// Finds the object
		final Weapon weapon = (Weapon) equipDAO
				.findByClassname(weaponClassname);

		if (weapon == null)
			throw new FinderException("weapon classname " + weaponClassname
					+ " not found");

		// Transforms domain objects into DTO
		final WeaponDTO weaponDTO = transformWeapon2DTO(weapon);

		return weaponDTO;
	}

	public Collection<MagazineDTO> FindEquipMagazines(String weaponClassname, boolean noACE)
			throws FinderException {

		// Finds all the magazines to a given weapon
		final Collection<String> magazines = equipDAO
				.findMagazines(weaponClassname);

		if (magazines.size() == 0 || magazines == null)
			throw new FinderException("weapon classname " + weaponClassname
					+ " not found");

		final Collection<Magazine> magazineList = new ArrayList();

		for (String magazineClassname : magazines) {
			Magazine mag = magazineDAO.findByClassname(magazineClassname);
			if (mag == null)
				throw new FinderException("magazine classname "
						+ magazineClassname + " not found");
			
			if (noACE) {
				if (mag.getAce() == false) {
					magazineList.add(mag);
				}
			} else {
				magazineList.add(mag);
			}
		}

		// Transforms domain objects into DTOs
		final Collection magazinesDTO = transformMagazines2DTOs(magazineList);

		return magazinesDTO;
	}

	// ---Item---//

	public Collection<WeaponDTO> FindItems() throws FinderException {

		// Finds all the objects
		final Collection<Weapon> weapons = itemDAO.findAll();

		if (weapons == null)
			throw new FinderException("can't find all classnames");

		CommonService commonService = new CommonService();
		Collection<String> selectedMods = commonService.getSelectedMods();

		// Sort weapons in weapon name alphabetic order
		List<Weapon> listWeapons = new ArrayList<Weapon>();
		for (Weapon weapon : weapons) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}
		Collections.sort(listWeapons);

		// Transforms domain objects into DTOs
		final Collection weaponsDTO = transformWeapons2DTOs(listWeapons);

		return weaponsDTO;
	}

	public Collection<WeaponDTO> FindItems(String factionName)
			throws FinderException {

		// Finds all the objects, faction is default only
		final Collection<Weapon> weaponsDefault = itemDAO
				.findByFaction("Default");

		if (weaponsDefault.size() == 0 || weaponsDefault == null)
			throw new FinderException("faction default not found");

		CommonService commonService = new CommonService();
		Collection<String> selectedMods = commonService.getSelectedMods();

		// Sort weapons in weapon name alphabetic order
		List<Weapon> listWeapons = new ArrayList<Weapon>();
		for (Weapon weapon : weaponsDefault) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}
		Collections.sort(listWeapons);

		// Transforms domain objects into DTOs
		final Collection<WeaponDTO> weaponsDefaultDTO = transformWeapons2DTOs(listWeapons);

		return weaponsDefaultDTO;
	}

	public WeaponDTO FindItemDescription(String weaponClassname)
			throws FinderException {

		// Finds the object
		final Weapon weapon = (Weapon) itemDAO.findByClassname(weaponClassname);

		if (weapon == null)
			throw new FinderException("weapon classname " + weaponClassname
					+ " not found");

		// Transforms domain objects into DTO
		final WeaponDTO weaponDTO = transformWeapon2DTO(weapon);

		return weaponDTO;
	}

	// ---Ruck---//

	public Collection<WeaponDTO> FindRucks() throws FinderException {

		// Finds all the objects
		final Collection<Weapon> weapons = ruckDAO.findAll();

		if (weapons == null)
			throw new FinderException("can't find all classnames");

		CommonService commonService = new CommonService();
		Collection<String> selectedMods = commonService.getSelectedMods();

		// Sort weapons in weapon name alphabetic order
		List<Weapon> listWeapons = new ArrayList<Weapon>();
		for (Weapon weapon : weapons) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}
		Collections.sort(listWeapons);

		// Transforms domain objects into DTOs
		final Collection weaponsDTO = transformWeapons2DTOs(listWeapons);

		return weaponsDTO;
	}

	public Collection<WeaponDTO> FindRucks(String factionName)
			throws FinderException {

		// Finds all the objects
		final Collection<Weapon> weaponsFaction = ruckDAO
				.findByFaction(factionName);

		final Collection<Weapon> weaponsDefault = ruckDAO
				.findByFaction("Default");

		CommonService commonService = new CommonService();
		Collection<String> selectedMods = commonService.getSelectedMods();

		List<Weapon> listWeapons = new ArrayList<Weapon>();
		for (Weapon weapon : weaponsFaction) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}
		for (Weapon weapon : weaponsDefault) {
			if (selectedMods.contains(weapon.getMod().toUpperCase())
					|| weapon.getMod().equals("")) {
				listWeapons.add(weapon);
			}
		}

		if (listWeapons.size() == 0)
			throw new FinderException("faction" + factionName + " has no ruck");

		Collections.sort(listWeapons);

		// Transforms domain objects into DTOs
		final Collection<WeaponDTO> weaponsFactionDTO = transformWeapons2DTOs(listWeapons);

		return weaponsFactionDTO;
	}

	public WeaponDTO FindRuckDescription(String weaponClassname)
			throws FinderException {

		// Finds the object
		final Weapon weapon = (Weapon) ruckDAO.findByClassname(weaponClassname);

		if (weapon == null)
			throw new FinderException("weapon classname " + weaponClassname
					+ " not found");

		// Transforms domain objects into DTO
		final WeaponDTO weaponDTO = transformWeapon2DTO(weapon);

		return weaponDTO;
	}

	// --- Factions ---//

	public Collection<String> FindFactions() throws FinderException {

		Collection<String> selectedMods = new ArrayList<String>();

		String listMods = commonDAO.getMods();
		StringTokenizer stringTokenizer = new StringTokenizer(listMods, ",");
		while (stringTokenizer.hasMoreTokens()) {
			selectedMods.add(stringTokenizer.nextToken());
		}

		// Finds the objects
		Collection<String> listFactions = weaponDAO.findFactions(selectedMods);

		if (listFactions.size() == 0 || listFactions == null)
			throw new FinderException("can't find factions");

		return listFactions;
	}

	// --- Weapons ---//

	public WeaponDTO FindDescription(String weaponClassname)
			throws FinderException {

		// Finds the object
		final Weapon weapon = (Weapon) weaponDAO
				.findByClassname(weaponClassname);

		if (weapon == null)
			throw new FinderException("weapon classname " + weaponClassname
					+ " not found");

		// Transforms domain objects into DTO
		final WeaponDTO weaponDTO = transformWeapon2DTO(weapon);

		return weaponDTO;
	}

	public WeakReference<ImageIcon> FindWeaponImageIcon(String weaponClassname) {
		// Finds the object
		String installPath = commonDAO.getInstallationPath();

		return weaponDAO.findImageIcon(installPath, weaponClassname);
	}

	public WeakReference<ImageIcon> FindWeaponImageIconInIconesFolder(
			String weaponClassname) throws FinderException {
		String installPath = commonDAO.getInstallationPath();
		return weaponDAO.findImageIconInIconesFolder(installPath,
				weaponClassname);
	}

	public int readWeaponsClasslist() {

		int numberWeapons = weaponDAO.readWeaponsClasslist(commonDAO
				.getInstallationPath());
		return numberWeapons;
	}

	public void loadWeapons() {
		weaponDAO.createDBWeapons();
	}

	public AbstractWeaponDAO getWeaponDAO() {
		return weaponDAO;
	}

	public Set<String> getAvailableMods() {
		return weaponDAO.getAvailableMods();
	}

	// Private business methods

	private Collection transformWeapons2DTOs(final Collection weapons) {
		final Collection weaponsDTO = new ArrayList();
		for (Iterator iterator = weapons.iterator(); iterator.hasNext();) {
			final Weapon weapon = (Weapon) iterator.next();
			weaponsDTO.add(transformWeapon2DTO(weapon));
		}
		return weaponsDTO;
	}

	private WeaponDTO transformWeapon2DTO(Weapon weapon) {
		final WeaponDTO weaponDTO = new WeaponDTO(weapon.getClassname());
		weaponDTO.setName(weapon.getName());
		weaponDTO.setAce(weapon.getAce());
		weaponDTO.setAceWeight(weapon.getAceWeight());
		weaponDTO.setCaliber(weapon.getCaliber());
		weaponDTO.setMod(weapon.getMod());
		weaponDTO.setType(weapon.getType());
		return weaponDTO;
	}

	private Collection transformMagazines2DTOs(final Collection magazines) {
		final Collection magazinesDTO = new ArrayList();
		for (Iterator iterator = magazines.iterator(); iterator.hasNext();) {
			final Magazine magazine = (Magazine) iterator.next();
			magazinesDTO.add(transformMagazine2DTO(magazine));
		}
		return magazinesDTO;
	}

	private MagazineDTO transformMagazine2DTO(Magazine magazine) {
		final MagazineDTO magazineDTO = new MagazineDTO(magazine.getClassname());
		magazineDTO.setName(magazine.getName());
		magazineDTO.setAceWeight(magazine.getAceWeight());
		magazineDTO.setCaliber(magazine.getCaliber());
		magazineDTO.setAce(magazine.getAce());
		magazineDTO.setType(magazine.getType());
		magazineDTO.setCategory(magazine.getCategory());
		magazineDTO.setSlot(magazine.getSlot());
		magazineDTO.setMod(magazine.getMod());
		return magazineDTO;
	}

}
