package com.lea.ui;

import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Collection;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;

import com.lea.dto.MagazineDTO;
import com.lea.dto.WeaponDTO;
import com.lea.exception.FinderException;
import com.lea.service.MagazineService;
import com.lea.service.MissionService;
import com.lea.service.WeaponService;

public class PreferencesPanel extends JDialog {

	private JCheckBox checkBoxGPS;
	private JCheckBox checkBoxRadio;
	private JButton buttonOK;
	private JCheckBox checkBoxBandage;
	private JCheckBox checkBoxMedkit;
	private JCheckBox checkBoxEpinephrine;
	private JCheckBox checkBoxMorphine;
	private JCheckBox checkBoxNVG;
	private JCheckBox checkBoxGoggles;
	private JCheckBox checkBoxWatch;
	private JCheckBox checkBoxCompass;
	private JCheckBox checkBoxMap;
	private JCheckBox checkBoxEarPlugs;
	private Collection<WeaponDTO> listItemsPreferences = new ArrayList<WeaponDTO>();
	private Collection<WeaponDTO> listOpticalsPreferences = new ArrayList<WeaponDTO>();
	private Collection<MagazineDTO> listEquipPreferences = new ArrayList<MagazineDTO>();
	private Facade facade;

	/**
	 * Preferences dialog.
	 * 
	 */

	public PreferencesPanel(Facade facade) {
		super(facade.getMissionEditor(), "Preferences", true);
		this.facade = facade;
		setResizable(false);
		this.setLayout(null);
		this.setResizable(false);
		this.setSize(260, 220);
		Image icone = Toolkit
				.getDefaultToolkit()
				.getImage(
						java.lang.ClassLoader
								.getSystemResource("resources/pictures/system/lea.png"));
		setIconImage(icone);

		this.setLocation((int) facade.getMissionEditor().getLocation().getX()
				+ facade.getMissionEditor().getWidth() / 2 - this.getWidth()
				/ 2, (int) facade.getMissionEditor().getLocation().getY()
				+ facade.getMissionEditor().getHeight() / 2 - this.getHeight()
				/ 2);

		getContentPane().setLayout(null);
		{
			checkBoxGPS = new JCheckBox();
			getContentPane().add(checkBoxGPS);
			checkBoxGPS.setText("GPS");
			checkBoxGPS.setBounds(85, 83, 55, 26);
		}
		{
			checkBoxRadio = new JCheckBox();
			getContentPane().add(checkBoxRadio);
			checkBoxRadio.setText("PRC343");
			checkBoxRadio.setBounds(13, 85, 72, 21);
		}
		{
			checkBoxEarPlugs = new JCheckBox();
			getContentPane().add(checkBoxEarPlugs);
			checkBoxEarPlugs.setText("Ear Plugs");
			checkBoxEarPlugs.setBounds(13, 22, 73, 33);
		}
		{
			checkBoxMap = new JCheckBox();
			getContentPane().add(checkBoxMap);
			checkBoxMap.setText("Map");
			checkBoxMap.setBounds(85, 55, 64, 25);
		}
		{
			checkBoxCompass = new JCheckBox();
			getContentPane().add(checkBoxCompass);
			checkBoxCompass.setText("Compass");
			checkBoxCompass.setBounds(85, 25, 70, 27);
		}
		{
			checkBoxWatch = new JCheckBox();
			getContentPane().add(checkBoxWatch);
			checkBoxWatch.setText("Watch");
			checkBoxWatch.setBounds(13, 56, 59, 22);
		}
		{
			buttonOK = new JButton();
			getContentPane().add(buttonOK);
			buttonOK.setText("OK");
			buttonOK.setBounds(85, 149, 82, 28);
		}
		{
			checkBoxGoggles = new JCheckBox();
			getContentPane().add(checkBoxGoggles);
			checkBoxGoggles.setText("Goggles");
			checkBoxGoggles.setBounds(13, 109, 68, 27);
		}
		{
			checkBoxNVG = new JCheckBox();
			getContentPane().add(checkBoxNVG);
			checkBoxNVG.setText("NVG");
			checkBoxNVG.setBounds(85, 108, 56, 28);
		}
		{
			checkBoxBandage = new JCheckBox();
			getContentPane().add(checkBoxBandage);
			checkBoxBandage.setText("Bandage");
			checkBoxBandage.setBounds(153, 28, 77, 21);
		}
		{
			checkBoxMorphine = new JCheckBox();
			getContentPane().add(checkBoxMorphine);
			checkBoxMorphine.setText("Morphine");
			checkBoxMorphine.setBounds(153, 55, 73, 25);
		}
		{
			checkBoxEpinephrine = new JCheckBox();
			getContentPane().add(checkBoxEpinephrine);
			checkBoxEpinephrine.setText("Epinephrine");
			checkBoxEpinephrine.setBounds(153, 85, 90, 23);
		}
		{
			checkBoxMedkit = new JCheckBox();
			getContentPane().add(checkBoxMedkit);
			checkBoxMedkit.setText("Medkit");
			checkBoxMedkit.setBounds(153, 112, 58, 20);
		}

		buttonOK.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				buttonOKPerformed();
			}
		});
		// Add Listeners
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				menuExitPerformed();
			}
		});
		init();
	}

	private void init() {

		MissionService missionService = new MissionService();

		try {
			listItemsPreferences = missionService
					.loadMissionListItemsPreferences();

			for (WeaponDTO weaponDTO : listItemsPreferences) {
				String classname = weaponDTO.getClassname();
				if (classname.equals("ACRE_PRC343"))
					checkBoxRadio.setSelected(true);
				else if (classname.equals("ItemGPS"))
					checkBoxGPS.setSelected(true);
				else if (classname.equals("ACE_Earplugs"))
					checkBoxEarPlugs.setSelected(true);
				else if (classname.equals("ItemMap"))
					checkBoxMap.setSelected(true);
				else if (classname.equals("ItemCompass"))
					checkBoxCompass.setSelected(true);
				else if (classname.equals("ItemWatch"))
					checkBoxWatch.setSelected(true);
			}
		} catch (FinderException e) {
		}

		try {
			listOpticalsPreferences = missionService
					.loadMissionListOpticalsPreferences();

			for (WeaponDTO weaponDTO : listOpticalsPreferences) {
				String classname = weaponDTO.getClassname();
				if (classname.equals("NVGoggles"))
					checkBoxNVG.setSelected(true);
				else if (classname.equals("Binocular"))
					checkBoxGoggles.setSelected(true);
			}
		} catch (FinderException e) {
		}

		try {

			listEquipPreferences = missionService
					.loadMissionListEquipPreferences();

			for (MagazineDTO magazineDTO : listEquipPreferences) {
				String classname = magazineDTO.getClassname();
				if (classname.equals("ACE_Medkit"))
					checkBoxMedkit.setSelected(true);
				else if (classname.equals("ACE_Bandage"))
					checkBoxBandage.setSelected(true);
				else if (classname.equals("ACE_Morphine"))
					checkBoxMorphine.setSelected(true);
				else if (classname.equals("ACE_Epinephrine"))
					checkBoxEpinephrine.setSelected(true);
			}

		} catch (FinderException e) {
		}

		if (facade.isNoACE()) {
			checkBoxEarPlugs.setEnabled(false);
			checkBoxEarPlugs.setSelected(false);
			checkBoxMedkit.setEnabled(false);
			checkBoxMedkit.setSelected(false);
			checkBoxBandage.setEnabled(false);
			checkBoxBandage.setSelected(false);
			checkBoxMorphine.setEnabled(false);
			checkBoxMorphine.setSelected(false);
			checkBoxEpinephrine.setEnabled(false);
			checkBoxEpinephrine.setSelected(false);
		}
		if (facade.isNoACRE()) {
			checkBoxRadio.setEnabled(false);
			checkBoxRadio.setSelected(false);
		}
	}

	public void buttonOKPerformed() {

		WeaponService weaponService = new WeaponService();
		MagazineService magazineService = new MagazineService();
		listItemsPreferences.clear();
		listOpticalsPreferences.clear();
		listEquipPreferences.clear();
		try {
			if (checkBoxGPS.isSelected()) {
				WeaponDTO weaponDTO = weaponService.FindDescription("ItemGPS");
				listItemsPreferences.add(weaponDTO);
			}
			if (checkBoxRadio.isSelected()) {
				WeaponDTO weaponDTO = weaponService
						.FindDescription("ACRE_PRC343");
				listItemsPreferences.add(weaponDTO);
			}
			if (checkBoxEarPlugs.isSelected()) {
				WeaponDTO weaponDTO = weaponService
						.FindDescription("ACE_Earplugs");
				listItemsPreferences.add(weaponDTO);
			}
			if (checkBoxMap.isSelected()) {
				WeaponDTO weaponDTO = weaponService.FindDescription("ItemMap");
				listItemsPreferences.add(weaponDTO);
			}
			if (checkBoxCompass.isSelected()) {
				WeaponDTO weaponDTO = weaponService
						.FindDescription("ItemCompass");
				listItemsPreferences.add(weaponDTO);
			}
			if (checkBoxWatch.isSelected()) {
				WeaponDTO weaponDTO = weaponService
						.FindDescription("ItemWatch");
				listItemsPreferences.add(weaponDTO);
			}
			if (checkBoxNVG.isSelected()) {
				WeaponDTO weaponDTO = weaponService
						.FindDescription("NVGoggles");
				listOpticalsPreferences.add(weaponDTO);
			}
			if (checkBoxGoggles.isSelected()) {
				WeaponDTO weaponDTO = weaponService
						.FindDescription("Binocular");
				listOpticalsPreferences.add(weaponDTO);
			}
			if (checkBoxBandage.isSelected()) {
				MagazineDTO magazineDTO = magazineService
						.FindMagazineDescription("ACE_Bandage");
				listEquipPreferences.add(magazineDTO);
				listEquipPreferences.add(magazineDTO);
			}
			if (checkBoxMorphine.isSelected()) {
				MagazineDTO magazineDTO = magazineService
						.FindMagazineDescription("ACE_Morphine");
				listEquipPreferences.add(magazineDTO);
				listEquipPreferences.add(magazineDTO);
			}
			if (checkBoxEpinephrine.isSelected()) {
				MagazineDTO magazineDTO = magazineService
						.FindMagazineDescription("ACE_Epinephrine");
				listEquipPreferences.add(magazineDTO);
				listEquipPreferences.add(magazineDTO);
			}
			if (checkBoxMedkit.isSelected()) {
				MagazineDTO magazineDTO = magazineService
						.FindMagazineDescription("ACE_Medkit");
				listEquipPreferences.add(magazineDTO);
				listEquipPreferences.add(magazineDTO);
			}

			MissionService missionService = new MissionService();
			missionService
					.saveMissionListItemsPreferences(listItemsPreferences);
			missionService
					.saveMissionListOpticalsPreferences(listOpticalsPreferences);
			missionService
					.saveMissionListEquipPreferences(listEquipPreferences);

			facade.getMissionEditor().loadUnitLoadout();

		} catch (Exception e) {
			// error
			e.printStackTrace();
		} finally {
			dispose();
			facade.getMissionEditor().setEnabled(true);
			facade.getMissionEditor().toFront();
		}
	}

	private void menuExitPerformed() {
		this.dispose();
	}
}
