package com.lea.ui.ftp.filemanager;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.io.File;
import java.text.DateFormat;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.EtchedBorder;

import org.apache.commons.net.ftp.FTPFile;


/**
 * @author Major_Shepard for the =GiFr= team
 */

public class DetailViewFtp extends JPanel implements LanguageAware {

	private JPanel dataPanel;
	private JPanel labelPanel;
	private JPanel fieldPanel;
	private JPanel statusPanel;

	private JLabel nameLabel;
	private JLabel sizeLabel;
	private JLabel dateLabel;
	private JLabel attribLabel;
	private JLabel statusLabel;

	private JTextField nameField;
	private JTextField sizeField;
	private JTextField dateField;
	private JTextField attribField;

	private JButton execButton;

	private EtchedBorder statusPanelBorder;
	private Font font;

	private LanguageData ld;

	private File file;

	public DetailViewFtp() {

		ld = new LanguageData();
		// Initialize the components
		nameLabel = new JLabel(ld.get(LanguageData.L_NAME) + ": ");
		nameLabel.setFont(font);
		nameField = new JTextField();
		nameField.setEditable(false);
		nameField.setBackground(Color.white);
		nameField.setFont(font);

		sizeLabel = new JLabel(ld.get(LanguageData.L_SIZE) + ": ");
		sizeLabel.setFont(font);
		sizeField = new JTextField();
		sizeField.setEditable(false);
		sizeField.setBackground(Color.white);
		sizeField.setFont(font);
		sizeField.setHorizontalAlignment(SwingConstants.RIGHT);

		dateLabel = new JLabel(ld.get(LanguageData.L_DATE) + ": ");
		dateLabel.setFont(font);
		dateField = new JTextField();
		dateField.setEditable(false);
		dateField.setBackground(Color.white);
		dateField.setFont(font);
		dateField.setHorizontalAlignment(SwingConstants.RIGHT);

		attribLabel = new JLabel(ld.get(LanguageData.L_ATTRIB) + ": ");
		attribLabel.setFont(font);
		attribField = new JTextField();
		attribField.setEditable(false);
		attribField.setBackground(Color.white);
		attribField.setFont(font);

		statusPanelBorder = new EtchedBorder(1);
		statusLabel = new JLabel(" ");
		statusLabel.setFont(font);

		// Set up the panels
		// 1. the labels
		labelPanel = new JPanel(new GridLayout(4, 1, 0, 0));
		labelPanel.add(nameLabel, null);
		labelPanel.add(sizeLabel, null);
		labelPanel.add(dateLabel, null);
		labelPanel.add(attribLabel, null);

		// 2. the fields
		fieldPanel = new JPanel(new GridLayout(4, 1, 0, 0));
		fieldPanel.add(nameField, null);
		fieldPanel.add(sizeField, null);
		fieldPanel.add(dateField, null);
		fieldPanel.add(attribField, null);

		// 3. the status
		statusPanel = new JPanel(new BorderLayout());
		statusPanel.setBorder(statusPanelBorder);
		statusPanel.add(statusLabel, BorderLayout.WEST);

		// 4. the containers
		dataPanel = new JPanel(new BorderLayout());
		dataPanel.add(labelPanel, BorderLayout.WEST);
		dataPanel.add(fieldPanel, BorderLayout.CENTER);

		this.setLayout(new BorderLayout());
		this.add(dataPanel, BorderLayout.CENTER);
		this.add(statusPanel, BorderLayout.SOUTH);
		
	}

	public boolean openExternalWindow() {
		boolean retVal = true;
		if (file != null && file.isFile()) {
			switch (ExtendedFile.getExtensionType(file)) {
			case ExtendedFile.ZIP:
				ZipListWindow zlw = new ZipListWindow(file);
				break;
			case ExtendedFile.HTML:
				HTMLWindow htmlw = new HTMLWindow(file);
				break;
			case ExtendedFile.IMAGE:
				ImageWindow imw = new ImageWindow(file);
				break;
			case ExtendedFile.PLAIN:
				PlainTextWindow ptw = new PlainTextWindow(file);
				break;
			default:
				retVal = false;
				break;
			}
		} else {
			retVal = false;
		}
		return retVal;
	}

	public boolean showInfo(FTPFile ftpfile) {
		this.nameField.setText(ftpfile.getName());
		this.nameField.setCaretPosition(JTextField.LEFT);
		this.sizeField.setText(ExtendedFileFtp.getBeautifulFileSize(ftpfile));
		DateFormat df = DateFormat.getDateTimeInstance(DateFormat.MEDIUM,
				DateFormat.MEDIUM);
		this.dateField.setText(df.format(ftpfile.getTimestamp().getTime())); 
		this.attribField.setText((ftpfile.isDirectory() ? "D" : "-")
				+ (ftpfile.hasPermission(ftpfile.USER_ACCESS,
						ftpfile.READ_PERMISSION) ? "R" : "-")
				+ (ftpfile.hasPermission(ftpfile.USER_ACCESS,
						ftpfile.WRITE_PERMISSION) ? "W" : "-") + ("-"));
		return true;
	}

	public int getPreferredHeight() {
		return nameField.getPreferredSize().height
				+ sizeField.getPreferredSize().height
				+ dateField.getPreferredSize().height
				+ attribField.getPreferredSize().height
				+ statusLabel.getPreferredSize().height
				+ ((System.getProperty("os.name").toLowerCase().indexOf("win") > 0) ? 20
						: 5);
	}


	public void setStatusLabel(String path) {
		statusLabel.setText(path);
	}

	public void changeLanguage(LanguageData l) {
		nameLabel.setText(l.get(l.L_NAME) + ": ");
		sizeLabel.setText(l.get(l.L_SIZE) + ": ");
		dateLabel.setText(l.get(l.L_DATE) + ": ");
		attribLabel.setText(l.get(l.L_ATTRIB) + ": ");
	}
}
