/**
 * Title:        JFileManager<p>
 * Description:  <p>
 * Copyright:    Copyright (c) Thorsten Jens, Kai Benjamins<p>
 * Company:      <p>
 * @author Thorsten Jens, Kai Benjamins
 * @version 1.0
 */
package com.lea.ui.ftp.filemanager;

import java.math.BigDecimal;
import java.text.NumberFormat;

import javax.swing.ImageIcon;

import org.apache.commons.net.ftp.FTPFile;

/**
 * @author Major_Shepard for the =GiFr= team
 */

public class ExtendedFileFtp {

	private static final int KB = 1024;
	private static final int MB = KB * KB;

	/** Datei ist eine ZIP- oder JAR-Datei */
	public static final int ZIP = 1;
	/** Datei ist eine HTML-Datei */
	public static final int HTML = 2;
	/** Datei ist eine von Java direkt untersttzte Bilddatei (jpg, gif) */
	public static final int IMAGE = 3;
	/** Datei ist eine Nur-Text-Datei (ASCII-Text, Sourcecode, etc.) */
	public static final int PLAIN = 4;
	/** Datei ist keine der untersttzen Typen */
	public static final int OTHER = -1;

	public static final int LEA = 5;
	public static final int PBO = 6;

	/** Die Icons fr die entsprechenden Dateitypen. */
	public static final ImageIcon[] icons = {
			new ImageIcon(MainWindow.class.getResource("folder.gif")), // Ordner
			new ImageIcon(MainWindow.class.getResource("xfm_z.gif")), // ZIP
			new ImageIcon(MainWindow.class.getResource("html.gif")), // HTML
			new ImageIcon(MainWindow.class.getResource("image.gif")), // IMAGE
			new ImageIcon(MainWindow.class.getResource("plain.gif")), // PLAIN
			new ImageIcon(MainWindow.class.getResource("lea3.gif")), // LEA
			new ImageIcon(MainWindow.class.getResource("pbo2.gif")) // PBO
	};

	/**
	 * Leerer Konstruktor, da keine Instanzenmethoden
	 */
	public ExtendedFileFtp() {
	}

	/**
	 * Liefert den (hintersten) Dateisuffix einer Datei.
	 * 
	 * @return Dateisuffix der Datei, ohne fhrenden Punkt (Bsp.: "<b>zip</b>",
	 *         "<b>html</b>"
	 * @param f
	 *            Die zu untersuchende Datei.
	 * @see #getExtensionType
	 * 
	 */
	public static String getExtensionString(FTPFile ftpFile) {
		String tmp = ftpFile.getName();
		return tmp.substring(tmp.lastIndexOf(".") + 1, tmp.length());
	}

	public static int getExtensionType(FTPFile ftpFile) {
		int retVal = OTHER;
		if (getExtensionString(ftpFile).contains("mission")
				|| getExtensionString(ftpFile).contains("profile")) {
			retVal = LEA;
		} else if (getExtensionString(ftpFile).contains("pbo"))
			retVal = PBO;
		return retVal;
	}

	/**
	 * Liefert eine "schne" Darstellung der Gre einer Datei.
	 * 
	 * @params f Die zu untersuchende Datei
	 * @returns Ein String, der die Dateigre in Bytes, und je nach Gre der
	 *          Datei, ihre Gre in Kilobytes oder Megabytes in Klammern.
	 */
	public static String getBeautifulFileSize(FTPFile f) {
		String retVal = new String();
		long size = f.getSize();
		BigDecimal bd;
		double s = size;
		NumberFormat nf = NumberFormat.getNumberInstance();
		retVal = nf.format(size) + " Bytes"; // Bytes werden immer dargestellt

		// Jetzt wird entschieden, ob zustzlich MB oder KB dargestellt wird.
		if (size > MB) {
			bd = new BigDecimal(s / MB);
			bd = bd.setScale(2, BigDecimal.ROUND_HALF_UP);
			retVal = retVal + " (" + bd.toString() + " MB)";
		} else if (size > KB) {
			bd = new BigDecimal(s / KB);
			bd = bd.setScale(2, BigDecimal.ROUND_HALF_UP);
			retVal = retVal + " (" + bd.toString() + " KB)";
		}
		return retVal;
	};

	/**
	 * @params f Die zu untersuchende Datei
	 * @returns <code>true</code>, wenn die Datei anzeigbar ist, sonst
	 *          <code>false</code>.
	 */
	static public boolean isViewable(FTPFile ftpFile) {
		return (getExtensionType(ftpFile) > 0);
	};

	/**
	 * Liefert das Icon (16x16) zu einer Datei. Zur Verwendung in der FileTable.
	 * 
	 * @see FileTable
	 * @params f Die Datei, fr die das Icon gesucht wird.
	 * @returns Gibt das passende Icon zurck oder <code>null</code>, falls die
	 *          Datei nicht untersttzt wird.
	 */
	static public ImageIcon getIcon(FTPFile ftpFile) {
		if (ftpFile.isDirectory()) {
			return icons[0];
		} else {
			int type = getExtensionType(ftpFile);
			return (type > 0) ? icons[type] : null;
		}
	}

}
