/**
 * Title:        JFileManager<p>
 * Description:  <p>
 * Copyright:    Copyright (c) Thorsten Jens, Kai Benjamins<p>
 * Company:      <p>
 * @author Thorsten Jens, Kai Benjamins
 * @version 1.0
 */
package com.lea.ui.ftp.filemanager;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.IOException;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.TableColumn;

import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;

import com.lea.exception.FtpException;
import com.lea.service.FtpService;



/**
 * @author Major_Shepard for the =GiFr= team
 */

public class FileTableFtp extends JPanel implements LanguageAware {
	String path;
	JScrollPane fileScrollPane;
	File[] rootdirs;
	FileTableModelFtp Model;
	TableColumn column;
	ListSelectionModel rowSM;
	JTable table;
	DetailViewFtp dv;
	int selectedRow;
	Vector selectedFiles;
	LanguageData ld;
	private String remotePath;
	private String homePath;

	public FileTableFtp(DetailViewFtp rightDetailView, LanguageData ld,
			String remotePath) {
		this.dv = rightDetailView;
		this.ld = ld;
		this.remotePath = remotePath;
		this.homePath = remotePath;
		this.initializeFileTable();
	}

	public void initializeFileTable() {

		FTPFile[] ftpFiles = null;
		try {
			FtpService ftpService = new FtpService();
			FTPClient ftpClient = ftpService.getFtpClient();
			ftpClient.changeWorkingDirectory(homePath);
			remotePath = homePath;
			ftpFiles = ftpClient.listFiles();
			Model = new FileTableModelFtp(ftpFiles, this.dv, ld);
		}

		catch (IOException e) {
			e.printStackTrace();
		}

		table = new JTable(Model);
		selectedFiles = new Vector();

		table.getColumnModel().getColumn(0).setPreferredWidth(18);
		table.getColumnModel().getColumn(1).setPreferredWidth(1000);
		table.getColumnModel().getColumn(2).setPreferredWidth(200);
		table.setShowGrid(false);
		table.setFont(new Font("Thoama", Font.PLAIN, 11));
		fileScrollPane = new JScrollPane(table);
		this.setLayout(new BorderLayout());
		this.add(fileScrollPane, BorderLayout.CENTER);

		table.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		rowSM = table.getSelectionModel();
		rowSM.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				selectedFiles.removeAllElements();
				if (e.getValueIsAdjusting()) {
					return;
				}
				ListSelectionModel lsm = (ListSelectionModel) e.getSource();
				if (lsm.isSelectionEmpty()) {
					return;
				} else {
					int min_index = lsm.getMinSelectionIndex();
					int max_index = lsm.getMaxSelectionIndex();
					for (int i = min_index; i <= max_index; i++) {
						if (lsm.isSelectedIndex(i)) {
							// if (Model.getSelectedFile(i).isFile())
							selectedFiles.addElement(Model.getSelectedFile(i));
						}
					}
				}
			}
		});

		table.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					dv.openExternalWindow();
					changeDirectory(table.rowAtPoint(e.getPoint()));
				} else {
					Model.showFileOrDirectoryInfo(table
							.rowAtPoint(e.getPoint()));
					selectedRow = table.rowAtPoint(e.getPoint());
				}
			}
		});
	}

	public String returnHomePath() {
		return homePath;
	}

	public void setHomeTable() {
		try {
			FtpService ftpService = new FtpService();
			FTPClient ftpClient = ftpService.getFtpClient();
			try {
				ftpClient.disconnect();
			} catch (Exception e) {
				// e.printStackTrace();
			}
			ftpClient = ftpService.reconnect();

			ftpClient.changeWorkingDirectory(homePath);
			remotePath = homePath;
			FTPFile[] ftpFiles = ftpClient.listFiles();
			Model.changeFileTable(ftpFiles);
		} catch (FtpException e) {
			JOptionPane.showMessageDialog(this, e.getMessage(), "Error",
					JOptionPane.ERROR_MESSAGE);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void changeDirectory(int row) {
		try {
			FtpService ftpService = new FtpService();
			FTPClient ftpClient = ftpService.getFtpClient();
			try {
				ftpClient.disconnect();
			} catch (Exception e3) {
				// e3.printStackTrace();
			}
			ftpClient = ftpService.reconnect();
			ftpClient.changeWorkingDirectory(remotePath);

			FTPFile[] ftpFiles = ftpClient.listFiles();
			
			if (remotePath.equals("/")){
				remotePath = "";
			}

			if (ftpFiles[row].isDirectory()) {
				remotePath = remotePath + "/" + ftpFiles[row].getName();
				ftpClient.changeWorkingDirectory(remotePath);
				ftpFiles = ftpClient.listFiles();
				Model.changeDirectoryFileTable(ftpFiles);
			}
		}

		catch (FtpException e) {
			JOptionPane.showMessageDialog(this, e.getMessage(), "Error",
					JOptionPane.ERROR_MESSAGE);
		}

		catch (IOException e) {
			e.printStackTrace();
		}
	}

	public String getActualPath() {
		return remotePath;
	}

	public Vector getSelectedFiles() {
		return selectedFiles;
	}

	public void setParentDirectoryTable() {
		try {
			FtpService ftpService = new FtpService();
			FTPClient ftpClient = ftpService.getFtpClient();
			try {
				ftpClient.disconnect();
			} catch (Exception e) {
				// e.printStackTrace();
			}
			ftpClient = ftpService.reconnect();

			StringTokenizer stringTokenizer = new StringTokenizer(remotePath,
					"/");
			String stg = "";
			int count = stringTokenizer.countTokens();

			if (count == 0||count ==1) {
				stg = "/";
			} else {
				for (int i = 1; i <= count - 1; i++) {
					stg = stg + "/" + stringTokenizer.nextToken();
				}
			}

			remotePath = stg;
			ftpClient.changeWorkingDirectory(remotePath);
			FTPFile[] ftpFiles = ftpClient.listFiles();
			Model.changeFileTable(ftpFiles);

		} catch (FtpException e) {
			JOptionPane.showMessageDialog(this, e.getMessage(), "Error",
					JOptionPane.ERROR_MESSAGE);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public void changeLanguage(LanguageData l) {
		Model.changeLanguage(l);
	}

	public String getRemotePath() {
		return remotePath;
	}
}
