/**
 * Title:        JFileManager<p>
 * Description:  <p>
 * Copyright:    Copyright (c) Thorsten Jens, Kai Benjamins<p>
 * Company:      <p>
 * @author Thorsten Jens, Kai Benjamins
 * @version 1.0
 */
package com.lea.ui.ftp.filemanager;

import javax.swing.table.*;
import javax.swing.ImageIcon;
import java.io.*;
import java.util.Vector;

/**
 * Diese Klasse verwaltet die Daten, die in der FileTable dargestellt werden.
 *
 * @author Kai Benjamins
 * @author Thorsten Jens
 * @version $Id: FileTableModel.java,v 1.2 2001/02/22 22:02:58 thodi Exp $ 
 */

public class FileTableModel extends AbstractTableModel implements LanguageAware {
    protected File dir;
    protected String[] filenames;
    protected String[] columnNames;
    protected Class[] columnClasses=new Class[]{ImageIcon.class, String.class, Long.class};

    DetailView dv;

    /**
    * Der Konstruktor erhlt als Parameter das Home-Verzeichnis des Benutzers.
    *
    */
    public FileTableModel(File dir, DetailView dv, LanguageData ld) {
        this.dir=dir;
//        LanguageData ld = new LanguageData();
        this.columnNames = new String[]{"", "Name", "Size"};
        this.filenames=dir.list();
        this.dv = dv;
        this.fireTableDataChanged();
       
    }

    /**
     * Ein anderes Verzeichnis anzeigen
     */
    public void changeFileTable(File dir) {
        this.dir=dir;
        if(dir.exists()) {
            this.filenames=dir.list();
            this.fireTableDataChanged();
        }
    }

    /**
     * Liefert die ausgewhlte Datei
     * @return die ausgewhlte Datei
     */
    public File getSelectedFile(int row) {
        File f = new File(dir,filenames[row]);
        return f;
    }

    /**
     * Anzeigen der Dateiinformationen
     */
    public void showFileOrDirectoryInfo(int row) {
        File f = new File(dir,filenames[row]);
        dv.showInfo(f);
    }

    /**
     * In das, durch Doppelklick, angewhlte Verzeichnis wechseln
     */
    public void changeDirectoryFileTable(int row) {
        File f= new File(dir,filenames[row]);
        if(f.exists() && f.isDirectory() && f.canRead()) {
            this.dir=f;
            this.filenames=dir.list();
        }
        this.fireTableDataChanged();
    }

    /**
     * Liefert die Spaltenanzahl
     * @return Spaltenanzahl
     */
    public int getColumnCount() {
        return 3;
    }
    /**
     * Liefert die Zeilenanzahl
     * @return Zeilenanzahl
     */
    public int getRowCount() {
        return filenames.length;
    }
    /**
     * Liefert die Spaltenberschrift
     * @return Spaltenberschrift
     */
    public String getColumnName(int col) {
        return columnNames[col];
    }

    public Class getColumnClass(int col) {
        return columnClasses[col];
    }
    /**
     * Liefert die Daten der ausgewhlten Zelle
     * @return Zellendaten
     */
    public Object getValueAt(int row, int col) {
        File f=new File(dir, filenames[row]);
        switch(col) {
        case 0:
            return ExtendedFile.getIcon(f);
        case 1:
            return filenames[row];
        case 2:
            return new Long(f.length());
        default:
            return null;
        }
    }


  	public void changeLanguage(LanguageData l) {
			columnNames[1] = "Name";
   		columnNames[2] = "Size";
      this.fireTableStructureChanged();
   	}

}
