package com.lea.ui.ftp.filemanager;

import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowEvent;
import java.beans.PropertyChangeEvent;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JProgressBar;
import javax.swing.JSeparator;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.SwingUtilities;
import javax.swing.UnsupportedLookAndFeelException;

import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPFile;

import com.lea.controler.Observateur;
import com.lea.domain.Ftp;
import com.lea.exception.FtpException;
import com.lea.service.FtpService;
import com.lea.ui.Facade;



/**
 * @see FileTable
 * @see DetailView
 * @author Thorsten Jens
 * @author Kai Benjamins
 * @author Major_Shepard for the =GiFr= Team
 */
public class MainWindow extends JFrame implements LanguageAware {

	public static final String JFM_VERSION = "$Id: MainWindow.java,v 1.3 2001/02/22 22:02:58 thodi Exp $";
	boolean firstTime = true;
	LanguageData ld = new LanguageData();
	JPanel contentPane;
	boolean linkDetailViews = false;
	boolean leftTable = false;
	private boolean abort = false;

	BorderLayout borderLayout1 = new BorderLayout();
	JToolBar mainToolBar = new JToolBar();
	JButton searchButton = new JButton();
	JPanel statusBarPanel = new JPanel();
	GridLayout gridLayout1 = new GridLayout();
	JLabel leftStatusBar = new JLabel();
	JLabel rightStatusBar = new JLabel();
	JSplitPane mainSplitPane = new JSplitPane();
	JSplitPane leftSplitPane = new JSplitPane();
	JSplitPane rightSplitPane = new JSplitPane();
	DetailView leftDetailView = new DetailView();
	DetailViewFtp rightDetailView = new DetailViewFtp();

	FileTable leftFileTable;
	FileTableFtp rightFileTable;

	JPanel leftHelpPanel = new JPanel(new BorderLayout());
	JPanel rightHelpPanel = new JPanel(new BorderLayout());

	JToolBar leftToolBar = new JToolBar();
	JToolBar rightToolBar = new JToolBar();

	JButton leftBackButton = new JButton();
	JButton leftDelButton = new JButton();
	JButton leftHomeButton = new JButton();
	JButton leftRenameButton = new JButton();
	JButton leftMakeDirectoryButton = new JButton();
	JButton leftTransfertButton = new JButton();
	JButton rightBackButton = new JButton();
	JButton rightDelButton = new JButton();
	JButton rightHomeButton = new JButton();
	JButton rightRenameButton = new JButton();
	JButton rightMakeDirectoryButton = new JButton();
	JButton rightTransfertButton = new JButton();
	JProgressBar leftProgressBar = new JProgressBar();
	JProgressBar rightProgressBar = new JProgressBar();
	JTextField leftTextField = new JTextField("    Local");
	JTextField rightTextField = new JTextField("    Remote");

	JComboBox leftComboBox = new JComboBox();
	JComboBox rightComboBox = new JComboBox();
	JButton infoButton = new JButton();
	JButton exitButton = new JButton();
	JButton prefsButton = new JButton();
	JToggleButton linkSizesButton = new JToggleButton();

	JPopupMenu popup = new JPopupMenu();
	// JMenuItem popupItem;

	private JMenuBar menuBar;
	private JMenu menuFile;
	private JMenuItem menuItemRefresh, menuItemQuit, menuItemAbort;
	
	private UploadFile uploadFile;
	private DownloadFile downloadFile;

	// Construct the frame
	public MainWindow(String localPath, String remotePath) {

		Image icone = Toolkit
				.getDefaultToolkit()
				.getImage(
						java.lang.ClassLoader
								.getSystemResource("resources/pictures/system/lea.png"));

		this.setIconImage(icone);

		FtpService ftpService = new FtpService();
		Ftp ftp = ftpService.getFtp();

		this.setTitle("LEA FTP Client - connected on " + "ftp://"
				+ ftp.getLogin() + "@" + ftp.getUrl() + ":" + ftp.getPort());

		this.leftFileTable = new FileTable(leftDetailView, ld, localPath);
		this.rightFileTable = new FileTableFtp(rightDetailView, ld, remotePath);

		enableEvents(AWTEvent.WINDOW_EVENT_MASK);
		try {
			jbInit();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	// Component initialization
	private void jbInit() throws Exception {
		rightStatusBar.setText("");
		leftStatusBar.setText("");

		// ...fr die linke JTable
		leftFileTable.table.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				showPopup(e);
			}

			public void mouseReleased(MouseEvent e) {
				showPopup(e);
			}

			private void showPopup(MouseEvent e) {
				if (e.isPopupTrigger()) {
					leftFileTable.table.clearSelection();
					leftFileTable.table.addRowSelectionInterval(
							leftFileTable.table.rowAtPoint(e.getPoint()),
							leftFileTable.table.rowAtPoint(e.getPoint()));
					leftTable = true;
					popup.show(e.getComponent(), e.getX(), e.getY());
				}
			}
		});
		rightFileTable.table.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) {
				showPopup(e);
			}

			public void mouseReleased(MouseEvent e) {
				showPopup(e);
			}

			private void showPopup(MouseEvent e) {
				if (e.isPopupTrigger()) {
					rightFileTable.table.clearSelection();
					rightFileTable.table.addRowSelectionInterval(
							rightFileTable.table.rowAtPoint(e.getPoint()),
							leftFileTable.table.rowAtPoint(e.getPoint()));
					leftTable = false;
					popup.show(e.getComponent(), e.getX(), e.getY());
				}
			}
		});

		contentPane = (JPanel) this.getContentPane();
		contentPane.setLayout(borderLayout1);
		this.setSize(800, 600);

		leftTextField.setEditable(false);
		leftTextField.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));
		rightTextField.setEditable(false);
		rightTextField.setBorder(BorderFactory.createEmptyBorder(0, 0, 0, 0));

		statusBarPanel.setLayout(gridLayout1);
		gridLayout1.setColumns(2);
		mainSplitPane.setLeftComponent(leftSplitPane);
		mainSplitPane.setOneTouchExpandable(true);
		mainSplitPane
				.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
					public void propertyChange(PropertyChangeEvent e) {
						mainSplitPane_propertyChange(e);
					}
				});

		leftSplitPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
		leftSplitPane.setOneTouchExpandable(false);
		leftSplitPane.setResizeWeight(1);
		leftSplitPane
				.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
					public void propertyChange(PropertyChangeEvent e) {
						leftSplitPane_propertyChange(e);
					}
				});
		rightSplitPane.setOrientation(JSplitPane.VERTICAL_SPLIT);
		rightSplitPane.setOneTouchExpandable(false);
		rightSplitPane.setResizeWeight(1);
		rightSplitPane
				.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
					public void propertyChange(PropertyChangeEvent e) {
						rightSplitPane_propertyChange(e);
					}
				});
		rightBackButton.setIcon(new ImageIcon(MainWindow.class
				.getResource("back16.gif")));
		rightBackButton.setToolTipText(ld.get(ld.L_DIRBACK));
		rightBackButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				rightBackButton_actionPerformed(e);
			}
		});
		rightHomeButton.setIcon(new ImageIcon(MainWindow.class
				.getResource("home16.gif")));
		rightHomeButton.setToolTipText(ld.get(ld.L_HOMEDIR));
		rightHomeButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				rightHomeButton_actionPerformed(e);
			}
		});

		rightDelButton.setIcon(new ImageIcon(MainWindow.class
				.getResource("delete16.gif")));
		rightDelButton.setToolTipText(ld.get(ld.L_DELETE_FILES));
		rightDelButton.addActionListener(new java.awt.event.ActionListener() {

			public void actionPerformed(ActionEvent e) {
				rightDelButton_actionPerformed(e);
			}
		});

		leftDelButton.setIcon(new ImageIcon(MainWindow.class
				.getResource("delete16.gif")));
		leftDelButton.setToolTipText(ld.get(ld.L_DELETE_FILES));
		leftDelButton.addActionListener(new java.awt.event.ActionListener() {

			public void actionPerformed(ActionEvent e) {
				leftDelButton_actionPerformed(e);
			}
		});

		leftBackButton.setIcon(new ImageIcon(MainWindow.class
				.getResource("back16.gif")));
		leftBackButton.setToolTipText(ld.get(ld.L_DIRBACK));
		leftBackButton.addActionListener(new java.awt.event.ActionListener() {

			public void actionPerformed(ActionEvent e) {
				leftBackButton_actionPerformed(e);
			}
		});
		leftHomeButton.setIcon(new ImageIcon(MainWindow.class
				.getResource("home16.gif")));
		leftHomeButton.setToolTipText(ld.get(ld.L_HOMEDIR));
		leftHomeButton.addActionListener(new java.awt.event.ActionListener() {

			public void actionPerformed(ActionEvent e) {
				leftHomeButton_actionPerformed(e);
			}
		});

		leftRenameButton.setIcon(new ImageIcon(MainWindow.class
				.getResource("Move16.gif")));
		leftRenameButton.setToolTipText("Rename file(s)");
		leftRenameButton.addActionListener(new java.awt.event.ActionListener() {

			public void actionPerformed(ActionEvent e) {
				leftRenameButton_actionPerformed(e);
			}
		});

		leftMakeDirectoryButton.setIcon(new ImageIcon(MainWindow.class
				.getResource("folder2.gif")));
		leftMakeDirectoryButton.setToolTipText("Make Directory");
		leftMakeDirectoryButton
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(ActionEvent e) {
						leftMakeDirectoryButton_actionPerformed(e);
					}
				});

		leftTransfertButton.setIcon(new ImageIcon(MainWindow.class
				.getResource("Copy16.gif")));
		leftTransfertButton.setToolTipText("Upload file(s)");
		leftTransfertButton
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(ActionEvent e) {
						leftTransfertButton_actionPerformed(e);
					}
				});

		rightRenameButton.setIcon(new ImageIcon(MainWindow.class
				.getResource("Move16.gif")));
		rightRenameButton.setToolTipText("Rename file(s)");
		rightRenameButton
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(ActionEvent e) {
						rightRenameButton_actionPerformed(e);
					}
				});

		rightMakeDirectoryButton.setIcon(new ImageIcon(MainWindow.class
				.getResource("folder2.gif")));
		rightMakeDirectoryButton.setToolTipText("Make Directory");
		rightMakeDirectoryButton
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(ActionEvent e) {
						rightMakeDirectoryButton_actionPerformed(e);
					}
				});

		rightTransfertButton.setIcon(new ImageIcon(MainWindow.class
				.getResource("Copy16.gif")));
		rightTransfertButton.setToolTipText("Download file(s)");
		rightTransfertButton
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(ActionEvent e) {
						rightTransfertButton_actionPerformed();
					}
				});

		exitButton.setIcon(new ImageIcon(MainWindow.class
				.getResource("stop24.gif")));
		exitButton.addActionListener(new java.awt.event.ActionListener() {

			public void actionPerformed(ActionEvent e) {
				exitButton_actionPerformed(e);
			}
		});
		prefsButton.setIcon(new ImageIcon(MainWindow.class
				.getResource("preferences24.gif")));
		prefsButton.addActionListener(new java.awt.event.ActionListener() {

			public void actionPerformed(ActionEvent e) {
				prefsButton_actionPerformed(e);
			}
		});

		linkSizesButton.addActionListener(new java.awt.event.ActionListener() {

			public void actionPerformed(ActionEvent e) {
				linkSizesButton_actionPerformed(e);
			}
		});
		linkSizesButton.setIcon(new ImageIcon(MainWindow.class
				.getResource("linkSizes.gif")));
		linkSizesButton.setMargin(new Insets(0, 0, 0, 0));
		linkSizesButton.setMaximumSize(new Dimension(31, 31));
		linkSizesButton.setMinimumSize(new Dimension(31, 31));
		contentPane.add(mainToolBar, BorderLayout.NORTH);

		exitButton.setToolTipText(ld.get(ld.L_EXIT));
		infoButton.setToolTipText(ld.get(ld.L_HELP));
		prefsButton.setToolTipText(ld.get(ld.L_PREFERENCES));
		searchButton.setToolTipText(ld.get(ld.L_SEARCH_FOR_FILE));
		linkSizesButton.setToolTipText(ld.get(ld.L_LINK_DETAIL_SIZES));

		menuBar = new JMenuBar();
		menuFile = new JMenu("File");
		menuBar.add(menuFile);
		menuItemAbort = new JMenuItem("Abort");
		menuFile.add(menuItemAbort);
		menuItemRefresh = new JMenuItem("Refresh");
		menuFile.add(menuItemRefresh);
		menuFile.add(new JSeparator());
		menuItemQuit = new JMenuItem("Quit");
		menuFile.add(menuItemQuit);
		setJMenuBar(menuBar);

		contentPane.add(statusBarPanel, BorderLayout.SOUTH);
		statusBarPanel.add(leftStatusBar, null);
		statusBarPanel.add(rightStatusBar, null);

		leftToolBar.add(leftBackButton);
		leftToolBar.add(leftHomeButton);
		leftToolBar.add(leftDelButton);
		leftToolBar.add(leftRenameButton);
		leftToolBar.add(leftMakeDirectoryButton);
		leftToolBar.addSeparator();
		leftToolBar.add(leftTransfertButton);
		leftToolBar.add(leftTextField);
		leftToolBar.add(leftProgressBar);

		rightToolBar.add(rightBackButton);
		rightToolBar.add(rightHomeButton);
		rightToolBar.add(rightDelButton);
		rightToolBar.add(rightRenameButton);
		rightToolBar.add(rightMakeDirectoryButton);
		rightToolBar.addSeparator();
		rightToolBar.add(rightTransfertButton);
		rightToolBar.add(rightTextField);
		rightToolBar.add(rightProgressBar);

		leftProgressBar.setVisible(false);
		rightProgressBar.setVisible(false);

		leftFileTable.table.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				setDetailViewStatusLabel();
			}
		});

		rightFileTable.table.addMouseListener(new MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				setDetailViewStatusLabel();
			}
		});

		contentPane.add(mainSplitPane, BorderLayout.CENTER);
		mainSplitPane.add(leftSplitPane, JSplitPane.LEFT);
		leftHelpPanel.add(leftFileTable, BorderLayout.CENTER);
		leftHelpPanel.add(leftToolBar, BorderLayout.NORTH);
		leftSplitPane.add(leftHelpPanel, JSplitPane.TOP);
		leftSplitPane.add(leftDetailView, JSplitPane.BOTTOM);

		mainSplitPane.add(rightSplitPane, JSplitPane.RIGHT);
		rightHelpPanel.add(rightFileTable, BorderLayout.CENTER);
		rightHelpPanel.add(rightToolBar, BorderLayout.NORTH);
		rightSplitPane.add(rightHelpPanel, JSplitPane.TOP);
		rightSplitPane.add(rightDetailView, JSplitPane.BOTTOM);

		mainSplitPane.setDividerLocation((this.getWidth() / 2) - 2
				* (mainSplitPane.getDividerSize()));

		rightSplitPane
				.setDividerLocation(mainSplitPane.getPreferredSize().height
						- rightDetailView.getPreferredHeight());

		leftSplitPane
				.setDividerLocation(mainSplitPane.getPreferredSize().height
						- leftDetailView.getPreferredHeight());

		firstTime = false;

		/* Load English Language */
		ld.load("English");
		changeLanguage(ld);

		menuItemAbort.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				itemAbortSelectionPerformed();
			}
		});

		menuItemRefresh.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				itemRefreshSelectionPerformed();
			}
		});

		menuItemQuit.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				itemQuitSelectionPerformed();
			}
		});
	}

	private void itemQuitSelectionPerformed() {
		FtpService ftpService = new FtpService();
		ftpService.disconnect();
		dispose();
	}
	
	private void itemAbortSelectionPerformed() {
		FtpService ftpService = new FtpService();
		FTPClient ftpClient = ftpService.getFtpClient();
		try {
				uploadFile.interrupt();
				downloadFile.interrupt();
				abort = ftpClient.abort();
				JOptionPane.showMessageDialog(this, "Transfert aborted.",
						"Abort", JOptionPane.WARNING_MESSAGE);
				rightProgressBar.setValue(0);
				rightProgressBar.setVisible(false);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void itemRefreshSelectionPerformed() {
		try {
			FtpService ftpService = new FtpService();
			File file = new File(leftFileTable.getActualPath());
			leftFileTable.Model.changeFileTable(file);
			FTPClient ftpClient = ftpService.getFtpClient();
			try {
				ftpClient.disconnect();
			} catch (Exception e3) {
				// e3.printStackTrace();
			}
			ftpClient = ftpService.reconnect();
			ftpClient.changeWorkingDirectory(rightFileTable.getRemotePath());
			FTPFile[] ftpFiles = ftpService.getFtpClient().listFiles();
			rightFileTable.Model.changeFileTable(ftpFiles);
		} catch (IOException e) {
			e.printStackTrace();
		} catch (FtpException e) {
			e.printStackTrace();
		}
	}

	// File | Exit action performed
	public void exitButton_actionPerformed(ActionEvent e) {
		this.dispose();
		FtpService ftpService = new FtpService();
		ftpService.disconnect();
	}

	/*
	 * //Help | About action performed public void
	 * helpAbout_actionPerformed(ActionEvent e) {}
	 */

	protected void processWindowEvent(WindowEvent e) {
		super.processWindowEvent(e);
		if (e.getID() == WindowEvent.WINDOW_CLOSING) {
			exitButton_actionPerformed(null);
		}
	}

	private void leftBackButton_actionPerformed(ActionEvent e) {
		leftFileTable.setParentDirectoryTable();
		setDetailViewStatusLabel();
	}

	private void rightBackButton_actionPerformed(ActionEvent e) {
		rightFileTable.setParentDirectoryTable();
		setDetailViewStatusLabel();
	}

	private void leftHomeButton_actionPerformed(ActionEvent e) {
		leftFileTable.setHomeTable();
		setDetailViewStatusLabel();
	}

	private void rightHomeButton_actionPerformed(ActionEvent e) {
		rightFileTable.setHomeTable();
		setDetailViewStatusLabel();
	}

	private void leftDelButton_actionPerformed(ActionEvent e) {
		if (leftFileTable.getSelectedFiles().size() > 0) {
			DeleteFileWindow delete = new DeleteFileWindow(leftFileTable
					.getSelectedFiles(), leftFileTable, ld, this);
		}
	}

	private void rightDelButton_actionPerformed(ActionEvent e) {
		if (rightFileTable.getSelectedFiles().size() > 0) {
			DeleteFileWindowFtp delete = new DeleteFileWindowFtp(rightFileTable
					.getSelectedFiles(), rightFileTable, ld, this);
		}
	}

	private void setDetailViewStatusLabel() {
		leftDetailView.setStatusLabel(leftFileTable.getActualPath());
		rightDetailView.setStatusLabel(rightFileTable.getActualPath());
	}

	private void leftRenameButton_actionPerformed(ActionEvent e) {
		if (leftFileTable.getSelectedFiles().size() == 1) {
			RenameFileWindow rename = new RenameFileWindow(leftFileTable
					.getSelectedFiles(), leftFileTable.getActualPath(),
					leftFileTable, this);
		}
	}

	private void rightRenameButton_actionPerformed(ActionEvent e) {
		if (rightFileTable.getSelectedFiles().size() == 1) {
			RenameFileWindowFtp rename = new RenameFileWindowFtp(rightFileTable
					.getSelectedFiles(), rightFileTable.getActualPath(),
					rightFileTable, this);
		}
	}

	private void leftMakeDirectoryButton_actionPerformed(ActionEvent e) {
		MakeDirectoryWindow directory = new MakeDirectoryWindow(leftFileTable
				.getSelectedFiles(), leftFileTable, leftFileTable
				.getActualPath(), this);
	}

	private void rightMakeDirectoryButton_actionPerformed(ActionEvent e) {
		MakeDirectoryWindowFtp directory = new MakeDirectoryWindowFtp(
				rightFileTable.getSelectedFiles(), rightFileTable,
				rightFileTable.getActualPath(), this);
	}

	private void leftTransfertButton_actionPerformed(ActionEvent e) {
		uploadFile = new UploadFile(this,leftFileTable,rightFileTable,leftProgressBar);
		uploadFile.start();
	}

	private void rightTransfertButton_actionPerformed() {
		downloadFile = new DownloadFile(this,leftFileTable,rightFileTable,rightProgressBar);
		downloadFile.start();
	}

	void contentPane_componentResized(ComponentEvent e) {
		if (!firstTime) {
			rightSplitPane
					.setDividerLocation(mainSplitPane.getPreferredSize().height
							- rightDetailView.getPreferredHeight());
			leftSplitPane
					.setDividerLocation(mainSplitPane.getPreferredSize().height
							- rightDetailView.getPreferredHeight());
		}
	}

	void prefsButton_actionPerformed(ActionEvent e) {
		Object o;
		String newLanguage;
		String[] availableLanguages = ld.getAvailableLanguages();
		// int numLangs = availableLanguages.length;

		o = JOptionPane.showInputDialog(this, ld.get(ld.L_SELECT_LANGUAGE), ld
				.get(ld.L_OPTIONS), JOptionPane.QUESTION_MESSAGE, null,
				availableLanguages, availableLanguages[0]);
		if (o != null) {
			newLanguage = (String) o;
			if (ld.load(newLanguage) != ld.LOAD_FILE_NOT_FOUND) {
				changeLanguage(ld);
			} else {
				JOptionPane.showMessageDialog(this, ld.get(ld.L_FILE_ERROR)
						+ " \"" + newLanguage + ".langdata\"", ld
						.get(ld.L_ERROR), JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	public void changeLanguage(LanguageData l) {
		// DetailViews
		leftDetailView.changeLanguage(l);
		rightDetailView.changeLanguage(l);

		// popupItem.setText(ld.get(ld.L_OPEN_WITH));
		// Toolbar
		exitButton.setToolTipText(l.get(l.L_EXIT));
		infoButton.setToolTipText(l.get(l.L_HELP));
		prefsButton.setToolTipText(l.get(l.L_PREFERENCES));
		searchButton.setToolTipText(l.get(l.L_SEARCH_FOR_FILE));
		linkSizesButton.setToolTipText(l.get(l.L_LINK_DETAIL_SIZES));
		leftRenameButton.setToolTipText("Rename file(s)");
		rightRenameButton.setToolTipText("Rename file(s)");
		leftDelButton.setToolTipText(ld.get(ld.L_DELETE_FILES));
		rightDelButton.setToolTipText(ld.get(ld.L_DELETE_FILES));
		leftHomeButton.setToolTipText(ld.get(ld.L_HOMEDIR));
		rightHomeButton.setToolTipText(ld.get(ld.L_HOMEDIR));
		leftBackButton.setToolTipText(ld.get(ld.L_DIRBACK));
		rightBackButton.setToolTipText(ld.get(ld.L_DIRBACK));

	}

	private void mainSplitPane_propertyChange(PropertyChangeEvent e) {
		// mainSplitPane.setDividerLocation((this.getWidth() / 2) - 2
		// * (mainSplitPane.getDividerSize()));
		// mainSplitPane.setDividerLocation((this.getWidth() / 2));
	}

	void leftSplitPane_propertyChange(PropertyChangeEvent e) {
		if (linkDetailViews) {
			rightSplitPane.setDividerLocation(leftSplitPane
					.getDividerLocation());
		}
	}

	void rightSplitPane_propertyChange(PropertyChangeEvent e) {
		if (linkDetailViews) {
			leftSplitPane.setDividerLocation(rightSplitPane
					.getDividerLocation());
		}
	}

	void linkSizesButton_actionPerformed(ActionEvent e) {
		linkDetailViews = linkSizesButton.isSelected();
	}

	void popupItem_actionPerformed(ActionEvent e) {
		if (leftTable) {
			if (leftFileTable.getSelectedFiles().size() == 1) {
				OpenWithWindow runProg = new OpenWithWindow(leftFileTable
						.getSelectedFiles(), ld);
			}
		} else {
			if (rightFileTable.getSelectedFiles().size() == 1) {
				OpenWithWindow runProg = new OpenWithWindow(rightFileTable
						.getSelectedFiles(), ld);
			}
		}
	}

	public boolean isAbort() {
		return abort;
	}
}
