package com.lea.ui.mission;

import java.awt.Image;

import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.ButtonGroup;
import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JRadioButton;
import javax.swing.UnsupportedLookAndFeelException;

import com.lea.dto.MagazineDTO;
import com.lea.dto.ProfileDTO;
import com.lea.dto.UnitDTO;
import com.lea.dto.WeaponDTO;
import com.lea.exception.FinderException;
import com.lea.service.MissionService;
import com.lea.service.ProfileService;
import com.lea.ui.Facade;



/**
 * Profile apply panel dialog.
 * 
 */

public class ProfileApplyPanel extends JDialog {

	private JComboBox comboBoxSelectProfile;
	private JButton buttonCancel;
	private JButton buttonApply;
	private Facade facade;
	private JRadioButton radioButtonUnit;
	private JRadioButton radioButtonGroup;
	private JRadioButton radioButtonSide;
	private String profileName = "Empty";

	public ProfileApplyPanel(Facade facade) {

		this.facade = facade;
		this.setResizable(false);

		this.setSize(309, 117);
		this.setLayout(null);

		this.setTitle("Apply Profile to");
		Image icone = Toolkit
				.getDefaultToolkit()
				.getImage(
						java.lang.ClassLoader
								.getSystemResource("resources/pictures/system/lea.png"));
		setIconImage(icone);

		this.setLocation((int) facade.getMissionEditor().getLocation().getX()
				+ facade.getMissionEditor().getWidth() / 2 - this.getWidth()
				/ 2, (int) facade.getMissionEditor().getLocation().getY()
				+ facade.getMissionEditor().getHeight() / 2 - this.getHeight()
				/ 2);

		comboBoxSelectProfile = new JComboBox();
		this.add(comboBoxSelectProfile);
		comboBoxSelectProfile.setBounds(15, 45, 203, 23);

		{
			buttonApply = new JButton();
			this.add(buttonApply);
			buttonApply.setText("Apply");
			buttonApply.setBounds(228, 11, 65, 23);
		}
		{
			buttonCancel = new JButton();
			this.add(buttonCancel);
			buttonCancel.setText("Cancel");
			buttonCancel.setBounds(228, 45, 65, 23);
		}
		{
			radioButtonUnit = new JRadioButton();
			getContentPane().add(radioButtonUnit);
			radioButtonUnit.setText("This Unit");
			radioButtonUnit.setBounds(15, 9, 66, 26);
		}
		{
			radioButtonGroup = new JRadioButton();
			getContentPane().add(radioButtonGroup);
			radioButtonGroup.setText("This Group");
			radioButtonGroup.setBounds(83, 10, 77, 25);
		}
		{
			radioButtonSide = new JRadioButton();
			getContentPane().add(radioButtonSide);
			radioButtonSide.setText("This Side");
			radioButtonSide.setBounds(157, 13, 67, 18);
		}

		ButtonGroup buttonsGroupe = new ButtonGroup();
		buttonsGroupe.add(radioButtonUnit);
		buttonsGroupe.add(radioButtonGroup);
		buttonsGroupe.add(radioButtonSide);

		radioButtonUnit.setSelected(true);

		init();

		buttonApply.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String profileName = (String) comboBoxSelectProfile
						.getSelectedItem();
				if (profileName != null) {
					if (!profileName.equals(""))
						applyButtonPerformed(profileName);
				}
			}
		});
		buttonCancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				cancelButtonPerformed();
			}
		});

		// Add Listeners
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				menuExitPerformed();
			}
		});

	}

	private void init() {
		MissionService missionService = new MissionService();

		try {
			UnitDTO unitDTO = missionService.loadUnit(facade.getId());

			ProfileService profileService = new ProfileService();
			Collection<ProfileDTO> profilesDTO;

			profilesDTO = profileService.loadProfiles();

			List<String> list = new ArrayList<String>();
			list.add("");
			for (ProfileDTO profileDTO : profilesDTO) {
				if ((unitDTO.isVehicle() && profileDTO.isVehicle())
						|| (!unitDTO.isVehicle() && !profileDTO.isVehicle())) {
					list.add(profileDTO.getProfileName());
				}
			}

			String[] tabProfiles = new String[list.size()];
			int i = 0;
			for (String stg : list) {
				tabProfiles[i] = stg;
				i++;
			}

			ComboBoxModel model = new DefaultComboBoxModel(tabProfiles);
			comboBoxSelectProfile.setModel(model);
		} catch (Exception e) {
			// JOptionPane.showMessageDialog(this, "Can't load profiles!" + "\n"
			// + e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
		}
	}

	private void cancelButtonPerformed() {
		dispose();
		facade.getMissionEditor().setEnabled(true);
		facade.getMissionEditor().toFront();
	}

	private void applyButtonPerformed(String profileName) {

		this.profileName = profileName;

		ProfileService profileService = new ProfileService();
		MissionService missionService = new MissionService();
		facade.getLoadoutPanel().init();
		facade.getRuckandModsPanel().init();
		facade.getLoadoutPanel().initVisibilityPrimaryMagazinePanels();
		facade.getLoadoutPanel().initVisibilitySecondaryMagazinePanels();
		facade.getMissionEditor().saveUnitLoadout(facade.getId());

		WeaponDTO primaryWeapon = null;
		WeaponDTO secondaryWeapon = null;
		WeaponDTO backWeapon = null;
		WeaponDTO pistol = null;
		Collection<WeaponDTO> opticals = null;
		Collection<WeaponDTO> items = null;
		Collection<MagazineDTO> primaryMagazines = null;
		Collection<MagazineDTO> secondaryMagazines = null;
		Collection<WeaponDTO> ruckWeapons = null;
		Collection<MagazineDTO> ruckMagazines = null;
		Collection<MagazineDTO> firstAid = null;
		double weight = 0.0;

		try {

			int profileId = profileService.findProfile(profileName)
					.getProfileId();

			try {
				primaryWeapon = profileService
						.loadProfilePrimaryWeaponLoadout(profileId);
			} catch (FinderException e) {
			}
			try {
				secondaryWeapon = profileService
						.loadProfileSecondaryWeaponLoadout(profileId);
			} catch (FinderException e) {
			}
			try {
				backWeapon = profileService
						.loadProfileBackWeaponLoadout(profileId);
			} catch (FinderException e) {
			}
			try {
				pistol = profileService.loadProfilePistolLoadout(profileId);
			} catch (FinderException e) {
			}
			try {

				opticals = profileService.loadProfileOpticalLoadout(profileId);
			} catch (FinderException e) {
			}
			try {

				items = profileService.loadProfileItemLoadout(profileId);
			} catch (FinderException e) {
			}
			try {

				primaryMagazines = (List<MagazineDTO>) profileService
						.loadProfilePrimaryMagazineLoadout(profileId);
			} catch (FinderException e) {
			}
			try {

				secondaryMagazines = (List<MagazineDTO>) profileService
						.loadProfileSecondaryMagazineLoadout(profileId);
			} catch (FinderException e) {
			}
			try {

				weight = profileService.loadProfileWeight(profileId);
			} catch (FinderException e) {
			}
			try {

				ruckWeapons = profileService.loadProfileRuckWeapons(profileId);
			} catch (FinderException e) {
			}
			try {
				ruckMagazines = profileService
						.loadProfileRuckMagazines(profileId);
			} catch (FinderException e) {
			}
			try {
				firstAid = profileService
						.loadProfileFirstAidMagazineLoadout(profileId);
			} catch (FinderException e) {
			}

			if (radioButtonUnit.isSelected()) {

				UnitDTO unitDTO = missionService.loadUnit(facade.getId());
				int unitId = facade.getId();

				missionService.saveUnitProfileName(this.profileName, unitId);

				if (!unitDTO.isVehicle() && !facade.isbuttonVehiclePushed()) {
					missionService.saveUnitPrimaryWeaponLoadout(primaryWeapon,
							unitId);
					missionService.saveUnitSecondaryWeaponLoadout(
							secondaryWeapon, unitId);
					missionService
							.saveUnitBackWeaponLoadout(backWeapon, unitId);
					missionService.saveUnitPistolLoadout(pistol, unitId);
					missionService.saveUnitOpticalsLoadout(opticals, unitId);
					missionService.saveUnitItems(items, unitId);
					missionService.saveUnitPrimaryMagazines(primaryMagazines,
							unitId);
					missionService.saveUnitSecondaryMagazines(
							secondaryMagazines, unitId);
					missionService.saveUnitWeight(weight, unitId);
					missionService.saveUnitRuckWeapons(ruckWeapons, unitId);
					missionService.saveUnitRuckMagazines(ruckMagazines, unitId);
					missionService.saveUnitFirstAid(firstAid, unitId);
				} else {
					missionService.saveUnitWeight(weight, unitId);
					missionService.saveUnitRuckWeapons(ruckWeapons, unitId);
					missionService.saveUnitRuckMagazines(ruckMagazines, unitId);
				}

			}

			if (radioButtonGroup.isSelected()) {
				UnitDTO unitDTO = missionService.loadUnit(facade.getId());

				int groupNumber = unitDTO.getGroupNumber();
				Collection<UnitDTO> unitsDTO = missionService
						.loadUnits(groupNumber);

				for (UnitDTO uDTO : unitsDTO) {

					if (!uDTO.getEditorName().equals("not assigned")) {

						int unitId = uDTO.getId();
						missionService.saveUnitProfileName(this.profileName,
								unitId);

						if (!uDTO.isVehicle()) {
							missionService.saveUnitPrimaryWeaponLoadout(
									primaryWeapon, unitId);
							missionService.saveUnitSecondaryWeaponLoadout(
									secondaryWeapon, unitId);
							missionService.saveUnitBackWeaponLoadout(
									backWeapon, unitId);
							missionService
									.saveUnitPistolLoadout(pistol, unitId);
							missionService.saveUnitOpticalsLoadout(opticals,
									unitId);
							missionService.saveUnitItems(items, unitId);
							missionService.saveUnitPrimaryMagazines(
									primaryMagazines, unitId);
							missionService.saveUnitSecondaryMagazines(
									secondaryMagazines, unitId);
							missionService.saveUnitWeight(weight, unitId);
							missionService.saveUnitRuckWeapons(ruckWeapons,
									unitId);
							missionService.saveUnitRuckMagazines(ruckMagazines,
									unitId);
							missionService.saveUnitFirstAid(firstAid, unitId);
						} else {
							missionService.saveUnitWeight(weight, unitId);
							missionService.saveUnitRuckWeapons(ruckWeapons,
									unitId);
							missionService.saveUnitRuckMagazines(ruckMagazines,
									unitId);
						}
					}
				}
			}

			if (radioButtonSide.isSelected()) {
				UnitDTO unitDTO = missionService.loadUnit(facade.getId());
				String side = unitDTO.getSide();
				Collection<UnitDTO> unitsDTO = missionService.loadUnits(side);

				for (UnitDTO uDTO : unitsDTO) {

					if (!uDTO.getEditorName().equals("not assigned")) {

						int unitId = uDTO.getId();
						missionService.saveUnitProfileName(this.profileName,
								unitId);
						if (!uDTO.isVehicle()) {
							missionService.saveUnitPrimaryWeaponLoadout(
									primaryWeapon, unitId);
							missionService.saveUnitSecondaryWeaponLoadout(
									secondaryWeapon, unitId);
							missionService.saveUnitBackWeaponLoadout(
									backWeapon, unitId);
							missionService
									.saveUnitPistolLoadout(pistol, unitId);
							missionService.saveUnitOpticalsLoadout(opticals,
									unitId);
							missionService.saveUnitItems(items, unitId);
							missionService.saveUnitPrimaryMagazines(
									primaryMagazines, unitId);
							missionService.saveUnitSecondaryMagazines(
									secondaryMagazines, unitId);
							missionService.saveUnitWeight(weight, unitId);
							missionService.saveUnitRuckWeapons(ruckWeapons,
									unitId);
							missionService.saveUnitRuckMagazines(ruckMagazines,
									unitId);
							missionService.saveUnitFirstAid(firstAid, unitId);
						} else {
							missionService.saveUnitWeight(weight, unitId);
							missionService.saveUnitRuckWeapons(ruckWeapons,
									unitId);
							missionService.saveUnitRuckMagazines(ruckMagazines,
									unitId);
						}
					}
				}
			}
			missionService.applyMissionListItemsPreferences();
			missionService.applyMissionListOpticalsPreferences();
			missionService.applyMissionListEquipPreferences();
			facade.getMissionEditor().loadUnitLoadout();
			facade.getMissionEditor().setLabelProfileName(this.profileName);

		} catch (FinderException e) {
			JOptionPane.showMessageDialog(this, "Can't apply profiles!" + "\n"
					+ e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);
			facade.getMissionEditor().setLabelProfileName("none");

		} finally {
			facade.getMissionEditor().setEnabled(true);
			dispose();
			facade.getMissionEditor().toFront();
		}
	}

	private void menuExitPerformed() {
		facade.getMissionEditor().setEnabled(true);
		facade.getMissionEditor().toFront();
	}

}
