package com.lea.ui.mission;

import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.UnsupportedLookAndFeelException;

import com.lea.dto.ProfileDTO;
import com.lea.dto.UnitDTO;
import com.lea.exception.FinderException;
import com.lea.exception.LoadingException;
import com.lea.exception.WritingException;
import com.lea.service.MissionService;
import com.lea.service.ProfileService;
import com.lea.ui.Facade;



/**
 * Profil save panel dialog.
 *
 */

public class ProfileSavePanel extends JDialog {
	
	private JTextField textFieldProfileName;
	private JButton buttonCancel;
	private JButton buttonSave;
	private JComboBox comboBoxSelectProfile;
	private Facade facade;

	public ProfileSavePanel(Facade facade) {

		this.facade = facade;
		setLocationRelativeTo(facade.getMissionEditor());
		this.setResizable(false);
		this.setSize(309, 117);
		this.setLayout(null);
		
		this.setTitle("Save as Profile");
		Image icone = Toolkit
				.getDefaultToolkit()
				.getImage(
						java.lang.ClassLoader
								.getSystemResource("resources/pictures/system/lea.png"));
		setIconImage(icone);
		
		this.setLocation((int) facade.getMissionEditor().getLocation().getX()
				+ facade.getMissionEditor().getWidth() / 2 - this.getWidth()
				/ 2, (int) facade.getMissionEditor().getLocation().getY()
				+ facade.getMissionEditor().getHeight() / 2 - this.getHeight()
				/ 2);

		{
			textFieldProfileName = new JTextField();
			this.add(textFieldProfileName);
			textFieldProfileName.setBounds(15, 11, 203, 23);
			String profileName = facade.getMissionEditor().getLabelProfileName();
			if (!profileName.equals("none"))
				textFieldProfileName.setText(profileName);
		}
		{
			comboBoxSelectProfile = new JComboBox();
			this.add(comboBoxSelectProfile);
			comboBoxSelectProfile.setBounds(15, 45, 203, 23);
		}
		{
			buttonSave = new JButton();
			this.add(buttonSave);
			buttonSave.setText("Save");
			buttonSave.setBounds(228, 11, 65, 23);
		}
		{
			buttonCancel = new JButton();
			this.add(buttonCancel);
			buttonCancel.setText("Cancel");
			buttonCancel.setBounds(228, 45, 65, 23);
		}
		
		init();
		
		buttonSave.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!textFieldProfileName.getText().equals(""))
					saveButtonPerformed();
			}
		});
		buttonCancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				cancelButtonPerformed();
			}
		});
		comboBoxSelectProfile.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				String profileName = (String) comboBoxSelectProfile.getSelectedItem();
				textFieldProfileName.setText(profileName);
				if (profileName!=null) {
					if (profileName.equals("")) 
						textFieldProfileName.setText(profileName);
				}
			}
		});
		// Add Listeners
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				menuExitPerformed();
			}
		});
		
	}

	private void init() {
		ProfileService profileService = new ProfileService();
		Collection<ProfileDTO> profilesDTO;
		MissionService missionService = new MissionService();
		try {
			UnitDTO unitDTO = missionService.loadUnit(facade.getId());
			
			profilesDTO = profileService.loadProfiles();
			
			List<String> list = new ArrayList<String>();
			list.add("");
			for (ProfileDTO profileDTO : profilesDTO) {
				if ((unitDTO.isVehicle() && profileDTO.isVehicle())||(!unitDTO.isVehicle() && !profileDTO.isVehicle())) {
					list.add(profileDTO.getProfileName());
				}
			}
			
			String[] tabProfiles = new String[list.size()];
			int i =0;
			for (String stg: list){
				tabProfiles[i]=stg;
				i++;
			}
			
			ComboBoxModel model = new DefaultComboBoxModel(tabProfiles);
			comboBoxSelectProfile.setModel(model);
		} catch(FinderException e1){
		} 
	}

	private void saveButtonPerformed() {
		
			try {
				facade.getMissionEditor().saveUnitLoadout(facade.getId());
				ProfileService profileService = new ProfileService();
				profileService.saveUnitAsProfile(facade.getId(),textFieldProfileName.getText());
				facade.getMissionEditor().setLabelProfileName(textFieldProfileName.getText());
				MissionService missionService = new MissionService();
				missionService.saveUnitProfileName(textFieldProfileName.getText(), facade.getId());
				dispose();
			} catch (WritingException e) {
				JOptionPane.showMessageDialog(this, e.getMessage(), "Error",
						JOptionPane.ERROR_MESSAGE);
			} finally {
				facade.getMissionEditor().setEnabled(true);
				facade.getMissionEditor().toFront();
			}
	}

	private void cancelButtonPerformed() {
		dispose();
		facade.getMissionEditor().setEnabled(true);
		facade.getMissionEditor().toFront();
	}
	
	private void menuExitPerformed() {
		facade.getMissionEditor().setEnabled(true);
		facade.getMissionEditor().toFront();
	}
}
