package com.lea.ui.profile;

import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import javax.swing.ComboBoxModel;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JOptionPane;

import com.lea.dto.MagazineDTO;
import com.lea.dto.ProfileDTO;
import com.lea.dto.WeaponDTO;
import com.lea.exception.FinderException;
import com.lea.service.ProfileService;
import com.lea.ui.Facade;



/**
 * Apply profile dialog
 * 
 */

public class ProfilesApplyProfilePanel extends JDialog {

	private JComboBox comboBoxSelectProfile;
	private JButton buttonCancel;
	private JButton buttonApply;
	private String profileName = "Empty";
	private Facade profilesFacade;
	private List<ProfileDTO> listProfileDTOUnits, listProfileDTOVehicles;

	public ProfilesApplyProfilePanel(Facade profilesFacade) {
		super(profilesFacade.getProfilesEditor(),"Apply Profile", true);

		this.profilesFacade = profilesFacade;
		this.setResizable(false);
		this.setSize(240, 110);
		this.setLayout(null);

		Image icone = Toolkit
				.getDefaultToolkit()
				.getImage(
						java.lang.ClassLoader
								.getSystemResource("resources/pictures/system/lea.png"));
		setIconImage(icone);

		this.setLocation((int) profilesFacade.getProfilesEditor().getLocation()
				.getX()
				+ profilesFacade.getProfilesEditor().getWidth()
				/ 2
				- this.getWidth() / 2, (int) profilesFacade.getProfilesEditor()
				.getLocation().getY()
				+ profilesFacade.getProfilesEditor().getHeight()
				/ 2
				- this.getHeight() / 2);

		comboBoxSelectProfile = new JComboBox();
		this.add(comboBoxSelectProfile);
		comboBoxSelectProfile.setBounds(15, 11, 203, 23);

		{
			buttonApply = new JButton();
			this.add(buttonApply);
			buttonApply.setText("Apply");
			buttonApply.setBounds(25, 45, 65, 23);
		}
		{
			buttonCancel = new JButton();
			this.add(buttonCancel);
			buttonCancel.setText("Cancel");
			buttonCancel.setBounds(100, 45, 65, 23);
		}
		buttonApply.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String profileName = (String) comboBoxSelectProfile
						.getSelectedItem();
				if (profileName != null) {
					if (!profileName.equals(""))
						applyButtonPerformed(profileName);
				}
			}
		});
		buttonCancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent evt) {
				menuExitPerformed();
			}
		});

		// Add Listeners
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				menuExitPerformed();
			}
		});
		init();
	}

	private void init() {

		try {
			ProfileService profileService = new ProfileService();
			Collection<ProfileDTO> profilesDTO = profileService.loadProfiles();
			List<ProfileDTO> list = new ArrayList<ProfileDTO>();
			for (ProfileDTO profileDTO : profilesDTO) {
				if (profilesFacade.isVehicle() == profileDTO.isVehicle()) {
					list.add(profileDTO);
				}
			}
			String[] tabProfiles = new String[list.size()];
			int i = 0;
			for (ProfileDTO profileDTO : list) {
				tabProfiles[i] = profileDTO.getProfileName();
				i++;
			}
			ComboBoxModel model = new DefaultComboBoxModel(tabProfiles);
			comboBoxSelectProfile.setModel(model);
			profilesFacade.getProfilesEditor().getStatus().setText("");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void applyButtonPerformed(String profileName) {

		this.profileName = profileName;

		ProfileService profileService = new ProfileService();

		WeaponDTO primaryWeapon = null;
		WeaponDTO secondaryWeapon = null;
		WeaponDTO backWeapon = null;
		WeaponDTO pistol = null;
		Collection<WeaponDTO> opticals = null;
		Collection<WeaponDTO> items = null;
		Collection<MagazineDTO> primaryMagazines = null;
		Collection<MagazineDTO> secondaryMagazines = null;
		Collection<WeaponDTO> ruckWeapons = null;
		Collection<MagazineDTO> ruckMagazines = null;
		double weight = 0.0;

		try {
			ProfileDTO profileDTO = profileService.findProfile(profileName);
			int profileId = profileDTO.getProfileId();

			try {
				primaryWeapon = profileService
						.loadProfilePrimaryWeaponLoadout(profileId);
			} catch (FinderException e) {
			}
			try {
				secondaryWeapon = profileService
						.loadProfileSecondaryWeaponLoadout(profileId);
			} catch (FinderException e) {
			}
			try {
				backWeapon = profileService
						.loadProfileBackWeaponLoadout(profileId);
			} catch (FinderException e) {
			}
			try {
				pistol = profileService.loadProfilePistolLoadout(profileId);
			} catch (FinderException e) {
			}
			try {

				opticals = profileService.loadProfileOpticalLoadout(profileId);
			} catch (FinderException e) {
			}
			try {

				items = profileService.loadProfileItemLoadout(profileId);
			} catch (FinderException e) {
			}
			try {

				primaryMagazines = (List<MagazineDTO>) profileService
						.loadProfilePrimaryMagazineLoadout(profileId);
			} catch (FinderException e) {
			}
			try {

				secondaryMagazines = (List<MagazineDTO>) profileService
						.loadProfileSecondaryMagazineLoadout(profileId);
			} catch (FinderException e) {
			}
			try {

				weight = profileService.loadProfileWeight(profileId);
			} catch (FinderException e) {
			}
			try {

				ruckWeapons = profileService.loadProfileRuckWeapons(profileId);
			} catch (FinderException e) {
			}
			try {

				ruckMagazines = profileService
						.loadProfileRuckMagazines(profileId);
			} catch (FinderException e) {
			}

			if (!profileDTO.isVehicle()) {
				profileService.saveProfilePrimaryWeaponLoadout(primaryWeapon,
						profilesFacade.getId());
				profileService.saveProfileSecondaryWeaponLoadout(
						secondaryWeapon, profilesFacade.getId());
				profileService.saveProfileBackWeaponLoadout(backWeapon,
						profilesFacade.getId());
				profileService.saveProfilePistolLoadout(pistol, profilesFacade
						.getId());
				profileService.saveProfileOpticalsLoadout(opticals,
						profilesFacade.getId());
				profileService.saveProfileItems(items, profilesFacade.getId());
				profileService.saveProfilePrimaryMagazines(primaryMagazines,
						profilesFacade.getId());
				profileService.saveProfileSecondaryMagazines(
						secondaryMagazines, profilesFacade.getId());
				profileService
						.saveProfileWeight(weight, profilesFacade.getId());
				profileService.saveProfileRuckWeapons(ruckWeapons,
						profilesFacade.getId());
				profileService.saveProfileRuckMagazines(ruckMagazines,
						profilesFacade.getId());
			} else {
				profileService
						.saveProfileWeight(weight, profilesFacade.getId());
				profileService.saveProfileRuckWeapons(ruckWeapons,
						profilesFacade.getId());
				profileService.saveProfileRuckMagazines(ruckMagazines,
						profilesFacade.getId());
			}

			profilesFacade.getProfilesEditor().loadProfileLoadout(profileId);

		} catch (FinderException e) {
			JOptionPane.showMessageDialog(this, "Can't apply profile!" + "\n"
					+ e.getMessage(), "Error", JOptionPane.ERROR_MESSAGE);

		} finally {
			profilesFacade.getProfilesEditor().setEnabled(true);
			dispose();
		}
	}

	private void menuExitPerformed() {
		this.dispose();
	}

}
