package fr.soe.lea.service;

import java.io.File;
import java.io.IOException;

import org.apache.commons.net.ftp.FTP;
import org.apache.commons.net.ftp.FTPClient;
import org.apache.commons.net.ftp.FTPReply;
import org.dom4j.DocumentException;

import fr.soe.lea.dao.CommonDAO;
import fr.soe.lea.dao.FtpDAO;
import fr.soe.lea.dao.XmlDAO;
import fr.soe.lea.exception.FinderException;
import fr.soe.lea.exception.FtpException;
import fr.soe.lea.exception.WritingException;
import fr.soe.lea.exception.XmlException;

public class Service {

	private static FtpDAO ftpDAO = new FtpDAO();
	private static XmlDAO xmlDAO = new XmlDAO();
	private static CommonDAO commonDAO = new CommonDAO();
	private String version = "";
	private String zipFileName = "";
	private FTPClient ftpClient;
	private static final String UPDTATE_REPOSITORY_ADRESS = "www.sonsofexiled.fr";
	private static final int UPDTATE_REPOSITORY_PORT = 21;
	private static final String UPDTATE_REPOSITORY_LOGIN = "anonymous";
	private static final String UPDTATE_REPOSITORY_PASS = "";
	

	public String getVersion() throws XmlException {
		try {
			this.version = xmlDAO.getVersion();
			getZipFileName();
		} catch (DocumentException e) {
			e.printStackTrace();
			throw new XmlException("Can't get update version.");
		}
		return version;
	}

	private String getZipFileName() throws DocumentException {
		this.zipFileName = xmlDAO.getZipFileName();
		return this.zipFileName;
	}

	public long getSize() throws FtpException, FinderException {

		long size = 0;
		try {
			ftpClient = new FTPClient();
			ftpClient.connect(UPDTATE_REPOSITORY_ADRESS,UPDTATE_REPOSITORY_PORT) ;
			ftpClient.login(UPDTATE_REPOSITORY_LOGIN,UPDTATE_REPOSITORY_PASS);
			ftpClient.setFileType(FTP.BINARY_FILE_TYPE);// binary transfert mode
			ftpClient.enterLocalPassiveMode();// passive mode

			int reply = ftpClient.getReplyCode();

			if (FTPReply.isPositiveCompletion(reply)) {
				System.out.println("Connected Success");
			} else {
				System.out.println("Connection Failed");
				ftpClient.disconnect();
				throw new FtpException(
						"Fail to connect to remote repository.");
			}

		} catch (Exception e1) {
			e1.printStackTrace();
			throw new FtpException("Fail to connect to remote repository.");
		}
		
		try {
			size = ftpDAO.getFtpFileSize(zipFileName, ftpClient);
			if (size==0){
				throw new FinderException("Can't find update file on repository.");
			}
		} catch (IOException e) {
			e.printStackTrace();
			throw new FinderException("Can't find update file on repository.");
		} 
		return size;
	}

	public void setDownload() throws WritingException {
		try {
			ftpDAO.setDownload(zipFileName);
		} catch (Exception e) {
			e.printStackTrace();
			throw new WritingException("Can't write files on disk.");
		}
	}

	public void download() throws FtpException {

		try {
			ftpClient.setFileType(FTP.BINARY_FILE_TYPE);// binary transfert mode
			ftpClient.enterLocalPassiveMode();// passive mode
			int reply = ftpClient.getReplyCode();
			if (FTPReply.isPositiveCompletion(reply)) {
				System.out.println("Connected Success");
			} else {
				System.out.println("Connection Failed");
				ftpClient.disconnect();
				throw new FtpException(
						"Fail to connect to remote repository.");
			}
			ftpDAO.download(zipFileName, ftpClient);

		} catch (Exception e) {
			e.printStackTrace();
			throw new FtpException("Fail to download update!");
		}
	}

	public FtpDAO getFtpDAO() {
		return ftpDAO;
	}

	public void installLea() throws WritingException {
		try {
			ftpDAO.installLea();
		} catch (IOException e) {
			throw new WritingException("Can't write files on disk!");
		}
	}

	public void clean() {
		ftpDAO.clean();
	}

	public String checkForLeaAddon() {
		String destinationFolder = "";
		String userconfigPath = commonDAO.getUserconfigLeaPath();
		if (userconfigPath!=null){
			File leaPbo = new File (new File (userconfigPath).getParentFile().getAbsolutePath() + "/@lea/addons/lea.pbo");
			if (leaPbo.exists()){
				File  leaAddonFolder = new File (new File (userconfigPath).getParentFile().getAbsolutePath() + "/@lea");
				destinationFolder =  leaAddonFolder.getAbsolutePath();
			}
		}
		return destinationFolder;
	}

	public void updateLeaAddon(String leaAddonFolderPath) throws WritingException {
		try {
			ftpDAO.installLeaAddon(leaAddonFolderPath);
		} catch (IOException e) {
			throw new WritingException("Can't write files on disk!");
		}	
	}
}
